#!/bin/csh
#
# FUNCTION: dbtrans.csh
#
# PURPOSE:  Cat all of the appropriate data base records into big files
#	    and transfer them to the data base manager host system.
#	    Update the GLIS data base on the host system using these
#	    concatinated files.  This procedure will run as a cron job
#	    and calls the "transferdb.csh" shell script to do the bulk
#	    of the work.
#
#
#
# Version  Programmer       Date   Modifications
# -------  ---------------  -----  ---------------------------------------------
#   1.0    T. Baltzer       07/94  Initial development
#


#------------------------
# Assign constant values
#------------------------
set DBSYS = "edcsgs12"
set DBUSR = "opsavhrr"


#-------------------------------------------
# Make certain the environment has been set
#-------------------------------------------
if ((! $?LASROOT) && (-r /etc/login)) then
    source /etc/login
    eval `$masterlog response comfile LAS70D`
    unset response
    unset comfile
endif


#----------------------------
# Alias appropriate commands
#----------------------------
alias wrtlog $ADAPSBASE/write_log


#--------------------------------------------------------
# Create a log file.  This must be a LAS name for wrtlog.
#--------------------------------------------------------
set noglob
set logfile = `echo "[$ADAPSLOG]dbtrans" | tr / . | sed  's/\[\./\[/'`


#-------------------------------------
# Check the arguments (should be none)
#-------------------------------------
if ($#argv != 0) then
    wrtlog $logfile "" "dbtrans-parm" "Invalid usage: dbtrans.csh"
    exit 1
endif


#------------------------------
# Unalias appropriate commands
#------------------------------
unalias ping
unalias rm
unalias cat


#--------------------------------------------------------------
# Verify that the data base manager host system is operational
#--------------------------------------------------------------
ping -c 1 $DBSYS >> /dev/null
if ($status) then
    wrtlog $logfile $DBSYS "dbtrans-nohst" "The dbm host is not operational"
    exit 1
else
    wrtlog $logfile $DBSYS "informational" "The dbm host is operational"
endif


#------------------------------------------------------
# See if a lock file exists indicating that another
# process like this one is already running.  If there
# is, terminate; otherwise create a lock file.
#------------------------------------------------------
if (-e $ADAPSTABLES/dbtrans.lock) then
    wrtlog $logfile "" "dbtrans-mult" "DBTRANS procedure already running"
    exit 1
else
    echo "locked" >> $ADAPSTABLES/dbtrans.lock
endif


# Echo the date to standard output just for reference
#-----------------------------------------------------
echo " "
echo " "
date
echo "-----------------------------------------------------------------------"


#--------------------------------------------------
# Transfer the data base records to the dbm host:
# (1) Records to update indicating AVHRR scene has
#     now become a part of a stitched half orbit.
# (2) New AVHRR archive scenes (including orbital
#     stitched images).
#
#--------------------------------------------------
$ADAPSBASE/transferdb.csh $ADAPSDB orbit

$ADAPSBASE/transferdb.csh $ADAPSDB new


#--------------------------------------------------------
# Remove the lock file indicating that this process is no
# longer running.
#--------------------------------------------------------
rm -f $ADAPSTABLES/dbtrans.lock


#-----------------------
# Successful completion
#-----------------------
exit 0
