/*******************************************************************************
NAME:    CAL_LIN_SMP

PURPOSE: Calculate the line and sample of the input image corresponding to
	 a given latitude/longitude when it is in a valid projection.

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "proj.h"

FUNCTION cal_lin_smp(ddr,ddr_units,geo_code,geo_units,geo_zone,geo_parms,
		     lat,lon,line,samp)

struct DDR *ddr;    	 /* DDR information				*/
double	*lat;		 /* given latitude				*/
double	*lon;		 /* given longitude				*/
long   	*line;           /* starting line of input in the output space	*/
long   	*samp;           /* starting samp of input in the output space	*/
long	*geo_code,	 /* code for geographic projection (lat/long)	*/
	*geo_units,	 /* units for geographic projection 		*/
	*geo_zone,	 /* zone code for geographic projection 	*/
	*ddr_units;	 /* projection units from image ddr      	*/

double	geo_parms[];	 /* parms for geographic projection 		*/
{
double	coord1,coord2;	/* coordinates in the given projection		*/

/*
  Convert input lat,lon to projection coordinates
*/
if (c_trans(geo_code,geo_units, &ddr->proj_code,ddr_units,
           lon,lat,&coord2,&coord1) != E_SUCC)
    return(E_FAIL);

/*
  Calculate the starting line and sample 
*/
if (ddr->upleft[1] < ddr->upright[1])
    *samp = (long)((coord2 - ddr->upleft[1]) / ddr->pdist_x + 1.0);
else
    *samp = (long)((ddr->upleft[1] - coord2) / ddr->pdist_x + 1.0);

if (ddr->upleft[0] > ddr->loleft[0])
    *line = (long)((ddr->upleft[0] - coord1) / ddr->pdist_y + 1.0);
else
    *line = (long)((coord1 - ddr->upleft[0]) / ddr->pdist_y + 1.0);

return(E_SUCC);
}
