/*******************************************************************************
NAME:   PROJSEL

PURPOSE:        Select lines from WDBII for placement in the image which is 
		not in satellite projection (use DDR for projection information)

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "las.h"
#include "dcw.h"
#include "adaps.h"
#include "proj.h"
#include "dcw2ras.h"
 
FUNCTION projsel(vblock,ddr,cdpath,library,acount,dcw_coverage,
                pixval,linwt,window,pntint,msgtim,fp_scn,
		minlat,minlon,maxlat,maxlon)

struct  PARBLK  *vblock;        /* TAE parameter block                  */
struct  DDR	*ddr;		/* DDR information			*/
struct  DCW_COVERAGE *dcw_coverage;     /* Coverage areas               */

FILE    *fp_scn;                /* file pointer to scan file            */

char    *cdpath,                /* dcw location                          */ 
	library[][10];          /* list of libraries (areas)             */

long    acount,			/* number of areas to rasterize		*/
        pixval,                 /* pixel value of rasterized line       */
        linwt,                 	/* weight of the line to be processed   */
        window[],               /* window to process                    */
        pntint,         	/* point interval                       */
        *msgtim;                /* time between processing messages     */

double  minlat,                 /* minimum latitude			*/
	minlon,                 /* minimum longitude                    */
	maxlat,                 /* maximum latitude                     */
	maxlon;                 /* maximum longitude                    */
{
extern char tfiles[3][CMLEN];	/* output temporary host file names	     */
extern long numout;		/* number of output temp files		     */
extern long curent;		/* current "line" for processimg messages    */

struct DCW_VECTOR dcw_vector;	/* dcw vector (list of line segments)	*/

long	i, 		   	/* looping variables		     	*/
	len,			/* string length			*/
	zero = 0,	   	/* constant of zero		     	*/
	nonfatal = 1,	   	/* non-fatal error code		     	*/
	msgfmt = GEN1,	   	/* general message format one	     	*/
	max_entity = 9999, 	/* maximum entity for processing message*/
	segment,		/* current segment in vector		*/
	status,stat;   	   	/* return status			*/

char	curlib[10],     	/* current library name			*/
	ptext[ERRLEN],     	/* processing message text		*/
	fname = '\0';		/* transinit() file name		*/

long	geo_code = GEO,		/* projection code			*/
	geo_zone = NULZONE,	/* projection zone			*/
	geo_units = DEGREES,	/* projection units			*/
	ddr_units,		/* projection units from image ddr	*/
	prtprm[2];		/* transinit() print parameters		*/

double 	geo_parms[15],		/* projection parameters		*/
	lon1,lon2,		/* min,max longitude  			*/
	lat1,lat2;		/* min,max latitude 			*/

/*
  Set up projection information
*/
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;
status = c_ckunit(ddr->proj_units,&ddr_units);
if (status != E_SUCC)
    {
    c_errmsg("Invalid projection information in DDR","dcw2ras-proj",
			     &nonfatal);
    return(E_FAIL);
    }

/*
  Initialize projection information
*/
prtprm[GEO_TERM] = prtprm[GEO_FILE] = FALSE;
if (c_transinit(&geo_code,&geo_units,&geo_zone,&ddr->datum_code,geo_parms,
	&ddr->proj_code,&ddr_units,&ddr->zone_code,&ddr->datum_code,
	ddr->proj_coef,prtprm,fname) != E_SUCC)
    return(E_FAIL);

/*
  Rasterize each area
*/
for (i = 0; i < acount; i++)
    {
    /*
      Break the areas up so we don't read so many lines from dcw at once
    */
    lon1 = minlon;
    if ((maxlon - minlon) > 15.0)
        lon2 = ((int)(minlon/15) + 1.0 - (minlon<0.0) ) * 15.0;
    else
        lon2 = maxlon;

    /*
      Initailize and start the processing message
    */
    strcpy(curlib,library[i]);
    len = strlen(curlib);
    c_low2up(curlib,&len);
    sprintf(ptext,"for %s",curlib);
    curent = 1;
    c_inittm("DCW2RAS","Rasterized","line"," ",ptext,&max_entity,&zero,msgtim,
	     &msgfmt);
    c_startm();

    while (1)
	{
        lat1 = minlat;
        if ((maxlat - minlat) > 15.0)
            lat2 = ((int)(minlat/15) + 1.0 - (minlat<0.0) ) * 15.0;
        else
            lat2 = maxlat;

        while (1)
	    {
    	    /*
      	      Allocate first line memory block
      	      (if this block is exceeded, it will be reallocated within the
      	      dcw support routines).
    	    */
    	    dcw_vector.numlines = 0;
    	    dcw_vector.maxline = LINEBLOCK;
    	    dcw_vector.line = (struct DCW_LINE *)malloc(LINEBLOCK *
						sizeof(struct DCW_LINE));

    	    if (dcw_vector.line == NULL)
		{
		c_errmsg("Error allocating dynamic memory","dcw2ras-alloc",
	    	   &nonfatal);
		return(E_FAIL);
		}

    	    /*
      	      Perform the dcw query, loading all lines that fit our area
    	    */
    	    status = dcw_query(cdpath,library[i],dcw_coverage,&dcw_vector,
	      lat1,lon1,lat2,lon2);
    	    if (status == E_FAIL)
		{
		c_errmsg("Error occurred querying DCW","dcw2ras-query",
			&nonfatal);
		return(E_FAIL);
		}

    	    /*
      	      Process all lines found
    	    */
    	    for (segment = 0; segment < dcw_vector.numlines; segment++)
		{
		/*
	  	  Process each segment
		*/
		stat = procproj(&dcw_vector.line[segment],ddr,window,pixval,
			 linwt,pntint,fp_scn,&geo_code,&geo_zone,&geo_units,
			 &ddr_units,geo_parms);
		if (stat != E_SUCC)
	    	    {
	    	    fclose(fp_scn);
	    	    c_clean(vblock,"dcw2ras",tfiles,&numout);
	    	    }

		/*
	  	  Increment the segment counter
		*/
		curent++;
		}

            if (lat2 == maxlat) break;
            lat1 = lat2;
            lat2 += 15.0;
            if (lat2 > maxlat) lat2 = maxlat;

            /*
              Free lines
            */
            dcw_freelines(&dcw_vector);
            }

        if (lon2 == maxlon) break;
        lon1 = lon2;
        lon2 += 15.0;
        if (lon2 > maxlon) lon2 = maxlon;
        }

    /*
      Stop the processing message
    */
    curent--;
    c_stoptm();

    }

/*
  Close the scan file
*/
fclose(fp_scn);

return(E_SUCC);
}
