/*******************************************************************************
NAME:   	PROJMINMAX

PURPOSE:        Find the min and max lat and long of an image in a projection

PROJECT:        ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "proj.h"
#include "llrange.h"
 
FUNCTION projminmax(ddr,minlat,minlon,maxlat,maxlon)

struct  DDR	*ddr;		/* DDR information			*/

float	*minlat,		/* minumum latitude in the image     	*/
	*minlon,		/* minumum longitude in the image     	*/
	*maxlat,		/* maximum latitude in the image     	*/
	*maxlon;		/* maximum longitude in the image     	*/
{
long	i, 		   	/* looping variables		     	*/
	crossflg,		/* indicates crossing Internat'l Date Line */
	nonfatal = 1,	   	/* non-fatal error code		     	*/
	status;   	   	/* return status			*/

long	geo_code,		/* projection code			*/
	geo_zone,		/* projection zone			*/
	geo_units,		/* projection units			*/
	geo_datum,		/* projection datum			*/
	ddr_units;		/* projection units from image ddr	*/

double 	geo_parms[15],		/* projection parameters		*/
	lon1,lon2,		/* min,max longitude  			*/
	lat1,lat2;		/* min,max latitude 			*/

/*
  Check validity of the projection code, 
  corner projection coordinates, and projection distance
*/
if (ddr->valid[0] == INVAL)
    {
    c_errmsg("Projection code of image is invalid","llrange-pcode",
                 &nonfatal);
    return(E_FAIL);
    }
if (ddr->valid[DDCCV] == INVAL)
    {
    c_errmsg("Corner projection coordinates of image are invalid",
        "llrange-pcoord",&nonfatal);
    return(E_FAIL);
    }
if (ddr->valid[DDPDV] == INVAL)
    {
    c_errmsg("Projection distances per pixel of image are invalid",
        "llrange-pdist",&nonfatal);
    return(E_FAIL);
    }

/*
  Set up projection information
*/
geo_code = GEO;
geo_zone = NULZONE;
geo_units = DEGREES;
geo_datum = ddr->datum_code;	/* input & output datum must be same 	*/
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;
status = c_ckunit(ddr->proj_units,&ddr_units);
if (status != E_SUCC)
    {
    c_errmsg("Invalid projection unit","llrange-punit", &nonfatal);
    return(E_FAIL);
    }

/*
  Determine the min and max latitude and longitude of the image
*/
status = search_sides(ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms,
                      &lat1,&lon1,&lat2,&lon2);
if (status != E_SUCC)
    return(E_FAIL);

*minlat = (float)lat1;
*maxlat = (float)lat2;
*minlon = (float)lon1;
*maxlon = (float)lon2;

if ((*maxlon - *minlon) > 180.0)
    crossflg = TRUE;
else
    crossflg = FALSE;

return(E_SUCC);
}
