/******************************************************************************
NAME:		GET_NRD_PAR

PURPOSE:	Retrieves NORAD parameter information

ALGORITHM
	retrieve required parameters from the variable block

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include "tae.h"
#include "worgen.h"

#define TERM   0
#define LP     1
#define FIL    2


FUNCTION get_nrd_par(vblock,satid,epyr,epdy,epfd,inclin,rtasc,ecc,argper,
		     meanan,meanm,print_file,out,out_cnt)

struct PARBLK	*vblock;	/* TAE parameter block			*/

char		*satid,		/* Satellite id number			*/
		out[1][CMLEN];  /* List of output files                 */

long		*epyr,		/* Epoch Year				*/
      		*epdy,		/* Epoch Day				*/
     		*out_cnt;       /* Number of output files               */

double		*epfd,		/* Epoch Fractional Day			*/
       		*inclin,	/* Inclination				*/
      		*rtasc,		/* Right Ascension of the Ascending Node*/
      		*ecc,		/* Eccentricity				*/
      		*argper,	/* Argument of Perigee			*/
      		*meanan,	/* Mean Anomaly				*/
      		*meanm;		/* Mean Motion				*/

FILE            **print_file;   /* File descriptor of output file       */
 
{ 
struct VARIABLE *p_find();	/* Function to get a TAE variable	*/ 

struct VARIABLE *var;		/* Pointer to a TAE variable in vblock	*/

FILE   	*fopen();

char	temp[21],		/* Temporary string			*/
        print_filename[CMLEN],  /* output file name              */
	fileutil_file[CMLEN];	/* Temporary file for fileutil         */ 

long	nonfatal = 1,           /* Nonfatal error code                  */
	status,                 /* Return value                         */
	print[3],    	        /* Print options 	 		*/
	zero = 0;		/* Constant value			*/


/*
  Get the satellite name (N09, N10, N11, N12, N14, N15, N16)
*/

var = p_find(vblock, "SATID");
strcpy(satid, SVAL(*var, 0));

/*
  Get the epoch year
*/
var = p_find(vblock, "EPOCHYR");
*epyr = IVAL(*var,0);

/*
  Get the epoch day
*/
var = p_find(vblock, "EPOCHDY");
*epdy = IVAL(*var,0);

/*
  Get the epoch fractional day
*/
var = p_find(vblock, "EPOCHFD");
strcpy(temp,"0.");
strcat(temp, SVAL(*var,0));
sscanf(temp,"%lf",epfd);

/*
  Get the Inclination
*/
var = p_find(vblock, "INCLINE");
sscanf(SVAL(*var,0),"%lf",inclin);

/*
  Get the Right Ascension of the Ascending Node
*/
var = p_find(vblock, "RTASCEN");
sscanf(SVAL(*var,0),"%lf",rtasc);

/*
  Get the Eccentricity
*/
var = p_find(vblock, "ECCENT");
strcpy(temp,"0.");
strcat(temp, SVAL(*var, 0));
sscanf(temp,"%lf",ecc);

/*
  Get the Argument of Perigee
*/
var = p_find(vblock, "ARGPER");
sscanf(SVAL(*var,0),"%lf",argper);

/*
  Get the Mean Anomaly
*/
var = p_find(vblock, "MEANAN");
sscanf(SVAL(*var,0),"%lf",meanan);

/*
  Get the mean motion
*/
var = p_find(vblock, "MEANMO");
sscanf(SVAL(*var,0),"%lf",meanm);

/*
  Get print option and open output file, if necessary 
--------------------------------------------------------
*/
*out_cnt = 0;
strcpy (fileutil_file, "NORAD");
status = c_setprt (vblock, &print[TERM], &print[FIL], &print[LP],
         fileutil_file, print_filename);
if (status != E_SUCC)
	c_clean(vblock,"NORAD",fileutil_file,&zero);
if (print[FIL] == 1)
	{
        strcpy (out[*out_cnt], print_filename);
        *out_cnt += 1;
	}
*print_file = fopen (fileutil_file, "w");
if (*print_file == NULL)
	{
	c_errmsg("Error opening output file","norad-outopn",&nonfatal);
	c_clean(vblock,"NORAD",fileutil_file,&zero);
	}

return;
}
