/************************************************************************
FUNCTION:       INSERT

PURPOSE:	Insert a new pass into the pass list in the proper position

PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

*************************************************************************/
#include "schedcnflt.h"



int insert(

    struct PASS	**ptr,		/* IN: pointer to new pass		*/
    struct PASS	**passlist	/* IN/OUT: pointer to pass list		*/
)
{
long	inserted;		/* flag indicating pass is in list	*/

struct	PASS	*curr;		/* pointer to current pass		*/
struct	PASS	*prev;		/* pointer to previous pass		*/


/* Insert the pass into the linked list in its proper
location.  The list is ordered by date/time of start.
-----------------------------------------------------*/
if ( *passlist == NULL )
    *passlist = *ptr;
else
    {
    curr = *passlist;
    prev = NULL;
    inserted = FALSE;
    do
	{
	/* insert before current */
	if ( (*ptr)->sdate < curr->sdate )
	    {
	    if ( prev == NULL )
		{
		*passlist = *ptr;
		(*ptr)->next = curr;
		}
	    else
		{
		prev->next = *ptr;
		(*ptr)->next = curr;
		}
	    inserted = TRUE;
	    }
	/* insert after current */
	else if ( (*ptr)->sdate > curr->sdate )
	    {
	    if ( curr->next == NULL )
		{
		curr->next = *ptr;
		inserted = TRUE;
		}
	    else
		{
		prev = curr;
		curr = curr->next;
		}
	    }
	/* insert before current */
	else if ( (*ptr)->stime < curr->stime )
	    {
	    if ( prev == NULL )
		{
		*passlist = *ptr;
		(*ptr)->next = curr;
		}
	    else
		{
		prev->next = *ptr;
		(*ptr)->next = curr;
		}
	    inserted = TRUE;
	    }
	/* insert after current */
	else if ( (*ptr)->stime > curr->stime )
	    {
	    if ( curr->next == NULL )
		{
		curr->next = *ptr;
		inserted = TRUE;
		}
	    else
		{
		prev = curr;
		curr = curr->next;
		}
	    }
	/* insert after current */
	else
	    {
	    if ( curr->next == NULL )
		{
		curr->next = *ptr;
		inserted = TRUE;
		}
	    else
		{
		prev = curr;
		curr = curr->next;
		}
	    }
	}
    while ( !inserted );
    }

return E_SUCC;
}
