/************************************************************************
FUNCTION:	SET_LOCAL

PURPOSE:	Adjust integer date/time ( YYYYJJJ and SSSSSS ) from
		UTC to local time.


PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

*************************************************************************/
#include "schedcnflt.h"

int set_local(

    long	*date,		/* IN/OUT: integer date as YYYYJJJ	*/
    long	*ltime		/* IN/OUT: seconds since midnight	*/
)
{
long	i;			/* loop control variable		*/
long	jday;			/* julian day from date parameter	*/
long	temp;			/* temporary variable			*/
long	year;			/* current year				*/

static	long	ch_day;		/* julian day that time zone changes	*/
static	long	ch_flag;	/* flag: change in time zones		*/
static	long	first = TRUE;	/* flag: first call to function		*/
static	long	gmoffset[2];	/* difference(s) from UTC to local time	*/

struct	tm	*lctm;		/* time structure			*/

time_t	t;			/* seconds since beginning of unix time	*/


/* The first time this function is called we need to calculate the
difference between local time and UTC.  Also check ahead for 2 weeks
to see if we change to/from Daylight Savings time and calculate
that time difference as well.
--------------------------------------------------------------------*/
if ( first )
    {
    first = FALSE;
    t = time( NULL );
    lctm = localtime( &t );
    ch_flag = lctm->tm_isdst;
    if ( ch_flag )
	gmoffset[0] = -altzone;
    else
        gmoffset[0] = -timezone;
    for ( i = 0; i < 14; i++ )
        {
	t += SEC_PER_DAY;
	lctm = localtime( & t );
	if ( lctm->tm_isdst != ch_flag )
	    {
	    ch_flag = TRUE;
	    ch_day = lctm->tm_yday;
	    temp = lctm->tm_hour;
	    lctm = gmtime( &t );
	    gmoffset[1] = -(( lctm->tm_hour - temp ) * SEC_PER_HR );
	    break;
	    }
	}
    }

/* Extract the year and julian date from parameter date
-------------------------------------------------------*/
year = *date / 1000;
jday = *date % 1000;

/* Apply the appropriate offset to the time
-------------------------------------------*/
if ( jday == ch_day )
    *ltime += gmoffset[1];
else
    *ltime += gmoffset[0];

/* Correct the time for spanning day/month/year
-----------------------------------------------*/
if ( *ltime <= 0 )
    {
    *ltime += SEC_PER_DAY;
    jday--;
    if ( jday <= 0 )
        {
	year--;
	if ( c_isleap( &year ))
	    jday = 366;
	else
	    jday = 365;
	}
    }

/* Recombine the year and jday into date
----------------------------------------*/
*date = ( year * 1000 ) + jday;

return E_SUCC;
}
