/*****************************************************************************

HEADER:         readcpa.h

PURPOSE:        This header contains prototypes, universal constants, and
 		macros common to the READCPA application.

ALGORITHM REFERENCES

        Source code for "vmsbackup" by John D. Carey and Sven-Ove Westberg

*****************************************************************************/

#ifndef READCPA_H
#define READCPA_H

#define LABEL_SIZE 80		/* Size of ANSI label records		*/

/* Portability typedefs.  Much of the code to read the binary data from the
   backup tapes relies on data types of certain, forced sizes (16-bit or
   32-bit integers).  If this code is ever ported to a machine where the
   assumptions made below are not correct, then all that will need to be
   changed are the typedefs; the rest of the code will be fine. */

typedef long int32;		/* Both of these assumptions are true on */
typedef short int int16;	/* the SGIs, the Suns, and the DGs.      */

/* These two macros take a series of bytes and construct a single integer
   out of them, shifting each of their values into the appropriate 
   positions for the given data type (int16 or int32).  One might expect
   to be able to point a pointer to the target data type directly at the
   data in the buffer with an appropriate cast, but due to the possible
   four-byte alignment of data types, this does not work.  Rather than
   rely on compiler-specific extensions (#pragmas and whatnot) to
   control strict alignment, these more generic macros are used.  The
   use of these macros also automatically takes care of byte-swapping. */

#define to_int16(ptr, offset) ((ptr[offset + 1] << 8) + ptr[offset])
#define to_int32(ptr, offset) ((ptr[offset+3] << 24) + (ptr[offset+2] << 16) + \
                               (ptr[offset+1] << 8) + ptr[offset])

/* Mnemonic names for VMS BACKUP record types.  For the purposes of this
   application (at the current time), only types 0, 1, 3, and 4 will be
   present on the tape.  Record type 3 indicates the start of a file; 
   type 4 contains the actual file data.  These are the two most 
   important record types to be read from the tape.  Type 1 indicates a
   summary of the files present in the saveset. */

#define REC_NULL 0
#define REC_SUMMARY 1
#define REC_VOLUME 2
#define REC_FILE 3
#define REC_VBN 4
#define REC_PHYSVOL 5
#define REC_LBN 6
#define REC_FID 7

/* The number of sub-records within a record of type REC_FILE; currently
   we will hard-code this to 10 (presumably, because that's what the VMS
   BACKUP specification requires. */

#define FILE_RECS 10

/* An arbitrary limit to the number of savesets on a tape. */

#define MAX_IMG 100

/* If compiling with a standard C compiler, take advantage of function
   prototypes.  All conforming standard C compilers are required to
   #define __STDC__ (ISO/IEC 9899:1990) */

#ifdef __STDC__
int create_acquire(int, char *, char *);
int getpar(struct PARBLK *, char *, int *, int *, char *);
int read_header(long, long *, int *);
int read_trailer(long, int);
int mount_tape(char *, long *);
int close_tape(void);
#endif

#endif /* #ifndef READCPA_H */
