#ifndef PARSE_TP_H
#define PARSE_TP_H

#define DEFAULT_TAR_BLK 10240
#define RECORD_SIZE     512
#define RECORDS         DEFAULT_TAR_BLK/RECORD_SIZE
#define NAME_SIZE       100
#define TUNMLEN         32
#define TGNMLEN         32
#define SPARSE_EXT_HDR  21
#define SPARSE_IN_HDR   4
#define MIN_BAD         5

#define MAX_IMAGES 50

enum stats { HEADER_FAILURE, HEADER_SUCCESS, BUF_FULL, BUF_EMPTY, BUF_PRTL };

/* The sparse structure and the record union where
   both acquired from the GNU tar source.
--------------------------------------------------*/

/* This structure is used in the extended header portion of the
   record union.  The extended header is not implemented though.
----------------------------------------------------------------*/
struct sparse
{
   char offset[12];
   char numbytes[12];
};

/* This union is used to read retrieve the
   information contained in the tar header.
   The extended header structure is not used
   but it is left in for possible future use.
---------------------------------------------*/ 
union record
{
   char charptr[RECORD_SIZE];

   struct header
   {
      char name[NAME_SIZE];
      char mode[8];
      char uid[8];
      char gid[8];
      char size[12];
      char mtime[12];
      char check_sum[8];
      char link_flag;
      char link_name[NAME_SIZE];
      char magic[8];
      char uname[TUNMLEN];
      char gname[TGNMLEN];
      char devmajor[8];
      char devminor[8];
   } header;

   struct extended_header
   {
      struct sparse sp[SPARSE_EXT_HDR];
      char isextended;
   } ext_hdr;
};

#endif /* #ifndef PARSE_TP_H */
