/*******************************************************************************
FUNCTION:	packline

PURPOSE:	Pack the current line back into the buffer to store any user
		updates.

		This function will return E_SUCC upon successful completion or
		E_FAIL if an error is encountered.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "archio.h"


int packline
(
    struct ARCDESC *arc/* IN - Archive descriptor record                     */
)
{
unsigned char *pck_ptr; /* Ptr into the packed I/O buffer                     */


/* Swap the pixels for ascending passes being processed in a non-satellite
   view and set a pointer to the correct position in the packed buffer.
--------------------------------------------------------------------------*/
if (arc->ascending && !arc->satview)
    {
    c_pixflip(arc->samples, arc->words_img);
    pck_ptr = arc->packed + (arc->last - arc->current) * arc->arcbytes;
    }
else
    pck_ptr = arc->packed + (arc->current - arc->first) * arc->arcbytes;


/* Pack the minor frame and image data back into the I/O buffer
---------------------------------------------------------------*/
c_pck32(arc->mnr,     &pck_ptr[0],        &arc->words_mnr);
c_pck32(arc->samples, &pck_ptr[ARCH_MNR], &arc->words_img);


/* Set the saveline flag indicating the current line has been saved
-------------------------------------------------------------------*/
arc->saveline = FALSE;

return(E_SUCC);
}
 
