/******************************************************************************
FUNCTION:	GET_REC_TAPE

PURPOSE:	Get a record of Level 1b data from tape.

Version  Date   Programmer       Modifications
-------  -----  ---------------  ----------------------------------------------
  1.0    01/92  T.Baltzer        Development
  1.1    05/93  J.Fenno          Modified to use the new AVHRR model and the
				 new archio supports.
  1.2    09/97	D.Lloyd		 Modified for use as a support routine by
				 forcing externs to be passed as parameters.

PROJECT:	ADAPS
******************************************************************************/

#include "las.h"
#include "tapeio.h"


int get_rec_tape
(
    long *tun,			/* Unit number for tape drive		*/
    unsigned char *recptr,	/* Pointer to the Level 1b scan record	*/
    long *nbyte,		/* Record size of data to read		*/
    long *volnum,		/* Volume number			*/
    char comment[][CMLEN],	/* Description of tape			*/
    long *tape			/* Flag for tape open			*/
)
{
char    density[5];		/* default Tape density			*/
char	drive[CMLEN];           /* drive name                           */

long	inout = TREAD;          /* Read only from tape                  */
long	mode = HANDLE;		/* Handle tape errors			*/
long	rewopt = REWOFFL;       /* Rewind off-line option               */
long	skip = 0;		/* Number of files to skip              */
long	status;			/* Status variables			*/
long	zero = 0;               /* Value of zero                        */


status = c_tpread( tun, recptr, nbyte, &mode );
if ( status == *nbyte )
    return E_SUCC;

if (( status == TAPE_EOF ) || ( status == TAPE_EOV ))
   return status;

/* 
  End Of Tape or error reading tape:
  Close the tape.  If EOT and another tape has been specified to read,
  open it.
*/
else
   {
   if ( status == TAPE_EOT )
      {
      if ( strcmp( comment[*volnum], "\0" ) != 0 )
         {
	 status = c_tpclos( tun, &mode, &rewopt, &zero );
	 *tape = FALSE;
         strcpy( density, "0" );
	 rewopt = REWIND;
         status = c_tpopen( tun, &skip, &mode, &rewopt, density, &inout,
			comment[*volnum], drive );
         if ( status != TAPE_SUCC )
             return E_FAIL;
         *tape = TRUE;
         (*volnum)++;
         status = get_rec_tape( tun, recptr, nbyte, volnum, comment, tape );
         if ( status != E_SUCC )
            {
            c_lasmsg(FALSE,"get_rec_tape-span",
		     "Unable to read data spanning tape volumes");
            return E_FAIL;
            }
         }
      else
         {
         c_lasmsg(FALSE,"get_rec_tape-ueot","Unexpected End of Tape");
         return TAPE_EOT;
         }
     }
  else
     {
     c_lasmsg(FALSE,"get_rec_tape-reco","Recover data that has been read");
     return status;
     }
   }

return E_SUCC;

}  /* END get_rec_tape */
