/*****************************************************************************
FUNCTION:       PUT_REC_TAPE

PURPOSE:        Write a record of Level 1b data to tape.  If EOT is
		encountered a new tape is mounted and record is written
		to the new tape.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    09/97  D.Lloyd         Initial development
  1.1    10/97  D.Lloyd		Removed code to span tape.  If physical EOT,
				write tape marks, close tape and return.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "tapeio.h"


int put_rec_tape
(
    long *tun,             /* tape logical unit number             */
    unsigned char *recptr, /* pointer to level 1b scan record      */
    long *nbyte,	   /* number of bytes in one record        */
    long *tape             /* tape open flag                       */
)

{
long    mode = HANDLE;          /* handle tape errors                   */
long    rewopt = REWOFFL;       /* rewind off-line flag                 */
long    status;                 /* funtion return status                */
long    two = 2;                /* value of two                         */


/*
  Write the record and get the write status
*/
status = c_tpwrit( tun, recptr, nbyte, &mode );
if ( status == *nbyte )
    return E_SUCC;

/*
  An error occurred, check for end of tape or write error.
*/
else
    {
    /* If EOT, write tape mark, close tape and return error */
    if ( status == TAPE_EOT )
        {
	c_lasmsg(FALSE,"put_rec_tape-eot",
		 "EOT encountered: record not written");
        status = c_tpclos( tun, &mode, &rewopt, &two );
        *tape = FALSE;
        return status;
        }  /* END if */

    else
        {
        c_lasmsg(FALSE,"put_rec_tape-write","Tape write error");
        return status;
        }  /* END else */

    }  /* END else */


}  /* END put_rec_tape */
