/****************************************************************************
NAME:		CREATE_MODSID

PURPOSE:	Generate a 'modsid' file for an acquired image.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  1.0	 T. Baltzer    03/01	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>              /* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include <stdlib.h>
#include "acquman.h"

#define E_SUCC 0
#define E_FAIL -1

/* Local prototypes*/
void wrtascii
    (
    char filename[],		/* File to write to			*/
    char buffer[]		/* Output buffer			*/
    );



void create_modsid
    (
    char sceneid[] 		/* 16-character scene id	*/
    )

{

long now;		/* time now				*/
long status;		/* return status			*/

char modsidbuf[40];	/* Modify by scene id buffer		*/
char modsidfn[256];	/* Modify by scene id file name		*/
char tstamp[7];		/* Time stamp				*/


struct tm *curtime,	/* current time				*/
          *localtime();	/* get current time			*/


/* Generate the contents of the file
-----------------------------------------------------------*/
sprintf(modsidbuf, "%s - 15 D 00 - -    - -", sceneid);


/* Generate a unique name for the file
-----------------------------------------------------------*/
now = time(NULL);
curtime = localtime(&now);
sprintf(tstamp,"%02d%02d%02d",curtime->tm_hour,curtime->tm_min,curtime->tm_sec);
strcpy(modsidfn,getenv("ADAPSACQ"));
sprintf(modsidfn, "%s/modsid_%s.sch", modsidfn, tstamp);


/* Write the contents to the file
-----------------------------------------------------------*/
wrtascii(modsidfn,modsidbuf);


return;
}

/****************************************************************************
NAME:		WRTASCII

PURPOSE:	Write a line buffer out to the specified file.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  1.0	 T. Baltzer    12/93	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif
#ifdef sun
#include <fcntl.h>
#endif

void wrtascii
    (
    char filename[],		/* File to write to			*/
    char buffer[]		/* Output buffer			*/
    )

{
int	fd;			/* File descriptor to the file		*/

long	nonfatal = 1,		/* Non-fatal error code			*/
	maxtime = 60,		/* Maximum to wait on open/lock		*/
	flags,			/* Open flags				*/
	status,			/* Return status			*/
	len;			/* Length of the output buffer		*/


/*
  Open and lock the file for a moment of exclusive use
*/
flags = O_WRONLY | O_CREAT | O_APPEND;
status = cmlock(filename,&fd,flags,maxtime);
if (status != E_SUCC)
    {
    write_error( FALSE, "wrtascii", "lock", "Error locking the output file" );
    return;
    }

/*
  Seek to the end of the file because cmlock
  resets flag and opens the file for read/write, not append
*/
status = lseek(fd, 0, SEEK_END);
if (status == E_FAIL)
    {
    write_error(FALSE,"wrtascii", "skip", "Error skipping to end of the file" );
    return;
    }

/*
  Write a line of buffer data to the file
*/
len = strlen(buffer);
write(fd,buffer,len);

/*
  Unlock and close the file
*/
status = unlock(&fd,maxtime);
if (status != E_SUCC)
    {
    write_error( FALSE, "wrtascii", "unlock", "Error unlocking the file " );
    close(fd);
    }
    
return;
}
/******************************************************************************
FUNCTION:       cmlock

PURPOSE:        Opens the specified file and attempts to lock it.
                If the file is already locked, cmlock will wait a little
                while, then try again, up to the specified time limit.
                Cmlock returns E_SUCC if successful, E_FAIL otherwise.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            include file changes
  5.0      04/88    D. Hollaren         changed include file to las
                                        return status of E_SUCC/E_FAIL
  5.1      03/89    B. Davis            Changed return from unsuccessful open
                                        to allow more descriptive error message
  5.2      01/90    B. Davis            Returned ENOENT when trying to create
                                        and lock a file that does not exist. In
                                        certain situations, when the last dir
                                        member of the path name does not exist,
                                        an attempt is made to create the dir,
                                        then call cmlock again. 
  5.3      08/94    D. Etrheim          Change 777 to 666

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Open the specified host file.
VMS automatically locks a file so nothing to be done
    error if the file was to be opened for output and it already exists.
In UNIX
   attempt to lock the file until it gets locked
   or the maximum wait time is reached.

ALGORITHM REFERENCES    none

Note: cmlock does not put out error messages before returning e_fail
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/file.h>
#ifdef sun
#include <unistd.h>
#endif

int cmlock
(
    char *hname,            /* host name of file to open    */
    int *fd,                /* file descriptor              */
    int flags,              /* file access                  */
    int maxtime             /* max time to wait (seconds)   */
)
{
mode_t oldmode;                         /* return of umask              */
int gotit=0;                            /* got the lock? (1=yes)        */

/* On Silicon Graphics, read and write permission are required to lock */
/* a file ( see flock() BUGS ).  This fix will get around this bug.    */
/* the open call sets mode to 0666 first, then any umask bits.  this is*/
/* dangerous because a user umask could close off the file for write   */
/* access to subsequent users.  We therefore set umask to be harmless, */
/* then change it back after opening the file.                         */
#if defined (sgi) || defined (sun)
flags = O_CREAT | O_RDWR;
oldmode = umask(000);
#endif

*fd = open (hname,flags,0666);          /* open file - let mode default */
if (*fd < 0)
   {
   if (errno == ENOENT)
      return(errno);
   if (*fd == -1)
      return(-2);
   else
      return (E_FAIL);
   }

#if defined (sgi) || defined (sun)
umask(oldmode);
#endif

/* use flock on unix systems    */                              /* unix */
while ((!gotit) && (maxtime>0))                                 /* unix */
    {                                                           /* unix */
#ifdef sun
   if ( lockf(*fd,F_TLOCK,0) == 0)
#else
    if (flock (*fd,(LOCK_EX|LOCK_NB)) == 0)                     /* unix */
#endif
     gotit = 1;                                                 /* unix */
    else                                                        /* unix */
        {                                                       /* unix */
        sleep(5);                                               /* unix */
        maxtime -= 5;                                           /* unix */
        }                                                       /* unix */
    }                                                           /* unix */

if (maxtime <= 0)                                               /* unix */
    return (E_FAIL);                                            /* unix */

return (E_SUCC);
}
/******************************************************************************
FUNCTION:       unlock

PURPOSE:        calls flock as does cmlock, but passes LOCK_UN to the opera-
                tion argument.  this means unlock.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      05/90    B. Davis            New Development
  1.1      11/90    D. Etrheim          Close the file on VMS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Open the specified host file.
VMS automatically locks a file so nothing to be done
    error if the file was to be opened for output and it already exists.
In UNIX
   attempt to unlock the file with the unblock option until it gets locked
   or the maximum wait time is reached.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/file.h>
#ifdef sun
#include <unistd.h>
#endif


int unlock
(
    int *fd,                /* file descriptor              */
    int maxtime             /* max time to wait (seconds)   */
)
{
    long    status;                         /* return status                */
    int gotit=0;                            /* got the lock? (1=yes)        */

    while ((!gotit) && (maxtime>0))
    {
    #ifdef sun
        if ( lockf(*fd,F_ULOCK,0) == 0)
    #else
        if (flock (*fd,(LOCK_UN)) == 0)
    #endif
            gotit = 1;
        else
        {
            sleep(5);
            maxtime -= 5;
        }
    }


    if (maxtime <= 0)
        return (E_FAIL);

    status = close (*fd);
    if (status == EOF)
        return (E_FAIL); 


    return (E_SUCC);
}
