/****************************************************************************
FUNCTION:       fork_child

PURPOSE:        Fork a child process.

INPUT:          Child name

OUTPUT:         Child socket descriptor
		Child process id

RETURNS:        E_SUCC, E_FAIL

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   03/01   T. Baltzer      Initial development

PROJECT:        ADAPS
****************************************************************************/
#include <stdio.h>      /* standard C i/o facilities */
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>     /* Unix System Calls */
#include <sys/types.h>  /* system data type definitions */
#include <sys/socket.h> /* socket specific definitions */
#include <netinet/in.h> /* INET constants and stuff */
#include <arpa/inet.h>  /* IP address conversion stuff */
#include "acquman.h"	/* ACQUMAN definitions	*/

#define E_SUCC 0
#define E_FAIL -1


void fork_child
    (
    char childname[10],	/* Child name			*/
    int *csd,		/* Child socket descriptor	*/
    pid_t childid 	/* Child process id		*/
    )

{

int ld;				/* descriptor referencing the socket	*/
struct sockaddr_in skaddr;	/* socket structure			*/
struct sockaddr_in from;	/* socket structure			*/
int addrlen,length;		/* Length of structure			*/
char portarg[25];


/* create an endpoint socket for communication
-----------------------------------------------------*/
if ((ld = socket( PF_INET, SOCK_STREAM, 0 )) < 0)
    {
    write_error( FALSE, "fork_child", childname, "Problem creating socket" );
    return;
    }

skaddr.sin_family = AF_INET;
skaddr.sin_addr.s_addr = htonl(INADDR_ANY);
skaddr.sin_port = htons(0);

/* Bind a name to the socket created above
--------------------------------------------------------------------*/
if (bind(ld, (struct sockaddr *) &skaddr, sizeof(skaddr))<0)
    {
    write_error( FALSE, "fork_child", childname, "Problem binding socket" );
    return;
    }

/* Get the current name for the socket created above
------------------------------------------------------------*/
length = sizeof( skaddr );
if (getsockname(ld, (struct sockaddr *) &skaddr, &length)<0)
    {
    write_error( FALSE, "fork_child", childname, "Error calling getsockname" );
    return;
    }


/* fork the child with the port number
-------------------------------------*/
sprintf( portarg, "%d", ntohs(skaddr.sin_port));
if (( childid = fork()) < 0 )
    {
    write_error( FALSE, "fork_child", childname, "Error forking child" );
    return;
    }
else if ( childid == 0 )
    {
    if (( execlp( childname, childname, "-port", portarg, NULL )) < 0 )
        {
	write_error( FALSE, "fork_child", childname, "Cannot execute child" );
	return;
	}
    }

/* listen for child's connection
----------------------------------*/
if ( listen( ld, 5 ) < 0 )
    {
    write_error( FALSE, "fork_child", childname, "Error calling listen" );
    return;
    }

/* accept a connection on the socket
--------------------------------------*/
if ( (*csd = accept( ld, (struct sockaddr*) &from, &addrlen)) < 0)
    {
    write_error( FALSE, "fork_child", childname, "Error calling accept" );
    return;
    }

/* Get the current name for the socket accepted above
-------------------------------------------------------*/
length = sizeof( skaddr );
if (getsockname(*csd, (struct sockaddr *) &skaddr, &length)<0)
    {
    write_error( FALSE, "fork_child", childname, "Error calling getsockname" );
    return;
    }


return;
}
