/*******************************************************************************
FUNCTION:	start_display

PURPOSE:	Set up for a new display: open input file, clear old display
		and set 'Acquiring scene' message.  Call display.  Close
		input file on exit.

INPUT:		the graphics context
		pointer to the display image
		pixmap for the background display
		array of bands to display in rgb
		socket descriptor

OUTPUT:		none

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: start_display.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <unistd.h>
#include "shoacqu.h"


void start_display
    (
    GC     gc,			/* IN - the graphics context		*/
    XImage *theimage,		/* IN - pointer to the display image	*/
    Pixmap pixmap,		/* IN - background pixmap for display	*/
    int    *rgb_bands,          /* IN - arrays of bands to display rgb  */
    int    sock_fd		/* IN - socket file descriptor		*/
    )
    {
    char infile[STRLEN];	/* input file name			*/
    char *thebuf = theimage->data;/* pointer to the image data		*/

    int  daynightflag;		/* flag DAY or NIGHT			*/
    int  i;
    int  infd;			/* input file descriptor		*/

    const struct SHIFT *shift;	/* RGB shift values			*/

    XmString text;		/* X string for widget text		*/


    /* Change the message to 'acquiring scene'
    ------------------------------------------*/
    text = XmStringCreateLocalized( "Acquiring scene" );
    XtVaSetValues( getlabelw(), XmNlabelString, text, NULL );
    XmStringFree( text );

    /* Get the next scene information and open input file
    -----------------------------------------------------*/
    if ( sock_fd != 0 )
	get_next_scene( sock_fd, rgb_bands );
    shift = getshift();
    daynightflag = getdaynightflag();
    getinfile( infile );
    if ( infile[0] == '\0' )
	{
	write_log( FALSE, "start_display", "nofile", "No file name specified" );
        return;
	}
    infd = openinput( infile );

    /* Clear the display
    --------------------*/
    memset( thebuf, 0, IMGLINESIZE );
    for ( i = 0; i < IMGHEIGHT; i++ )
	XPutImage( XtDisplay( getdrawingarea()), pixmap,
	    gc, theimage, 0, 0, 0, i, IMGWIDTH, 1 );
    XCopyArea( XtDisplay( getdrawingarea()), pixmap,
        XtWindow( getdrawingarea()), gc, 0, 0, IMGWIDTH, IMGHEIGHT, 0, 0 );
    
    /* Do the display
    -----------------*/
    display_image( gc, theimage, pixmap, infd, daynightflag, rgb_bands, shift );

    /* Close and return
    -------------------*/
    close( infd );
    if ( sock_fd != 0 )
	get_next_time( sock_fd );
    }  /* END start_display */
