/****************************************************************************
NAME:				CORRELATE

PURPOSE:  Performs image-to-image correlation

VERSION	 DATE	AUTHOR	 
-------	 ----	------	  
  1.0	 8/92	D. Steinwand		Conversion to C
  2.0	 2/93	T. Mittan		Released code
  3.0	 7/94	T. Mittan		Released ADAPS version to LAS
  4.0	 1/95	T. Mittan		Modified to use any number of tiepoints	
  5.0    10/99  T. Ratliff              Removed the 2gig and prototyped the
                                          the functions.

REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "correlate.h"
#include "complex.h"
#include "util.h"
#include "typlim.h"

/* Prototypes
   ----------*/
void cabort
(
    long status			/* Error condition flag 	*/
);
long accept
(
    long confirm,          /* Boolean--confirm mode	 		*/
    struct TPLDATA *data   /* TPL data record 				*/
);
long get_tp
(
    struct MTPDATA *data, /* One Merged Tie Point data record (user input)  */
    long subcmd		  /* Correlation method                             */
);
long getpar
(
    struct PARBLK *vblk,      /* TAE variable block                         */
    struct CORRIMG *ref,      /* Name of the reference image                */
    struct CORRIMG *srch,     /* Name of the search image                   */
    char *mtp,		      /* Name of the input merged tie point file    */
    char *tpl,		      /* Name of the input tie point location file  */
    char *out,		      /* Name of the output tie point location file */
    struct CORRPAR *corr,     /* Correlation parameters                     */
    long *ss,
    long *inc,
    float *alpha,
    long *subcmd,
    long *confirm,            /* Confirm mode--boolean                      */
    float *threshold          /* Cloud cover threshold value                */
);
long extract
(
    struct FDESC **ptr,     /* File pointer to image file                   */
    struct CORRIMG *image,  /* Image file parameters                        */
    double *tp,		    /* Tie point (l,s)                              */
    long *size,		    /* Size of chip--nl,ns                          */
    char *name,		    /* Name of chip image file                      */
    long minval,	    /* Minimum chip size                            */
    long maxval,	    /* Maximum chip size                            */
    float *buffer	    /* Chip buffer                                  */
);


int main(void)
{
struct PARBLK vblk;		/* TAE parameter block 			*/
struct TPLHEAD tpl_header;	/* TPL file header record 		*/
struct TPLDATA tpl_data;	/* Tie point location data record 	*/
struct MTPHEAD mtp_header;	/* MTP file header record 		*/
struct MTPDATA mtp_data;	/* Merged tie point data record	 	*/
struct CORRPAR corr;		/* Correlation parameters 		*/
struct CORRIMG ref;		/* Reference image file parameters 	*/
struct CORRIMG srch;		/* Search image parameters 		*/

static float edens[16641];	/* Array of edge density values 	*/

float fltmax, diff;		/* Filter range variables 		*/
float *ref_chip;		/* Reference chip buffer 		*/
float *srch_chip;		/* Search chip buffer 			*/
float alpha;
float threshold = 0.0;		/* Cloud cover threshold value		*/

double nominal[2];		/* Nominal search coordinates 		*/
double chipcenter0,chipcenter1;

struct FDESC *rfdsc;		/* Full reference image file desc 	*/
struct FDESC *sfdsc;		/* Full search image file desc 		*/
struct PTDESC *in_mtp;		/* Input merged tie point file desc 	*/
struct PTDESC *in_tpl;		/* Input tie point location file desc 	*/
struct PTDESC *out_tpl;		/* Output tiepoint location file desc 	*/

long nscale[2];			/* Scale parameter 			*/
long size;			/* Size of buffers for malloc 		*/
long ptyp;			/* Type of tie point file 		*/
long opt[2];			/* Option for image I/O 		*/
long nbands;			/* Number of bands 			*/
long nl, ns;			/* Number of lines and samples 		*/
long acc;			/* File access mode 			*/
long count;			/* Loop counter 			*/
long i;				/* Loop counter 			*/
long manual_input;		/* Boolean--manual input of tie points? */
long one = 1;			/* Constant of 1 			*/
long four = 4;			/* Constant of 4 			*/
long std_out;			/* Dummy variable to standard output 	*/
long confirm;			/* Boolean--confirm flag 		*/
long STAT_FLAG;			/* Return status variable               */
long ssize;			/* Size of search image--phase corr 	*/
long inc;			/* Step size--phase corr 		*/
long subcmd;			/* Method of correlation used 		*/
long pts_input=0;		/* Number of points to check		*/
long pts_correlated=0;		/* Number of points that correlate	*/

short *bsrch;			/* Ptr to search binary edge subimage 	*/
short *bref;			/* Ptr to reference binary edge subimage */

char mtp[CMLEN];		/* Input Merged Tie Point file name 	*/
char tpl[CMLEN];		/* Input Tie Point Location file name 	*/
char out[CMLEN];		/* Output Tie Point Location file name 	*/
char mess[80];			/* Message buffer for informational messages */

c_inlas(&vblk, &std_out);

/* Get processing parameters from the user
  ---------------------------------------*/
CHECK(getpar(&vblk,&ref,&srch,mtp,tpl,out,&corr,&ssize,&inc,&alpha,
      &subcmd,&confirm,&threshold));

/* Dynamically allocate buffers
  ----------------------------*/
if(subcmd==EDGE_CORR)
   {
   size = SRCH_BUFSIZE * sizeof(short);
   bsrch = (short *)malloc(size);
   if (bsrch == NULL) cabort(NO_SPACE);

   size = REF_BUFSIZE * sizeof(short);
   bref = (short *)malloc(size);
   if (bref == NULL) cabort(NO_SPACE);
   }

size = SRCH_BUFSIZE * sizeof(float);
srch_chip = (float *)malloc(size);
if (srch_chip == NULL) cabort(NO_SPACE);

size = REF_BUFSIZE * sizeof(float);
ref_chip = (float *)malloc(size);
if (ref_chip == NULL) cabort(NO_SPACE);

/* Determine reference image mode--full image or chips?
  ----------------------------------------------------*/
if (strcmp(ref.name, "") == 0) ref.chip_flag = TRUE;
else 
   {
   ref.chip_flag = FALSE;
   opt[0] = opt[1] = 0;
   nbands = 0; nl = ns = 0;
   acc = IREAD;
   CHECK(c_eopenr(&rfdsc,ref.name,&nbands,&one,&one,&nl,&ns,&acc,&four,
       opt,&one));
   ref.nl = nl;
   ref.ns = ns;
   }

/* Determine search image mode--full image or chips?
  -------------------------------------------------*/
if (strcmp(srch.name, "") == 0) srch.chip_flag = TRUE;
else 
   {
   srch.chip_flag = FALSE;
   opt[0] = opt[1] = 0;
   nbands = 0; nl = ns = 0;
   acc = IREAD;
   CHECK(c_eopenr(&sfdsc,srch.name,&nbands,&one,&one,&nl,&ns,&acc,&four,
                opt,&one));
   srch.nl = nl;
   srch.ns = ns;
   }

/* Determine corrdinate input mode and open all tie point files
  ------------------------------------------------------------*/
if (strcmp(mtp, "") == 0) manual_input = TRUE;
else
   {
   manual_input = FALSE;
   acc = IREAD; ptyp = MTP;
   CHECK(c_ptopen(&in_mtp, mtp, &ptyp, &acc, &mtp_header));
   }

if (strcmp(tpl, "") != 0)
   {
   acc = IREAD; ptyp = TPL;
   CHECK(c_ptopen(&in_tpl, tpl, &ptyp, &acc, &tpl_header));
   acc = IWRITE;
   CHECK(c_ptopen(&out_tpl, out, &ptyp, &acc, &tpl_header));

   for (;;)
      {
      status = c_ptio(&in_tpl, &tpl_data);
      if (status == E_EOF) break;
      if (status == E_FAIL) cabort(status);
      if (!c_ckptid(tpl_data.pt_id)) cabort(BAD_PTID);
      CHECK(c_ptio(&out_tpl, &tpl_data));
      }

   corr.mincorr = tpl_header.min_str;
   corr.maxdiff = tpl_header.max_dis;
   corr.edgeval = tpl_header.edge_thres;

   if (strcmp(tpl_header.peak_method, "PARABOLOID") == 0) corr.fitmeth = PARAB;
   if (strcmp(tpl_header.peak_method, "GAUSSIAN") == 0) corr.fitmeth = GAUSS;
   if (strcmp(tpl_header.peak_method, "RECIPROCAL") == 0) corr.fitmeth = RECIP;
   if (strcmp(tpl_header.peak_method, "NEAR_INT") == 0) corr.fitmeth = INTEGER;
   if (strcmp(tpl_header.peak_method, "NO_CORR") == 0) cabort(NO_CORR);

   if ((strcmp(tpl_header.corr_type,"EDGE")!=0)&&(subcmd!=EDGE_CORR))
		cabort(CORR_METHOD);
   if ((strcmp(tpl_header.corr_type,"GREY")!=0)&&(subcmd!=GREY_CORR))
		cabort(CORR_METHOD);
   if ((strcmp(tpl_header.corr_type,"PHASE")!=0)&&(subcmd!=PHASE_CORR))
		cabort(CORR_METHOD);
   }
else
   {
   tpl_header.edge_thres = corr.edgeval;
   tpl_header.min_str = corr.mincorr;
   tpl_header.max_dis = corr.maxdiff;
   if (corr.fitmeth == PARAB) strcpy(tpl_header.peak_method,"PARABOLOID");
   if (corr.fitmeth == GAUSS) strcpy(tpl_header.peak_method,"GAUSSIAN");
   if (corr.fitmeth == RECIP) strcpy(tpl_header.peak_method,"RECIPROCAL");
   if (corr.fitmeth == INTEGER) strcpy(tpl_header.peak_method,"NEAR_INT");
   if(subcmd==EDGE_CORR)strcpy(tpl_header.corr_type,"EDGE");
   if(subcmd==GREY_CORR)strcpy(tpl_header.corr_type,"GREY");
   if(subcmd==PHASE_CORR)strcpy(tpl_header.corr_type,"PHASE");

   for (i = 0; i < 8; i++)
      tpl_header.corners[i] = mtp_header.corners[i];

   tpl_header.pdist[0] = mtp_header.pdist[0];
   tpl_header.pdist[1] = mtp_header.pdist[1];

   for (i = 0; i < 15; i++)
      tpl_header.projprms[i] = mtp_header.projprms[i];

   tpl_header.proj_valid = mtp_header.proj_valid;
   tpl_header.code = mtp_header.code;
   tpl_header.zone = mtp_header.zone;
   tpl_header.datum = mtp_header.datum;

   strcpy(tpl_header.ref_mode, mtp_header.chip_mode);
   strcpy(tpl_header.units, mtp_header.units);
   strcpy(tpl_header.ref_name, ref.name);
   strcpy(tpl_header.sea_name, srch.name);

   ptyp = TPL; acc = IWRITE;
   CHECK(c_ptopen(&out_tpl, out, &ptyp, &acc, &tpl_header));
   }

/* Determine scaling factor needed, if any, to insure at most 2**15
   histogram bins
  --------------*/
if (subcmd==EDGE_CORR)
   {
   if (srch.minval == 0 && srch.maxval == 0) 
      {
      if (srch.nbytes == 1) diff = MAXUTINY;
      if (srch.nbytes == 2) diff = MAXWORD;
      if (srch.nbytes == 3) diff = MAXLONG;
      if (srch.nbytes == 4) diff = MAXREAL;
      }
   else diff = (float)(srch.maxval - srch.minval);
   for (fltmax = 4.0 * diff, count = 0; fltmax > 32767; count++) fltmax /= 2.0;
   nscale[0] = count;

   if (ref.minval == 0 && ref.maxval == 0) 
      {
      if (ref.nbytes == 1) diff = MAXUTINY;
      if (ref.nbytes == 2) diff = MAXWORD;
      if (ref.nbytes == 3) diff = MAXLONG;
      if (ref.nbytes == 4) diff = MAXREAL;
      }
   else diff = (float)(ref.maxval - ref.minval);
   for (fltmax = 4.0 * diff, count = 0; fltmax > 32767; count++) fltmax /= 2.0;
   nscale[1] = count;
   }

/* Process all tie point pairs
  ---------------------------*/
for (;;)
   {
   if (manual_input)
      {
      status = get_tp(&mtp_data,subcmd);
      if (status == E_EOF) break;
      }
   else
      {
      status = c_ptio(&in_mtp, &mtp_data);
      if (status == E_EOF) break;
      if (status == E_FAIL) cabort(status);
      }
   pts_input++;

/* Initally set tpl active flag to "accept automatically extracted point"
  ---------------------------------------------------------------------*/
   tpl_data.active = 1;

/* Extract reference window
  ------------------------*/
   if(subcmd==PHASE_CORR)
      {
      mtp_data.chip_size[0]=32;
      mtp_data.chip_size[1]=32;
      }

   status = extract(&rfdsc, &ref, mtp_data.ref_coord, mtp_data.chip_size,
                 mtp_data.ref_chip, REF_MIN, REF_MAX, ref_chip);
   if (status == BAD_CHIP_DDR) tpl_data.active = 6;
   if (status == BAD_SIZE) tpl_data.active = 7;
   if (status == OUTSIDE_IMAGE) tpl_data.active = 8;
   if (status == READ_ERROR) tpl_data.active = 9;

/* Extract search window
  ---------------------*/
   if (tpl_data.active == 1)
      {
      if(subcmd==PHASE_CORR)
         {
         tpl_data.sea_size[0] = ssize;
         tpl_data.sea_size[1] = ssize;
         status = extract(&sfdsc, &srch, mtp_data.nominal, tpl_data.sea_size,
                 mtp_data.sea_chip, 32, 128, srch_chip);
         }
      else
         {
         tpl_data.sea_size[0]=mtp_data.chip_size[0]+2*mtp_data.offsets[0];
         tpl_data.sea_size[1]=mtp_data.chip_size[1]+2*mtp_data.offsets[1];
         status = extract(&sfdsc, &srch, mtp_data.nominal, tpl_data.sea_size,
                 mtp_data.sea_chip, SRCH_MIN, SRCH_MAX, srch_chip);
         }
      if ((status == BAD_COEFS) &&
        (strcmp(tpl_header.ref_mode,"CHIP-PRE") == 0))
                {
                tpl_data.active = 6;
                c_errmsg("Error reading chip DDR information", " ",&one);
                }
      if (status == BAD_CHIP_DDR) tpl_data.active = 6;
      if (status == BAD_SIZE) tpl_data.active = 7;
      if (status == OUTSIDE_IMAGE) tpl_data.active = 8;
      if (status == READ_ERROR) tpl_data.active = 9;
      }

/* Initialize the TPL data record with values from the MTP data record
  -------------------------------------------------------------------*/
   tpl_data.geo_coord[0] = mtp_data.geo_coord[0];
   tpl_data.geo_coord[1] = mtp_data.geo_coord[1];
   tpl_data.ref_coord[0] = mtp_data.ref_coord[0];
   tpl_data.ref_coord[1] = mtp_data.ref_coord[1];
   tpl_data.sea_coord[0] = mtp_data.nominal[0];
   tpl_data.sea_coord[1] = mtp_data.nominal[1];
   tpl_data.nominal[0] = mtp_data.nominal[0];
   tpl_data.nominal[1] = mtp_data.nominal[1];
   tpl_data.elevation = mtp_data.elevation;
   tpl_data.ref_size[0] = mtp_data.chip_size[0];
   tpl_data.ref_size[1] = mtp_data.chip_size[1];
   tpl_data.req_size[0] = mtp_data.chip_size[0];
   tpl_data.req_size[1] = mtp_data.chip_size[1];
   tpl_data.offsets[0] = mtp_data.offsets[0];
   tpl_data.offsets[1] = mtp_data.offsets[1];
   tpl_data.zoom_factor = mtp_data.zoom_factor;
   tpl_data.chip_loc[0] = srch.starting_line;
   tpl_data.chip_loc[1] = srch.starting_sample;
   tpl_data.location[0] = ref.starting_line;
   tpl_data.location[1] = ref.starting_sample;
   strcpy(tpl_data.pt_id, mtp_data.pt_id);
   strcpy(tpl_data.ref_chip, "");

   for (i = 0; i < 2; i++) nominal[i] = mtp_data.offsets[i];

/* Perform correlation
  -------------------*/
   if (tpl_data.active == 1)
	 {
         if(subcmd==PHASE_CORR) 
	   phase_corr(&tpl_data,&corr,ref_chip,srch_chip,ssize,inc,
		      alpha);
         else if(subcmd==EDGE_CORR) 
	   edge_corr(&tpl_data,&corr,ref_chip,srch_chip,nominal,nscale,bsrch,
                      bref, edens, threshold);
         else /* (subcmd==GREY_CORR) */
	   grey_corr(&tpl_data,&corr,ref_chip,srch_chip,nominal);
	 }

/* Adjust coordinates if chips were used
  -------------------------------------*/
   if (tpl_data.active != 6)
        {
        if (strcmp(tpl_header.ref_mode,"CHIP-TEM") == 0)

/* Image-to-image registration chips used (i.e. AVHRR or Landsat GCPLIB).  
   mtp_data.geo_coord is ignored and the nominal coordinates are equal to
   the full-scene reference coordinates.  The computed search image 
   coordinates are identical to full scene correlations...
  -------------------------------------------------------*/
           {
           chipcenter0 = tpl_data.ref_coord[0];
           chipcenter1 = tpl_data.ref_coord[1];
           tpl_data.ref_coord[0] = tpl_data.nominal[0];
           tpl_data.ref_coord[1] = tpl_data.nominal[1];
           tpl_data.nominal[0] = chipcenter0;
           tpl_data.nominal[1] = chipcenter1;
           }

        if (strcmp(tpl_header.ref_mode,"CHIP-FUL") == 0)

/* Reference chips establish a geographic registration.  mtp_data.geo_coord 
   contains a projection coordinate of the ref tie point in the projection 
   space defined by the mtp header record.  This projection coordinate must 
   be converted to output space image coordinates.  The search chip, if used, 
   is treated as if the chip mode was "CHIP-TEM", using the master coordinate 
   field.
  ------*/
           {
	   tpl_data.ref_coord[0] = ((tpl_header.corners[0] - 
		mtp_data.geo_coord[0]) / tpl_header.pdist[0]) + 1;
	   tpl_data.ref_coord[1] = ((mtp_data.geo_coord[1] -
		tpl_header.corners[1]) / tpl_header.pdist[1]) + 1;
           tpl_data.sea_coord[0] += (srch.master_line - 1);
           tpl_data.sea_coord[1] += (srch.master_samp - 1);
           }

        if (strcmp(tpl_header.ref_mode,"CHIP-PRE") == 0)

/* Reference chips are treated as they are when the mode is "CHIP-FUL" if 
   valid projection and framing information exists in the mpt/tpl file header 
   (geo_coord is also used...); or like "CHIP-TEM" if the proj & frame info 
   is not valid in the mtp/tpl file header.  The search chip must have a 
   user-defined coordinate system (in pixels), defining the rotation and 
   scaling of the chip.
  --------------------*/
           {
           if ((tpl_header.proj_valid == GEO_VALID) && (ref.chip_flag))
                {
	        tpl_data.ref_coord[0] = ((tpl_header.corners[0] - 
	          	mtp_data.geo_coord[0]) / tpl_header.pdist[0]) + 1;
	        tpl_data.ref_coord[1] = ((mtp_data.geo_coord[1] -
	          	tpl_header.corners[1]) / tpl_header.pdist[1]) + 1;
                }
           else
                {
                tpl_data.ref_coord[0] += (ref.master_line - 1);
                tpl_data.ref_coord[1] += (ref.master_samp - 1);
                }

           tpl_data.sea_coord[0] = c_eval(&one, srch.coef_y,
                        &(tpl_data.sea_coord[1]),&(tpl_data.sea_coord[0]));
  	   tpl_data.sea_coord[0] *= -1.0;
           tpl_data.sea_coord[1] = c_eval(&one, srch.coef_x,
                        &(tpl_data.sea_coord[1]),&(tpl_data.sea_coord[0]));
           }
        }

/* Report results to user and write to tie point location file
  -----------------------------------------------------------*/
   switch(tpl_data.active)
      {
      case 1: sprintf(mess,"%s [1] Successfully correlated",tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 2: sprintf(mess,
                "%s [2] Correlation peak too near edge of search image"
                ,tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 3: sprintf(mess,
                "%s [3] Subsidiary peak comparable in strength to main peak",
                tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 4: sprintf(mess,
                "%s [4] Strength of peak below minimum specified by user",
                tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 5: sprintf(mess,
                "%s [5] Diagonal displacement exceeds maximum specified",
                tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 6: return(E_FAIL);
      case 7: sprintf(mess,"%s [7] Size of image and chip do not match",
                tpl_data.pt_id);
              c_errmsg(mess," ",&one);
              break;
      case 8: sprintf(mess,"%s [8] Tie point outside image",
                tpl_data.pt_id);
              c_errmsg(mess," ",&one);
	      break;
      case 9: return(E_FAIL);
      }

   if (accept(confirm, &tpl_data)) 
      { 
      CHECK(c_ptio(&out_tpl, &tpl_data)); 
      if (tpl_data.active == 1)
         pts_correlated++;
      }
   }

/* Close all files
  ---------------*/
if (!ref.chip_flag) c_eclose(&rfdsc);
if (!srch.chip_flag) c_eclose(&sfdsc);
c_ptclse(&out_tpl);
if (strcmp(tpl, "") != 0) c_ptclse(&in_tpl);
if (!manual_input) c_ptclse(&in_mtp);

/* Write completion mesasge and exit
  ---------------------------------*/
sprintf(mess,"%d of %d points were successfully correlated (%5.2lf%c)",
        pts_correlated,pts_input,100.0*pts_correlated/pts_input,'%');
c_errmsg(mess," ",&one);
c_complt("CORRELATE", out, &one);
return (0);
}

/****************************************************************************
NAME:				accept

PURPOSE:  Prints correlation results and prompts user if confirm is set

PROGRAM HISTORY:
VERSION		DATE	AUTHOR
-------		----	------	
  1.0		8/92	D. Steinwand

*****************************************************************************/
long accept
(
    long confirm,          /* Boolean--confirm mode	 		*/
    struct TPLDATA *data   /* TPL data record 				*/
)
{
TEXT mess[80];		/* TAE message buffer 				*/
CODE term;		/* Terminator code from TAE terminal I/O 	*/
char answer[2];		/* Buffer containing answer to prompt 		*/

/* If no confirm mode, return TRUE
  -------------------------------*/
if (!confirm) return(TRUE);

/* Display results to the terminal
  -------------------------------*/
t_write(" ", T_STDCC);
sprintf(mess, "Correlation Results:");
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, 
   "  Tie point reference image coords (line, sample):  %9.2f %9.2f", 
   data->ref_coord[0], data->ref_coord[1]);
t_write(mess, T_STDCC);
sprintf(mess, 
   "  Tie point search image coords (line, sample):     %9.2f %9.2f", 
   data->sea_coord[0], data->sea_coord[1]);
t_write(mess, T_STDCC);
if (data->active == 1)
   {
   sprintf(mess, 
	"  Estimated errors in search coordinates:           %9.2f %9.2f",
           data->rms_error[0], data->rms_error[1]);
   t_write(mess, T_STDCC);
   sprintf(mess, "  Correlation strength:                             %9.2f",
           data->strength);
   t_write(mess, T_STDCC);
   sprintf(mess, "  Displacement from nominal:                        %9.2f",
           data->displacement);
   t_write(mess, T_STDCC);
   }
t_write(" ", T_STDCC);
sprintf(mess, "Search Parameters:");
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
sprintf(mess, 
   "  Nominal search coordinates (line, sample):        %9.2f %9.2f", 
   data->nominal[0], data->nominal[1]);
t_write(mess, T_STDCC);
sprintf(mess, 
        "  Requested reference subimage size (height, width):       %d   %d",
           data->req_size[0], data->req_size[1]);
t_write(mess, T_STDCC);
sprintf(mess, 
        "  Requested search offsets (above/below, left/right):      %d   %d",
           data->offsets[0], data->offsets[1]);
t_write(mess, T_STDCC);
sprintf(mess, "  Actual reference subimage window:  %9d %9d %4d %4d",
           data->location[0], data->location[1], data->ref_size[0],
           data->ref_size[1]);
t_write(mess, T_STDCC);
sprintf(mess, "  Actual search subimage window:     %9d %9d %4d %4d",
           data->chip_loc[0], data->chip_loc[1], data->sea_size[0],
           data->sea_size[1]);
t_write(mess, T_STDCC);
t_write(" ", T_STDCC);
t_write("Record this tie point in the tie point location file [Y]/N ? ", 
       T_NULL);

/* Loop until satisfactory input is given
  --------------------------------------*/
for (;;)
   {
   t_read(mess, &term);
   if (strlen(mess) == 0) 
	{
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
	return(TRUE);
	}
   sscanf(mess, "%s", answer);
   if ((strcmp(answer, "Y") == 0) || (strcmp(answer, "y") == 0)) 
	{
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
	return(TRUE);
	}
   if ((strcmp(answer, "N") == 0) || (strcmp(answer, "n") == 0)) 
	{
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
 	t_write(" ", T_STDCC);
	return(FALSE);
	}
   t_write("Please enter [Y] or N >  ", T_NULL);
   }
}

/****************************************************************************
NAME:				cabort

PURPOSE:  Handles all fatal error conditions & exits gracefully

VERSION		DATE	AUTHOR
-------		----	------	
  1.0		8/92	D. Steinwand

*****************************************************************************/
void cabort
(
    long status			/* Error condition flag 	*/
)
{
long pass = 1;			/* Do not abort processing 	*/
long fatal = -1;		/* Abort processing 		*/

switch(status)
   {
   case NO_SPACE:
	c_errmsg("Unable to allocate dynamic memory","correlate-alloc",&pass);
	break;
   case IN_BATCH:
	c_errmsg("Must enter an input MTP file in batch mode",
             "correlate-batch", &pass);
	break;
   case NO_CHIPS:
	c_errmsg("Chip processing not supported for manual parameter input",
            "correlate-nochips",&pass);
   case BAD_PTID:
	c_errmsg("Duplicate point identifiers exist in INTL","correlate-ptid",
            &pass);
	break;
   case NO_CORR:
   case CORR_METHOD:
	c_errmsg("TPL file header has invalid fields","correlate-header",
            &pass);
	break;
   }
c_errmsg("Fatal error encountered","correlate-fatal",&fatal);
}

/****************************************************************************
NAME:				get_tp

PURPOSE:  Gets a tie point coordinate from the user

VERSION		DATE	AUTHOR
-------		----	------	
  1.0	        8/92	D. Steinwand

*****************************************************************************/
long get_tp
(
    struct MTPDATA *data, /* One Merged Tie Point data record (user input)  */
    long subcmd		  /* Correlation method                             */
)
{
TEXT mess[80];		/* Buffer for user messages and input 		*/
CODE term;		/* Terminal flag for t_io 			*/
long status;		/* Status 					*/
static long pt_num = 0;	/* Prompt number (counter) 			*/

/* Increment coordinate counter
  ----------------------------*/
pt_num++;

/* Do until a valid point id is entered
  ------------------------------------*/
for (;;)
   {
#ifdef vms
   t_write(" ",T_STDCC);
#endif
   t_write(" ",T_STDCC);
   sprintf(mess," %3d.  Point identifier >  ",pt_num);
   t_write(mess,T_NULL);
   t_read(mess, &term);
   
/* Check for end of entry
  ----------------------*/
   if (strlen(mess) == 0) return(E_EOF);

/* Check that a unique point id was entered
  ----------------------------------------*/
   sscanf(mess, "%s", data->pt_id);
   if (c_ckptid(data->pt_id)) break;
   t_write(" Point id is not unique; re-enter!", T_STDCC);
   }

/* Get the reference coordinates
  -----------------------------*/
for (;;)
   {
#ifdef vms
   t_write(" ",T_STDCC);
#endif
   sprintf(mess,"       Reference line, sample >                        ");
   t_write(mess,T_NULL);
   t_read(mess, &term);
   status = sscanf(mess, "%lf %lf", &(data->ref_coord[0]), 
		&(data->ref_coord[1]));
   if (status == 2) break;
   }

/* Get the nominal search coordinates
  ----------------------------------*/
for (;;)
   {
#ifdef vms
   t_write(" ",T_STDCC);
#endif
   sprintf(mess,"       Nominal search line, sample >                   ");
   t_write(mess,T_NULL);
   t_read(mess, &term);
   status = sscanf(mess, "%lf %lf", &(data->nominal[0]), &(data->nominal[1]));
   if (status == 2) break;
   }

/* Get the ref chip size
  ----------------------*/
if(subcmd != PHASE_CORR)
   {
   for (;;)
      {
#ifdef vms
      t_write(" ",T_STDCC);
#endif
      sprintf(mess,"       Sub-image width, height [64,64] >               ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (strlen(mess) == 0) 
	{
	data->chip_size[0] = data->chip_size[1] = 64;
	break;
	}
      else 
	{
	status = sscanf(mess, "%d %d", &(data->chip_size[0]), 
				&(data->chip_size[1]));
	if (status == 2) break;
	}
      }

/* Get the search offsets
  -----------------------*/
   for (;;)
      {
#ifdef vms
      t_write(" ",T_STDCC);
#endif
      sprintf(mess,"       Horizontal, vertical search offsets [16,16] >   ");
      t_write(mess,T_NULL);
      t_read(mess, &term);
      if (strlen(mess) == 0) 
	{
	data->offsets[0] = data->offsets[1] = 16;
	break;
	}
      else 
	{
	status = sscanf(mess, "%d %d", &(data->offsets[0]), 
			&(data->offsets[1]));
	if (status == 2) break;
	}
      }
   }
data->elevation = 0.0;
data->zoom_factor = 0.0;
data->geo_coord[0] = data->geo_coord[1] = 0.0;
strcpy(data->ref_chip, "");
strcpy(data->sea_chip, "");

return(E_SUCC);
}

/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

VERSION		DATE	AUTHOR
-------		----	------	
  1.0	        8/92	D. Steinwand

*****************************************************************************/
long getpar
(
    struct PARBLK *vblk,      /* TAE variable block                         */
    struct CORRIMG *ref,      /* Name of the reference image                */
    struct CORRIMG *srch,     /* Name of the search image                   */
    char *mtp,		      /* Name of the input merged tie point file    */
    char *tpl,		      /* Name of the input tie point location file  */
    char *out,		      /* Name of the output tie point location file */
    struct CORRPAR *corr,     /* Correlation parameters                     */
    long *ss,
    long *inc,
    float *alpha,
    long *subcmd,
    long *confirm,            /* Confirm mode--boolean                      */
    float *threshold          /* Cloud cover threshold value                */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
long acc;			/* access flag for c_getnam */
long zero = 0;			/* Constant of zero */
long one = 1;			/* Constant of one */
long num_img;			/* Number of images entered */
long wind[1][4];		/* Window entered */
long bands[1][MAXBND + 1];	/* Band numbers entered */
long nbands;			/* Number of bands */
long totbands;			/* Total number of bands entered */
long same;			/* Same window flag for c_getin call */
long dtype;			/* Image data type			*/
long qflag[1][2];		/* Option flag for c_getin call */
char tdum[1][CMLEN];	/* Dummy variable for c_getin call */
double minval = MINLONG;
double maxval = MAXLONG;

/* Get the subcommand used
  -----------------------*/
var = p_find(vblk,"_SUBCMD");
if (strncmp("EDGE",SVAL(*var,0),4) == 0) *subcmd=EDGE_CORR;
else if (strncmp("PHASE",SVAL(*var,0),5) == 0) *subcmd=PHASE_CORR;
else *subcmd = GREY_CORR;

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTL", &zero, out, "tpl", &acc) != E_SUCC) return(E_FAIL);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, tpl, "tpl", &acc) != E_SUCC) return(E_FAIL);

/* Get the input merged tie point file name
  ----------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INMT", &zero, mtp, "mtp", &acc) != E_SUCC) return(E_FAIL);

/* Get the name of the reference image
  -----------------------------------*/
ref->minval = ref->maxval = 0;
ref->nbytes = 1;
var = p_find(vblk, "IN");
if (strcmp("",SVAL(*var,0)) != 0)
   {
   if(c_getin(vblk,"IN",&zero,tdum,ref->name,&num_img,wind,bands,&nbands,
            &totbands,&one,&same,qflag) != E_SUCC) return(E_FAIL);
   ref->band = bands[0][0];

   /* find minmax value for EDGE subcommand
   --------------------------------------*/
   if (*subcmd == EDGE_CORR)
      {
      c_minmax(ref->name,&one,wind,bands,&one,&minval,&maxval);
   
      if (minval > MAXLONG)
         ref->minval = MAXLONG;
      else if (minval < MINLONG)
         ref->minval = MINLONG;
      else
         ref->minval = (long)minval;

      if (maxval > MAXLONG)
         ref->maxval = MAXLONG;
      else if (maxval < MINLONG)
         ref->maxval = MINLONG;
      else
         ref->maxval = (long)maxval;
      }

      c_gettyp(ref->name,&one,&dtype);
   if (dtype == EBYTE)           ref->nbytes = 1;
   else if (dtype == EWORD)      ref->nbytes = 2;
   else if (dtype == ELONG)      ref->nbytes = 4;
   else if (dtype == EREAL)      ref->nbytes = 4;
   }
else strcpy(ref->name,"");

/* Get the name of the search image
  --------------------------------*/
srch->minval = srch->maxval = 0;
srch->nbytes = 1;
var = p_find(vblk, "IN");
if (strcmp("",SVAL(*var,1)) != 0)
   {
   if(c_getin(vblk,"IN",&one,tdum,srch->name,&num_img,wind,bands,&nbands,
            &totbands,&one,&same,qflag) != E_SUCC) return(E_FAIL);
   srch->band = bands[0][0];

   /* find minmax value for EDGE subcommand
   --------------------------------------*/
   if (*subcmd == EDGE_CORR)
      {
      c_minmax(srch->name,&one,wind,bands,&one,&minval,&maxval);

      if (minval > MAXLONG)
         srch->minval = MAXLONG;
      else if (minval < MINLONG)
         srch->minval = MINLONG;
      else
         srch->minval = (long)minval;

      if (maxval > MAXLONG)
         srch->maxval = MAXLONG;
      else if (maxval < MINLONG)
         srch->maxval = MINLONG;
      else
         srch->maxval = (long)maxval;
      }

   c_gettyp(srch->name,&one,&dtype);
   if (dtype == EBYTE)      srch->nbytes = 1;
   else if (dtype == EWORD) srch->nbytes = 2;
   else if (dtype == ELONG) srch->nbytes = 4;
   else if (dtype == EREAL) srch->nbytes = 4;
   }
else strcpy(srch->name,"");

/* Check the runtype. If runtype is batch and an input MTP wasn't given, abort
  ---------------------------------------------------------------------------*/
var = p_find(vblk, "$RUNTYPE");
if ((strcmp(SVAL(*var, 0), "INTERACTIVE") != 0) && (strcmp(mtp,"") == 0))
	return(IN_BATCH);

/* Check tie point mode.  If an input MTP file wasnt't given, srch & ref images
   must be.
  --------*/
if ((strcmp(mtp,"") == 0) && ((strcmp(srch->name,"") == 0) || 
                              (strcmp(ref->name,"") == 0))) return(NO_CHIPS);

/* Get correlation parameters
  --------------------------*/
var = p_find(vblk, "MINCORR");
corr->mincorr = RVAL(*var, 0);

var = p_find(vblk, "MAXDIFF");
corr->maxdiff = RVAL(*var, 0);

if(*subcmd!=PHASE_CORR)
   {
   var = p_find(vblk, "FITMETH");
   if (strcmp(SVAL(*var,  0), "PARAB") == 0) corr->fitmeth = PARAB;
   if (strcmp(SVAL(*var,  0), "GAUSS") == 0) corr->fitmeth = GAUSS;
   if (strcmp(SVAL(*var,  0), "RECIP") == 0) corr->fitmeth = RECIP;
   if (strcmp(SVAL(*var,  0), "INTEGER") == 0) corr->fitmeth = INTEGER;
   }
else corr->fitmeth = PARAB;

if(*subcmd==EDGE_CORR)
   {
   var = p_find(vblk, "EDGEVAL");
   corr->edgeval = RVAL(*var, 0);
   var = p_find(vblk, "THRVAL");
   *threshold = RVAL(*var, 0);
   }
else corr->edgeval=0.0;

if(*subcmd==PHASE_CORR)
   {
   var = p_find(vblk, "SIZE");
   *ss = IVAL(*var, 0);

   var = p_find(vblk, "STEP");
   *inc = IVAL(*var, 0);

   var = p_find(vblk, "EXPVAL");
   *alpha = RVAL(*var, 0);
   }

/* Get the confirm mode
  --------------------*/
*confirm = FALSE;
var = p_find(vblk, "CONFLG");
if (strcmp(SVAL(*var, 0), "YES") == 0) *confirm = TRUE;

return(E_SUCC);
}

/****************************************************************************
NAME:				extract

PURPOSE:  Extracts a subimage from a full image or reads a chip image

VERSION		DATE	AUTHOR
-------		----	------	
  1.0	        8/92	D. Steinwand

*****************************************************************************/
long extract
(
    struct FDESC **ptr,     /* File pointer to image file                   */
    struct CORRIMG *image,  /* Image file parameters                        */
    double *tp,		    /* Tie point (l,s)                              */
    long *size,		    /* Size of chip--nl,ns                          */
    char *name,		    /* Name of chip image file                      */
    long minval,	    /* Minimum chip size                            */
    long maxval,	    /* Maximum chip size                            */
    float *buffer	    /* Chip buffer                                  */
)
{
struct DDR ddr;		   /* DDR record */
float *temp;               /* Temporary buffer for image file read */
double icoef[6];           /* Inverse of temporary coefficients */
double corners[8];	   /* Corner projection/user-defined coordinates */
struct FDESC *chip_fdsc;   /* Chip file descriptor */
long image_coor[8];	   /* Image coordinates are all four corners */
long i,j;		   /* Loop counters */
long sl, ss;		   /* Starting line and sample of chip */
long line;		   /* Number of line being read from file */
long one = 1;		   /* Constant of one */
long four = 4;		   /* Constant of four */
long status;		   /* Return status code */
long prt_flag = FALSE;     /* Flag to print error messages in c_rtcoef */
long opt[2];		   /* Option flags for image I/O */
long nbands;		   /* Number of bands */
long nl, ns;		   /* Number of lines and samples */
long acc;		   /* File access mode */
char host[CMLEN];	   /* Host file name */

/* If processing chip subimages 
  ----------------------------*/
if (image->chip_flag)
   {

/* Get the chip file host name and open chip file
  ----------------------------------------------*/
   c_cmhost(name, host);
   opt[0] = opt[1] = 0;
   nbands = 0; nl = ns = 0;
   acc = IREAD;
   if (c_eopenr(&chip_fdsc,host,&nbands,&one,&one,&nl,&ns,&acc,&four,opt,
	&(size[0])) == E_FAIL) return(READ_ERROR);
   if ((nl != size[0]) || (ns != size[1])) return(BAD_SIZE);
   image->nl = nl;
   image->ns = ns;

/* Read the chip image
  -------------------*/
   if (c_eread(&chip_fdsc, &one, &one, (unsigned char *)buffer, &(size[0])) 
            == E_FAIL)
	return(READ_ERROR);
   image->starting_line = image->starting_sample = 1;

/* Close the chip image
  --------------------*/
   c_eclose(&chip_fdsc);

/* Get DDR info to track the chip back to a full image space
  ---------------------------------------------------------*/
   if (c_getddr(host, &ddr) != E_SUCC) return(BAD_CHIP_DDR);
   else
      {
      image->master_line = ddr.master_line;
      image->master_samp = ddr.master_sample;
   
      if (ddr.valid[6] == INVAL) return(BAD_COEFS);
      else

/* Calculate coefficients mapping from image coordinates to projection/user-
   defined space.
   -------------*/
           {
           image_coor[0] = image_coor[1] = 1;
           image_coor[2] = ddr.nl;
           image_coor[3] = 1;
           image_coor[4] = 1;
           image_coor[5] = ddr.ns;
           image_coor[6] = ddr.nl;
           image_coor[7] = ddr.ns;
           corners[0] = ddr.upleft[0];
           corners[1] = ddr.upleft[1];
           corners[2] = ddr.loleft[0];
           corners[3] = ddr.loleft[1];
           corners[4] = ddr.upright[0];
           corners[5] = ddr.upright[1];
           corners[6] = ddr.loright[0];
           corners[7] = ddr.loright[1];
   	   image->coef_y[0] = 0.0;
   	   image->coef_y[1] = 0.0;
   	   image->coef_y[2] = 1.0;
   	   image->coef_x[0] = 0.0;
   	   image->coef_x[1] = 1.0;
   	   image->coef_x[2] = 0.0;
           if(c_rtcoef(corners, image_coor, icoef, &prt_flag) != E_SUCC)
			return(BAD_COEFS);
   	   image->coef_y[0] = icoef[2];
   	   image->coef_y[1] = icoef[1];
   	   image->coef_y[2] = icoef[0];
   	   image->coef_x[0] = icoef[5];
   	   image->coef_x[1] = icoef[4];
   	   image->coef_x[2] = icoef[3];
           }
      }
   return(E_SUCC);
   }

/* Extract chips from full image
  -----------------------------*/
/* Check validity of chip size
  ---------------------------*/
if ((size[0] < minval) || (size[0] > maxval) || 
    (size[1] < minval) || (size[1] > maxval)) return(BAD_SIZE);

/* Determinvale image coordinates at which to extract chips
  -----------------------------------------------------*/
sl = (long)(tp[0] - (size[0] / 2.0) + 1.0);
ss = (long)(tp[1] - (size[1] / 2.0) + 1.0);
image->starting_line = sl;
image->starting_sample = ss;

/* Check window coordinates against image size
  -------------------------------------------*/
if((sl<1)||(ss<1)||((sl+size[0])>image->nl)||((ss+size[1])>image->ns))
	return(OUTSIDE_IMAGE);

/* Extract window from full image
  ------------------------------*/
temp = (float *)malloc(sizeof(float) * image->ns);
if (temp == NULL)
  c_lasmsg(TRUE,"correlate-alloc","Error allocating memory");

image->master_line = image->master_samp = 1;
ss--;
for (i = size[0], line = sl; i--; line++)
   {
   status = c_eread(ptr, &image->band, &line, (unsigned char *)temp, &one); 
   if (status == E_FAIL) return(READ_ERROR);
   for (j = 0; j < size[1]; j++)
      *buffer++ = temp[ss + j];
   }

free(temp);
return(E_SUCC);
}
