#ifndef _CORRELATE_H_
#define _CORRELATE_H_

#include "las.h"
#include "geompak.h"

#define STAT_FLAG status
#define CHECK(ARG) status = ARG; if (status != E_SUCC) cabort(status)

#define GREY_CORR	1
#define EDGE_CORR	2
#define PHASE_CORR	3

#define PARAB	1
#define GAUSS	2
#define RECIP	3
#define INTEGER	4

#define REF_MIN		32
#define REF_MAX	       128
#define REF_BUFSIZE  16384
#define SRCH_MIN	48 
#define SRCH_MAX       256	
#define SRCH_BUFSIZE 65536 

/* Error messages.  Additional error messages may be found in geompak.h
  --------------------------------------------------------------------*/
#define IN_BATCH	201
#define NO_CORR		202
#define CORR_METHOD	203
#define BAD_SIZE	204
#define OUTSIDE_IMAGE	205
#define READ_ERROR	206
#define BAD_COEFS	207
#define BAD_CHIP_DDR	208
#define NO_CHIPS	209
#define BAD_PTID	210

/* Structure holding correlation parameters
  ----------------------------------------*/
struct CORRPAR
   {
   double mincorr;		/* Minimum correlate value */
   double maxdiff;		/* Maximum diff between nominal & refined */
   double edgeval;		/* Fraction of pixels to be classified edges*/
   long fitmeth;		/* Method of peak fitting */
   }; 

/* Structure holding image specific parameters
  -------------------------------------------*/
struct CORRIMG
   {
   double coef_y[3], coef_x[3];	/* Coefs for pre-scaled/rotated srch chips */
   long nl;			/* Number of image lines */
   long ns;			/* Number of image samples */
   long band;			/* Band number to correlate on */
   long master_line;		/* Offset of coord to master image, if any */
   long master_samp;		/* Offset of coord to master image, if any */
   long starting_line;
   long starting_sample;
   long minval;			/* Mininim pixel value */
   long maxval;			/* Maximum pixel value */
   long nbytes;			/* Number of bytes per pixel */
   long chip_flag;		/* Boolean--are chip images used? */
   char name[CMLEN];		/* Image host name */
   };

/* Prototypes
   ----------*/
void edge_corr
(
    struct TPLDATA *tpl,        /* Tie point location data record       */
    struct CORRPAR *corr,       /* Correlation parameters               */
    float *ref,                 /* Reference subimage                   */
    float *srch,                /* Search subimage                      */
    double *nominal,            /* Nominal search image coordinates     */
    long *nscale,               /* Scale parameter                      */
    short *bsrch,               /* Binary edge search subimage          */
    short *bref,                /* Binary edge reference subimage       */
    float *edens,               /* Array of edge densities              */
    float threshold             /* Cloud cover threshold value          */
);
void eval
(
    long ncol,          /* Number of columns in coincidence-count array */
    long nrow,          /* Number of rows in coincidence-count array    */
    float *ccnorm,      /* Array of norm xcorr coeffs                   */
    float *pkval,       /* Table of top 32 normalized values            */
    long *ipkcol,       /* Table of column numbers for top 32 values    */
    long *ipkrow,       /* Table of row numbers for top 32 values       */
    float *sums,        /* Sum of all normalized values and sum of squares */
    float csmin,        /* Minimum acceptable correlation strength      */
    float *streng,      /* Strength of correlation                      */
    long *iacrej,       /* Accept/Reject code (see geompak.h)           */
    float *cpval        /* 5 by 5 array of xcorr vals in std dev        */
);
void fft2d
(
    float *data,
    long nrows,
    long ncols,
    long isign
);
void fitreg
(
    float *cpval,       /* 5 by 5 array of xcorr vals, in units of standard 
                           dev above background, centered on corr peak      */
    long mfit,          /* Method of surface fit: 1 - Elliptical paraboloid
                                                  2 - Elliptical Gaussian
                                                  3 - Reciprocal Paraboloid */
    float *pkoffs,      /* Best-fit horiz and vertical offsets of correlation 
                           peak relative to center of 5 by 5 array          */
    float *tlerrs       /* Estimated horizontal error, vertical error, and h-v 
                           cross term in best-fit offsets                   */
);
void grey_corr
(
    struct TPLDATA *tpl,        /* Tie point location data record       */
    struct CORRPAR *corr,       /* Correlation parameters               */
    float *ref,                 /* Reference subimage                   */
    float *srch,                /* Search subimage                      */
    double *nominal             /* Nominal coordinates                  */
);
void phase_corr
(
    struct TPLDATA *tpl,            /* Tie point location data record   */
    struct CORRPAR *corr,           /* Correlation parameters           */
    float *ref,                     /* Reference subimage               */
    float *srch,                    /* Search subimage                  */
    long ssize,
    long inc,
    float alpha
);

#endif /* _CORRELATE_H_ */
