/******************************************************************************
NAME:       	GTIF

PURPOSE: Read in tiff images from disk or CD and process them into bil
	 files which will be used to ingest through TRANSLAS-IN

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement
2.0	 02/00  D. Hames	  C    Modify process_band by setting outfile 
                                       to empty. This was done to eliminate
				       possible corrupt output file names.
				       SRF#3345
3.0	 03/00  D. Hames	  C    Add Arc 1960 to get_datum. CR#3454. 
				       Modify get_zone_datum and get_utmzonegen 
				       to read the geotiff keys instead of the
				       upper left corner coordinates for zone 
                                       location. Modify process_band to output 
                                       the bil file in lower case. CR#3474
4.0      04/00  D. Hames          C    Add warning messages to the functions
                                       that extract the projection information 
                                       from the geoTIFF keys. These messages
                                       will warn the user if there are any
                                       missing geoTIFF keys. Also modify these
                                       functions to change the DDR to invalid
                                       if there is required information missing
                                       from the DDR.  CR#3553
5.0      04/00	D. Hames	  C    Modify gtif and sort_bands by setting 
                                       nbands array and num_bands array to zero
                                       Set proj units to vaild in get_zone_datum
                                       get_stateplanezone CR#3657. Modify 
                                       process_band and bip2bil to support 
                                       multiband tiled images in contiguous 
                                       format. CR#3677.
6.0      09/00  D. Hames          C    Add NAD 27 to get_datum.
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "geotiffin.h"

int main 
(void)

{
   struct PARBLK vblk;          /* TAE parameter block                        */
   char btemp[MAXBND];          /* variable used to hold band from char_bands */
   char char_bands[MAXBND][CMLEN]; /* array of bands in string format         */
   char comment[50][CMLEN]; 	/* CD mount message to the operator           */
   char dpath[CMLEN]; 		/* Path to CD or disk image                   */
   char dpath_for_cd[MAX_FILE][CMLEN]; /* data path of CD 	              */
   char device[CMLEN];          /* device CD was mounted on                   */
   char files2process[MAX_FILE][CMLEN];/* geoTIFF files that will be processed*/
   char finalouts[MAX_FILE][CMLEN]; /* final output image file names          */
   char ftemp[MAXBND];         /*variable used to hold band from tiff_file_ary*/
   char drive[CMLEN];		/* drive name that operator mounted CD        */
   char newouts[MAX_FILE][CMLEN];   /* new outimg used if a band doesn't exixt*/
   char next_vol_bands[MAXBND][CMLEN];/* band array that will be used to 
                                           process bands on next CD           */
   char outfile[CMLEN]; 	/* Output BIL file for translas               */
   char outfiles[MAX_FILE][CMLEN];  /* bil files to output to tae             */
   char outimg[MAX_FILE][CMLEN];/* Names of output images	              */
   char outpath[CMLEN];		/* Path to output image and bil files	      */
   char outtiffs[MAX_FILE][CMLEN];/* geoTIFF file names               	      */
   char *ptr;			/* pointer used to point to chars in filenames*/
   char *sceneid_ptr;           /* pointer to usr_sceneid                     */
   char *string_ptrs[MAX_FILE]; /* used to return variables to TAE            */
   char temp[CMLEN];            /* temp string used to hold file name         */
   char temp2[CMLEN];		/* temp string used to hold band number       */
   char temp_dpath[CMLEN]; 	/* temp path used to retrieve geoTIFF files   */
   char tiff_file_ary[MAX_FILE][CMLEN];/* files retrieved from directory      */
   char tmpexten[5];		/* extension used for a multi resolution out  */
   char tmpstr[MAX_FILE][CMLEN];/* temporary string used to hold odtype       */
   char usr_sceneid[CMLEN];	/* sceneid supplied by user		      */
   double pixel[2];             /* resolution of the image		      */
   double resolx[MAX_FILE];     /* resolutions, one for ea bil file           */
   double resoly[MAX_FILE];     /* resolutions, one for ea bil file           */
   int  mounted;                /* flag indicating if the CD is mounted or not*/
   long bandnum[MAXBND];        /* band nums containing their true names      */
   long c=0;			/* long value used for looping                */
   long count;	 		/* total number of elements                   */
   long files2process_totnum=0; /* total number of files to be processed      */
   long group[MAX_FILE];        /* assigned grp #'s to output to tae          */
   long i=0;			/* long value used for looping                */
   long in_dtype[MAX_FILE];     /* data types to be output to tae             */
   long lan7=0;			/* flag indicating Landsat 7 data	      */
   long len;			/* used to hold the length of a string        */
   long long_bands[MAXBND];     /* bands supplied by user                     */
   long maxnum=1;		/* variable used for looping                  */
   long mode=HANDLE;	 	/* error message option			      */
   long n=0;			/* long value used for looping                */
   long n_files=0;		/* number of files retrieved from directory   */
   long nbands[MAXBND];	        /* num of bands to be copied to ea image file */
   long new_bandnum[MAXBND];    /* new bandnum used if band doesn't exist     */
   long new_nbands[MAXBND];     /* new nbands used if a band doesn't exist    */
   long next_vol_bands_totnum=0;/* total number of bands in next_vol_bands    */
   long num=0;			/* long value used for looping                */
   long num_bands[MAXBND];      /* total bands to be output to tae            */
   long num_finalouts;		/* total number of final output images        */
   long num_lines[MAX_FILE];    /* lines to be output to tae	              */
   long num_samps[MAX_FILE];    /* samples to be output to tae                */
   long numimg_files;   	/* Number of output image files               */
   long p=0; 			/* long value used for looping                */
   long q=0;			/* long value used for looping                */
   long r=0;			/* long value used for looping                */
   long rewopt=REWOFFL;    	/* rewind option		     	      */
   long s=0;			/* long value used for looping                */
   long status;		 	/* status returned by routines called	      */
   long stdlun;         	/* Logical unit number for std output         */
   long temp_array[MAX_FILE][MAX_FILE]; /* temporary array of longs           */
   long tmp_long;       	/* Dummy long to obtain lng value             */
   long tot_num_files=0;	/* Total number of files, used for looping    */
   long tot_loops=0;		/* number of times needed to loop             */
   long totals=0;	 	/* long used for tracking totals              */
   long totbands;		/* total number of bands to be processed      */
   long unit;                	/* Tape unit number                           */
   long w=0;			/* long value used for looping                */
   long zero=0;		 	/* value of zero			      */
   tsize_t image_length;	/* number of lines down in the image          */
   tsize_t image_width; 	/* number of bytes/samples across the image   */
   uint16 bitspersample;	/* number of bits per sample                  */
   uint16 numbands;             /* number of bands in the image               */

/*  Initialize TAE interface
-----------------------------*/
c_inlas(&vblk,&stdlun);

/* Set nbands array to 0 
-----------------------*/
memset(nbands, 0, MAXBND*sizeof(long));

/* Retrieve the input parameters
-------------------------------*/
getpar(&vblk,comment,dpath,outpath,long_bands,&numimg_files,nbands, 
      outimg,usr_sceneid,&totbands,&lan7);

/* If the user specified neither a COMMENT nor an INFILE, error out.
   If BOTH are specified, the CD will be read and mounted by operations.
------------------------------------------------------------------------*/
if ((strlen(dpath) == 0) && (strlen(comment[0]) == 0))
     c_lasmsg(FATAL, "geotiff-comment",
             "Either COMMENT or INFILE must be specified");


/* loop next part so if necessary a next CD can be mounted and 
   geoTIFFs processed
----------------------------------------------------------------------------*/
do     
   {
   /* Retrieve CD's from COMMENT and ask operator to mount
   -----------------------------------------------------*/
   if (strlen(comment[c]) != 0)
      {
      if(mount_cd(&vblk,&unit,comment[c],device,drive,&mounted) != TAPE_SUCC)
              return (E_FAIL);  
      /* after the CD is mounted use the device for the dpath */
      strcpy(dpath,device);
      }


   /* Retrieve list of geoTIFF files from data path supplied by user
   -----------------------------------------------------------*/

     /* if Landsat 7 data, the geoTIFF file will be named according to
     format 1 or format 2.  lan7 file naming convention is
     l7fppprrr_rrrYYYYMMDD_AAA.tif  f is ETM+ format.
     f=1 for bands 1-6l (for channel 1) and f=2 for bands 6h thru 8
     (for channel 2). Both naming conventions need to be recognized */
     if (lan7 == 1)
        {
        sceneid_ptr = usr_sceneid;
        /* landsat 7 sceneid has the length of 21 */ 
        if (strlen (sceneid_ptr) < 21)
           c_lasmsg(FATAL,"geotiffin-fatal","Invalid Landsat 7 sceneid");
        else
           sceneid_ptr[2] = '*';
        }

     /* Add sceneid to the directory path so c_getfiles can retrieve the
     list  of geoTIFF file names*/
     strcpy(temp_dpath,dpath);
     strcat(temp_dpath,usr_sceneid);
     strcat(temp_dpath,"*.tif");
     c_getfiles(temp_dpath,tiff_file_ary,&n_files,MAX_FILE,1,0);
     tot_num_files = n_files;
     strcat(dpath,usr_sceneid);
     strcat(dpath,"*.TIF");
     c_getfiles(dpath,(char (*)[256])tiff_file_ary[n_files],&n_files,
	(MAX_FILE - n_files),1,0);
     tot_num_files += n_files;
     strcpy(dpath_for_cd[0],temp_dpath);
     strcpy(dpath_for_cd[1],dpath);


   /* If the user specifies no bands, then all bands will be processed
      Copy all geoTIFF's in tiff_file_ary to files2process.
   -----------------------------------------------------------------*/
   if (totbands == 0)
      {
      for (i=0; i<tot_num_files; i++)
           {	
           strcpy(files2process[i], tiff_file_ary[i]);
           files2process_totnum++;
           } 
      }

   /* Retrieve the bands if they were specified 
   ------------------------------------------*/
   if(next_vol_bands_totnum == 0)    
      {
      if (lan7 == 1)
          { 
         /* These are the required band names for landsat7 data */
          for (i=0; i < totbands; i++)
             {
             if ((long_bands[i]<=5)||(long_bands[i]==7)||(long_bands[i]==8)) 
                {
                sprintf(char_bands[i],"%ld",long_bands[i]);
                strcat(char_bands[i],"0");
                }
             else if (long_bands[i] == 6) 
                {
                sprintf(char_bands[i],"%ld",long_bands[i]);
                strcat(char_bands[i],"1");
                }
             else if (long_bands[i] == 61)
                {
                strcpy(char_bands[i],"61");
                }  
             else if ((long_bands[i] == 9) || (long_bands[i] == 62))
                {
                strcpy(char_bands[i],"62");
                }
             } 
          } 
      else 
          {
          for (i=0; i < totbands; i++)
	     {
             sprintf(char_bands[i],"%ld",long_bands[i]);
	     }	
	  }

      /* Copy all char_bands to bandnum which will be passed to tae
      ------------------------------------------------------------*/
      for (i=0; i < totbands; i++)        
	 {
	 bandnum[i] = (long)atol(char_bands[i]);
         }
      }
   /* The information in next_vol_bands will be copied to char_bands. 
      This will be used to process geoTIFFs on next CD 
   ------------------------------------------------------------------*/
   else
      {
      for(q=0; q<MAXBND; q++)
	 {
	 strcpy(char_bands[q],"");
	 strcpy(char_bands[q],next_vol_bands[q]);
	 strcpy(next_vol_bands[q],"");
	 }
      next_vol_bands_totnum=0; 
      }	

   /* Now that char_bands array is filled with the bands the user requested, 
   compare char_bands[r] to each file in tiff_file_ary.  If they match copy 
   geoTIFF to files2process.  If not, go to next band in char_bands[r] and
   compare to each file in tiff_file_ary until all the bands requested are 
   copied to files2process. 

   If char_band contains a band that does not exist in tiff_file_ary then copy 
   that band to next_vol_bands.  This will be used after the next CD is mounted
   --------------------------------------------------------------------------*/
   i=0;
   w=0;
   r=0;
   p=0;
   strcpy(ftemp,"");
   while (r < totbands)
       {
       len = (long)strlen(char_bands[r]);
       strncpy(btemp,char_bands[r],len);
       if ((w >= tot_num_files)  && (strcmp(ftemp,btemp) != 0))
          {
          strcpy(next_vol_bands[n],char_bands[r]);
          next_vol_bands_totnum++;
          r++;
	  n++;
          i=0;
  	  w=0; 
          strncpy(btemp,char_bands[r],len);
          }	
       i=0;
       while (i < tot_num_files)
            {
            ptr = (char *) strrchr (tiff_file_ary[w],'_');
            if (ptr == NULL)
                c_lasmsg(FATAL,"geotiffin-fatal",
                       "Invalid band number in geoTIFF file name");
            else
                {  
                if (strlen(ptr) < (len+2))
                    c_lasmsg(FATAL,"geotiffin-fatal",
		       "Invalid band number in geoTIFF file name");
                else   
                    strncpy(ftemp,ptr+2,len);
                }   
	    if(strcmp(ftemp,btemp) == 0)
                {
	        strcpy(files2process[p],tiff_file_ary[i]);
                files2process_totnum++;   
                i++;
                r++; 
 	        p++;	
                w=0; 
	        break;
		} 
            else
		{
                i++;
 		w++;
		}
            }
       }

   /* Process geoTIFF files, one at a time 
   ---------------------------------*/
   for (i=0; i < files2process_totnum; i++)
        {
        process_band(files2process[i],outfile, pixel, &bitspersample,
                     &image_width,&image_length,&numbands,outpath);
        
        /* Copy the output information from process_band into arrays that
        will be passed to TAE.
        ------------------------------------------------------------------*/
        strcpy(outtiffs[num],files2process[i]);
        strcpy(outfiles[num],outfile);
        num_samps[num]=image_width;
        num_lines[num]=image_length;
        in_dtype[num]=bitspersample;
        resolx[num]=pixel[0];
        resoly[num]=pixel[1];
        num_bands[num]=numbands;
        num++;
        }

   /* count is the total number of elements that will be passed to TAE 
   ------------------------------------------------------------------*/
   count = num; 

   /* we need empty arrays in case it's necessary to loop though again 
   -----------------------------------------------------------------*/
   for(q=0; q<MAX_FILE ;q++)
      {
      strcpy(tiff_file_ary[q],"");
      strcpy(files2process[q],"");
      }

   if (mounted == 1)
      {
      /* all necessary information has been read from the individual
      geoTIFF files (ie resolution, nl, ns).  One CD needs to remain open
      to read the geoTIFF file for the common geographic information,
      which is used to update the ddr 
      ------------------------------------------------------------*/	

      /* If the user did not specify bands then all bands will be processed
      on all CD's that were requested */
      if ((totbands == 0) && (strlen(comment[c+1]) != 0))
         {
         status = c_tpclos(&unit,&mode,&rewopt,&zero);
         if (status != TAPE_SUCC)
              c_lasmsg(NONFATAL,"geotiffin-nonfatal",
                      "Error closing CD");
         mounted = 0;
         }

      /* if bands have been specified, use next_vol_bands and comment to close
      the CD, all but the last one */
      else if (totbands != 0)
	 {
         if ((next_vol_bands_totnum != 0) && (strlen(comment[c+1]) != 0))
	     {
             status = c_tpclos(&unit,&mode,&rewopt,&zero);
	     if (status != TAPE_SUCC)
                 c_lasmsg(NONFATAL,"geotiffin-nonfatal",
                      "Error closing CD");
             mounted = 0;
             }
	 }
      c++;
      }
   }
while ((next_vol_bands_totnum != 0) && (strlen(comment[c]) != 0));  
		/* end of do while statement */

/* if there are bands the user requested and they do not exist, update the
nbands, bandnum, and outimg by creating new ones. Also warn the user
of the nonexisting bands.
----------------------------------------------------------------------*/
if (next_vol_bands_totnum != 0)
   {
   nonexisting_bands(next_vol_bands,bandnum,nbands,totbands,outimg,newouts,
		     new_nbands,new_bandnum,lan7,next_vol_bands_totnum);
    
   for(i=0;i<MAXBND;i++)
      {
      nbands[i] = 0;
      nbands[i] = new_nbands[i];
      bandnum[i] = 0;
      bandnum[i] = new_bandnum[i];
      }

   for(i=0;i<MAX_FILE;i++)
      {
      strcpy(outimg[i],"");
      strcpy(outimg[i],newouts[i]);
      }
   }

/* if bands was not specified by user, we then need a list of all the 
bands. Get the band numbers from the outfiles array (which contains all 
files on all CD's or all files on disk and copy to bandnum which will
be passed through tae
--------------------------------------------------------------------*/
if (totbands == 0)
   {
   for (i=0; i < count; i++)
      {
      ptr = (char *) strrchr (outfiles[i],'/');
      if (ptr == NULL)
         c_lasmsg(FATAL,"geotiffin-fatal",
                   "Output bil file(s) do not exist");
      else
         strcpy(temp,ptr);
      /* if band number does not exit on file then leave char_bands empty */
      if (strrchr(temp, '_') != NULL)      /* indication of a band number */
         {
         ptr = (char *) strrchr (temp, '_');
         if (ptr != NULL)
            strcpy(temp2,ptr+2);
         len = (long)(strlen(temp2));	
         strncpy(char_bands[i],temp2,len-4);    /* 4 is the length of .bil */
         bandnum[i] = (long)atol(char_bands[i]);
         }
      }
   }

/* if nbands was not specified by the user, then nbands and totbands needs
to be given the total number of bands that are on CD(s) or disk 
------------------------------------------------------------------------*/
if (nbands[0] == 0)
    nbands[0] = count;
if (totbands == 0)
    totbands = count;
 
/* all bil files are processed now. Number of samples will determine the
size of the image. Separate the num_samps array into a temporary two
dimensional array. nbands will decide how the temp_array is grouped.
-------------------------------------------------------------------*/
n=0;
p=0;
q=0;
r=0;
i=0;
   while (nbands[n] != 0)
      {
      while (i < nbands[n])
	{
	temp_array[q][r] = num_samps[p];
	i++;
	r++;
	p++;
	}
      n++;
      i=0;
      q++;
      r=0;
      }

/* using the temporary array, assign a group number to each separate image 
size.  Group will determine which files will be "grouped" together and
ingested trough TRANSLAS-IN.
--------------------------------------------------------------------*/
totals=0;
r=0;
tot_loops=nbands[r];
i=1;
q=0;
s=0;
p=0;
while((temp_array[r][q]) != 0) 
  {
  do
    {
    while (q <= tot_loops)
       {       
       if ((group[q+totals] == 0) && (temp_array[r][q] != 0))
	  {
	  group[q+totals] = i;
	  break;
	  } 
       else
	  q++;
       } 
    for (p=q; tot_loops > p; p++)
       {
       if (temp_array[r][q] == temp_array[r][p+1])
	  group[p+totals+1] = group[q+totals];
       }
    q=0;
    i++;
    s++;
    }
  while(s < tot_loops);		/*end of do statement*/
  totals=nbands[r]+totals;
  r++;
  tot_loops=nbands[r];
  s=0;
  }

/* The user may not know how many image sizes are in a scene, therefore  
may not request enough outs. Since we cannot put two bands of different 
sizes (or resolutions) in the same output image, then one will be 
created using the "_r*" extension.
--------------------------------------------------------------------*/
n=0;
p=0;
q=1;
r=0;
i=0;
s=0;
num=2;
do
   {
   while(nbands[n] > i)
      {
      if (group[p] == q)
         {
         if (strlen(finalouts[s]) == 0)
             strcpy(finalouts[s],outimg[r]);
	 }
      else if(group[p] >= maxnum)
         {
         if (strlen(finalouts[s]) != 0)
            s++;	
         strcpy(finalouts[s],outimg[r]);
         sprintf(tmpexten,"_r%d",num);
         strcat(finalouts[s],tmpexten);
         if (num == 2)
	    {
	    sprintf(tmpexten,"_r%d",num-1);
	    strcat(finalouts[s-1],tmpexten);
	    }
	 num++;
	 q++;
	 }
      i++;
      if(group[p] > maxnum)
         maxnum = group[p];
      p++;
      }
   s++;
   n++;
   num=2;
   i=0;
   r++;
   q++;
   }
while(group[p] != 0);

num_finalouts = s;

/* if outtiffs[0] does not exist, then the file(s) requested by the 
user does not exist
-----------------------------------------------------------------*/
if (strlen(outtiffs[0]) == 0)
   {
   c_lasmsg(FATAL,"geotiffin-fatal","Input geoTIFF file does not exist");
   }

/* Output values to TAE
----------------------*/
if(E_SUCC != lng2tae(&vblk,&lan7,1,1,"LAN7",&tmp_long))
   c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable LAN7 to TAE");

if(E_SUCC != lng2tae(&vblk,num_samps,1,count,"NS",&tmp_long))
   c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable NS to TAE");

if(E_SUCC != lng2tae(&vblk,num_lines,1,count,"NL",&tmp_long))
   c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable NL to TAE");

if(q_real(&vblk,"RESOLX",count,resolx,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_real", "Error passing variable RESOLX to TAE");

if(q_real(&vblk,"RESOLY",count,resoly,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_real", "Error passing variable RESOLY to TAE");
 
if(E_SUCC != lng2tae(&vblk,num_bands,1,count,"NB",&tmp_long))
   c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable NB to TAE");

if(E_SUCC != lng2tae(&vblk,group,1,count,"GROUP",&tmp_long))
  c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable GROUP to TAE");

if(E_SUCC != lng2tae(&vblk,bandnum,1,totbands,"BANDNUM",&tmp_long))
  c_lasmsg(FATAL, "geotiffin-lng2tae", "Error passing variable BANDNUM to TAE");

for (i = 0; i < count; i++)
    string_ptrs[i] = outfiles[i];
if(q_string(&vblk, "BILFILE",count,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable BILFILE to TAE");

for (i = 0; i < num_finalouts; i++)
    string_ptrs[i] = finalouts[i];
if(q_string(&vblk,"FINALOUTS",num_finalouts,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable FINALOUTS to TAE");

for (i = 0; i < 2; i++)
    string_ptrs[i] = dpath_for_cd[i];
if(q_string(&vblk,"CD_DPATH",2,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable FINALOUTS to TAE");
   

for(i=0; i<count;i++)
   {
   string_ptrs[i] = tmpstr[i];
   switch (in_dtype[i])
     {
     case 8  : strcpy(tmpstr[i],"BYTE");
               break;
     case 16 : strcpy(tmpstr[i],"I*2");
               break;
     case 32 : strcpy(tmpstr[i],"I*4");
               break;
     }
   }
if(q_string(&vblk,"IDTYPE",count,string_ptrs,P_UPDATE) != SUCCESS)
  c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable IDTYPE to TAE");


for (i = 0; i < count; i++)
    string_ptrs[i] = outtiffs[i];
if(q_string(&vblk, "TIFFILE",count,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable TIFFILE to TAE"); 

    string_ptrs[0] = usr_sceneid;
if(q_string(&vblk, "SCENEID",1,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable SCENEID to TAE");
   
if(q_out(&vblk) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-qout", "Error passing variables to TAE");

return (E_SUCC);
}
