/*****************************************************************************
NAME:               EXPYRAMID

PURPOSE:  Up samples with interpolation (Used to create a Lapacian Pyramid)

VERSION     DATE    AUTHOR      CODE/CONT 
  1.0       4/92    D. Steinwand
*****************************************************************************/

#include "ezio.h"


static void filteru2
(
    unsigned char *in,      /* Input buffer */
    long ns,            /* Number of samples output */
    float *w,           /* Weight vector */
    float *out         /* Results of convolution & downsampling */
);

int main(void)
{

long in_nl,
in_ns,
in_nb;         /* Number of lines, samples, bands (input) */
long out_nl,
out_ns;        /* Number of lines, samples (output) */
long line,
i,
j;             /* Loop counters */

float a;            /* A value for kernel construction */
float *workbuf;         /* Working buffer (results of x conv) */
float w[5];
float *pm1,
*p0,
*p1,
*pm1tmp;          /* Pointers into workbuf */

unsigned char *inbuf;       /* Input buffer */
unsigned char *outbuf1,
*outbuf2;     /* Output buffers */

/* Initialize processing, images, and buffer space
-----------------------------------------------*/
start("EXPYRAMID");
init_inimg(&in_nl, &in_ns, &in_nb, 1, BYTE);
out_nl = in_nl * 2;
out_ns = in_ns * 2;
init_outimg(out_nl, out_ns, 1, BYTE);
inbuf = (unsigned char *) init_buf(in_ns, BYTE);
workbuf = (float *) init_buf(out_ns*3, FLOAT);
pm1 = workbuf;
p0 = pm1 + out_ns;
p1 = p0 + out_ns;
outbuf1 = (unsigned char *) init_buf(out_ns, BYTE);
outbuf2 = (unsigned char *) init_buf(out_ns, BYTE);
a = get_float("WEIGHT");

/* Setup convolution kernel and prime image buffers
------------------------------------------------*/
w[2] = a;
w[3] = w[1] = 0.25;
w[4] = w[0] = 0.25 - (a / 2.0);
for(i = 0; i < 5; i++)
    w[i] *= 2.0;
get_line(inbuf);
filteru2(inbuf ,out_ns, w, pm1);
get_line(inbuf);
filteru2(inbuf, out_ns, w, p0);
for(i = 0; i < in_ns; i++)
    outbuf1[i]=0;
put_line(outbuf1);  /* Write 1st line */

/* Loop thru the output image(s), line-by-line, pixel-by-pixel
-----------------------------------------------------------*/
proc_mess();
for (line=2; line<out_nl-2; line+=2)
    {
    get_line(inbuf); filteru2(inbuf,out_ns,w,p1);
    for (j=0; j<out_ns;j++)
        {
        outbuf1[j]=w[0]*pm1[j]; outbuf1[j]+=w[2]*p0[j]; outbuf1[j]+=w[4]*p1[j];
        outbuf2[j]=w[1]*p0[j]; outbuf2[j]+= w[3]*p1[j];
        }
    put_line(outbuf1);
    put_line(outbuf2);   /* Write 2 lines of data to disk */
    pm1tmp = pm1;
    pm1 = p0;
    p0 = p1;
    p1 = pm1tmp;
    }
for(i = 0; i < out_ns; i++) outbuf1[i] = 0;
put_line(outbuf1);
put_line(outbuf1);
put_line(outbuf1);
finish(0);
return 0;
}

/* Function to apply 5 point convolution in the x dimension and upsample
   results by two.
  ---------------*/
static void filteru2
(
    unsigned char *in,      /* Input buffer */
    long ns,            /* Number of samples output */
    float *w,           /* Weight vector */
    float *out         /* Results of convolution & downsampling */
)
{
long i,j;

out[0] = 0.0;
for(j = 1; j < (ns - 2); j++)
    {
    for(out[j] = 0.0, i = 1; i < 5; i += 2)
        out[j] += w[i] * in[((j + i) - 2) / 2];
    j++;
    for(out[j] = 0.0, i=0; i<5; i+=2)
        out[j] += w[i] * in[((j + i)- 2) / 2];
    }
out[ns-2] = out[ns-1]=0.0;
}
