/*****************************************************************************
NAME:     GPYRAMID

PURPOSE:  Down samples an image by two (Used to create a Gaussian Pyramid)

VERSION     DATE    AUTHOR      CODE/CONT 
-------     ----    ------      ---------
  1.0       4/92    D. Steinwand

*****************************************************************************/

#include "ezio.h"

static void filterd2
(
    unsigned char *in,      /* Input buffer */
    long ns,            /* Number of samples output */
    float *w,           /* Weight vector */
    float *out         /* Results of convolution & downsampling */
);


int main(void)
{

long in_nl,
in_ns,
in_nb;         /* Number of lines, samples, bands (input) */
long out_nl,
out_ns;        /* Number of lines, samples (output) */
long line,
i;             /* Loop counters */

float weightscale;      /* A value for kernel construction */
float *workbuf;         /* Working buffer (results of x conv) */
float w[5];

unsigned char *inbuf,       /* Input Buffer */
*outbuf;      /* Output buffer */

float *pm2,
*pm1,
*p0,
*p1,
*p2,
*pm2tmp,
*pm1tmp;            /* Pointers into workbuf */

/* Initialize processing, images, and buffer space
-----------------------------------------------*/
start("GPYRAMID");
init_inimg(&in_nl, &in_ns, &in_nb, 1, BYTE);
out_nl = in_nl / 2;
out_ns = in_ns / 2;
init_outimg(out_nl, out_ns, 1, BYTE);
inbuf = (unsigned char *) init_buf(in_ns, BYTE);
workbuf = (float *) init_buf(out_ns*5, FLOAT);
pm2 = workbuf;
pm1 = pm2 + out_ns;
p0 = pm1 + out_ns;
p1 = p0 + out_ns;
p2 = p1 + out_ns;
outbuf = (unsigned char *) init_buf(out_ns, BYTE);
weightscale = get_float("WEIGHT");

/* Setup convolution kernel and prime image buffers
------------------------------------------------*/
w[2] = weightscale;
w[3] = w[1] = 0.25;
w[4] = w[0] = 0.25 - weightscale / 2.0;
for(i = 0; i < out_ns; i++)
    outbuf[i]=0;
get_line(inbuf);
filterd2(inbuf, out_ns, w, pm2);
get_line(inbuf);
filterd2(inbuf, out_ns, w, pm1);
get_line(inbuf);
filterd2(inbuf, out_ns, w, p0);
put_line(outbuf);           /* Write 1st line of zeros */

/* Loop thru the output image(s), line-by-line, pixel-by-pixel
-----------------------------------------------------------*/
proc_mess();
for (line=0; line<out_nl-2; line++)
    {
    get_line(inbuf);
    filterd2(inbuf, out_ns, w, p1);
    get_line(inbuf);
    filterd2(inbuf, out_ns, w, p2);
    for (i=0; i < out_ns; i++)
        outbuf[i] = w[0] * pm2[i] + w[1] * pm1[i] + w[2] * p0[i]
        + w[3] * p1[i] + w[4] * p2[i];
    put_line(outbuf);           /* Write line of data to disk */
    pm2tmp = pm2;
    pm1tmp = pm1;
    pm2 = p0;
    pm1 = p1;
    p0 = p2;
    p1 = pm2tmp;
    p2 = pm1tmp;
    }
for (i = 0; i < out_ns; i++)
    outbuf [i] = 0;
put_line(outbuf);           /* Write last line of zeros */
finish(1);               /* Close down processing and exit */
return 0;
}

/* Function to apply 5 point convolution in the x dimension and downsample
   results by two.
  ---------------*/
static void filterd2
(
    unsigned char *in,      /* Input buffer */
    long ns,            /* Number of samples output */
    float *w,           /* Weight vector */
    float *out         /* Results of convolution & downsampling */
)
{
long i,j;

out[0] = 0.0;
for(i = 1, j = 2; i < (ns - 1); j += 2, i++)
    out[i] = w[0] * in[j-2] + w[1] * in[j-1] + w[2] * in[j] + w[3]
    * in [j+1] +w[4] * in[j+2];
out[ns-1]=0.0;
}
