/******************************************************************************
FUNCTION:	CDSKIP_BSQ

PURPOSE:	Skip to the correct spot in disk file (using standard C file
		I/O functions)

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ltwgin.h"

extern long cdflag;	  	/* flag for disk image input		*/
extern char sceneid[SCENELEN]; 	/* naming convention for NLAPS files	*/ 

int cdskip_bsq
(
  long *window,                /* Window of image to be copied */
  long *cb,                    /* Current band being processed */
  long *dmod,                  /* Part of curblk needed for the window */
  long *blkfct,                /* Blocking factor */
  FILE *infp,                  /* Input file */
  long reclength,              /* Length of image descrip. record */
  long *recsize,               /* Length of image data records */
  char hpath[CMLEN]            /* Path to image data on disk */
)
{
char band[3];		/* character string representing band number 	 */
char filename[11];	/* name of new image file to open 		 */
char imgfile[CMLEN];	/* full path to image file on disk 		 */
long nrec;		/* number of records to skip 			 */
long status;            /* holds result of fseek();  			 */

*dmod = 0;

/* Create the proper filename to open. BSQ image files on CD use the filename
   convention img_xx.001, where xx is the band number the data represents,
   i.e., if current band == 2, the corresponding filename is img_02.001.
   Also, NLAPS uses sceneid.ix 
-----------------------------------------------------------------------------*/

if (cdflag == NLAPS)
    {
    strncpy(filename, sceneid, 8);
    filename[8] = '\0';
    sprintf(band, "%1ld", *cb);
    strcat(filename, ".i");
    strcat(filename, band); 
    }  
else
    {
    sprintf(band, "%2ld", *cb);
    band[2] = '\0';
    if (*cb < 10)
	band[0] = '0';			/* (long) 1 == "01" */
    sprintf(filename, "img_%s.001", band);
    }

strcpy(imgfile, hpath);
strcat(imgfile, filename);

/* Open the image file
----------------------*/

infp = fopen(imgfile, "r");
if (infp == NULL)
    return (E_FAIL);


/* Skip the image descriptor record
-----------------------------------*/

status = fseek(infp, reclength, 0); /* can't use ANSI C #defines here */ 
if (status != 0)
    return(E_FAIL);


/* Skip records to starting line of the specified window
--------------------------------------------------------*/

if (window[SL] > 1)
    {
    nrec = (window[SL] - 1) / *blkfct;
    *dmod = (window[SL] - 1) % *blkfct;
    status = fseek(infp, nrec * (*recsize), 1);
    if (status != 0)
	return(E_FAIL);
    }
     
return(E_SUCC);
}
