/******************************************************************************

FUNCTION:	det_ds

PURPOSE:	Determines the directory structure of image data on a CD.

ALGORITHM REFERENCES:	none

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "cm.h"
#include "ltwgin.h"

void det_ds
(
  char hpath[CMLEN],            /* Qualified path to image on CD */
  long *ds,                     /* Denotes type of structure */
  char sceneid[SCENELEN]        /* Naming convention for NLAPS files */
)
{
char filename[10][CMLEN];/* Array of filenames extracted		*/
char path[CMLEN];	/* Path specification for call to c_getfiles()	*/
long nfiles;		/* Number of filenames returned			*/
long i;			/* Loop counter					*/
long pathlength;	/* Length of original path			*/

pathlength = (long)strlen(hpath);	

strcpy(path, hpath);
strcat(path, "*");

c_getfiles(path, filename, &nfiles, 10, 1, 0);

/*----------------------------------------------------------------------------
   Check for South Africa MSS data.  This data is stored using the directory
   structure as follows:

   scene1/img_01.001
	  img_02.001
	  img_03.001
	  vdf_dat.001
	  tra_01.001
	  lea_01.001
	     .
	     .
	     .

   In this case, the function will key off the presence or absence of a
   file which begins with "vdf_dat".
----------------------------------------------------------------------------*/

for (i = 0; i < nfiles; i++)
  if (strncmp ((filename[i] + pathlength * sizeof(char)), "vdf_dat", 7) == 0)
    *ds = SAFRICAMSS; 

/*----------------------------------------------------------------------------
   Check for SPOT BIL data.  This data is found in the form:
  
   scene01/imag_01.dat
	   lead_01.dat
	   trai_01.dat
	   vold_01.dat
	      .
	      .
	      .

   The filename "vold*" will be used as the key.
-----------------------------------------------------------------------------*/

for (i = 0; i < nfiles; i++)
  if (strncmp ((filename[i] + pathlength * sizeof(char)), "vold", 4) == 0)
    *ds = SPOTBIL;

/*----------------------------------------------------------------------------
   Check for NLAPS data.  This data is found in the form:
   
   sceneid.i1
   sceneid.l1
   sceneid.t1
   sceneid.sv
   sceneid.sf
      .
      .
      .

   The filename extension ".i" will be used as the key.
-----------------------------------------------------------------------------*/
for (i = 0; i < nfiles; i++)
  if (strncmp ((filename[i] + pathlength + SCENELEN), ".i", 2) == 0)
  {
    *ds = NLAPS;
    strncpy(sceneid,(filename[i]+pathlength*sizeof(char)),SCENELEN);
  } 	
}
