/******************************************************************************

FUNCTION:       GETENF

PURPOSE:        Reads in the end files (trailer and supplemental) from tapes 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "labserv.h"
#include "ltwgin.h"

extern long cdflag;             /* flag for disk image input            */
extern char sceneid[SCENELEN];  /* naming convention for NLAPS files    */

int getenf
(
   char hosout[][CMLEN],         /* Ouput host names */
   long *unit,                   /* Tape unit number */
   long *wrtdsk,                 /* Ancillary files to read in */
   long *num_out,                /* Number of output files */
   char comment[][CMLEN],        /* Mount message to the operator */
   long *num_trailrec,           /* Number of trailer records */
   long *tapenum,                /* Current tape mounted */
   long *trailsiz,               /* Record size of the trailer file */
   long *numtapes,               /* Number of tapes specified */
   long *no_lead,                /* Flag if there is a leader file on tape
                                    There is not a leader file if an
                                    image band is split across two tapes */
   char *satnam,                 /* Satellite name */
   char hpath[CMLEN],            /* Host name of input file/directory */
   long ceosid                   /* Ground station identifier */
)
{
   FILE *fp=NULL;          /* file descriptor                           */
   FILE *infp=NULL;        /* input file for CD or disk                 */
   
   long  i;                /* loop counter                              */
   long  dumlong;          /* dummy value                               */
   long  dum2 = FALSE;     /* dummy value                               */
   long  mode = HANDLE;    /* tape I/O error flag                       */
   long  numrecs;          /* number of records in the file             */
   long  one = 1;          /* constant of one                           */
   long  recsize;          /* record size of the file                   */
   long  status;           /* return status                             */
   long  skip;             /* number of files to skip                   */
   long nfiles;            /* number of filenames returned              */

   unsigned char *in_buf;  /* buffer read from image on tape            */
   char tempbuf[10];       /* temp work buffer                          */
   char ttype[CMLEN];      /* tape type                                 */
   char trafile[CMLEN];    /* formal name of trailer file (on disk)     */
   char scenenum[3];       /* string representation of image number     */
   char tpath[CMLEN];      /* temporary host file path for trailer file */ 
   char filename[10][CMLEN]; /* array of filenames returned             */ 
   char supfile[CMLEN];    /* supplemental filename                     */

   /* Mount the last tape if not already mounted.  Some tape sets 
      have these files at the end of each tape, but it is not consistent.
   ----------------------------------------------------------------------*/
   if (!cdflag)
   {
      if (*tapenum != *numtapes)
      {
         close_tape(unit);
         if (mount_tape(unit, comment, *numtapes, numtapes,ttype) != E_SUCC)
            return(E_FAIL);

         /* Read the volume descriptor file
         --------------------------------*/
         if (getvdf(unit,&dum2,hosout,&dumlong,&dumlong,&dumlong,&dumlong,
                    tempbuf,&dumlong,&dumlong,&dumlong,hpath,&ceosid) != E_SUCC)
            return(E_FAIL);

         /* Skip the leader file and/or the image date portion 
         ----------------------------------------------------*/
         if (*no_lead)
            skip = 1;
         else
            skip = 2;
         if (c_tpfsf(unit,&skip,&mode) != TAPE_SUCC)
            return(E_FAIL);
      }
   }

   /* Write the trailer file to disk
   ----------------------------------*/
   if (wrtdsk[TRAI])
   {
      /* Allocate memory for the input buffer
      ---------------------------------------*/
      in_buf = (unsigned char *) malloc(*trailsiz);
      if (in_buf == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
         return(E_FAIL);
      }

      /* Create the file name
      -----------------------*/
      c_lsmknm(hosout[0],"trai",hosout[*num_out]);
      fp = fopen(hosout[(*num_out)++],"w");
      if (fp == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-write", 
                  "Error opening trailer file for writing");
         free(in_buf);
         return(E_FAIL);
      }

      /* If input image is on CD, open the trailer file
      -------------------------------------------------*/
      if (cdflag)
      {
         strcpy(trafile, hpath);
         if (cdflag == SAFRICAMSS)
            strcat(trafile, "tra_01.001");
         if (cdflag == SPOTBIL)
         {
            if(imgnum(hpath, scenenum) != E_SUCC)
            {
               c_lasmsg(FALSE, "ltwgin-imgnum", 
                        "Error retrieving the image number from CD.");
               free(in_buf);
               return(E_FAIL);
            }
            strcat(trafile, "trai_");
            strcat(trafile, scenenum);
            strcat(trafile, ".dat");
         }
         if (cdflag == NLAPS)
         {
            strcpy(tpath,hpath);
            strcat(tpath,"*.t*");
            c_getfiles(tpath,filename,&nfiles,10,1,10);
            if (nfiles > 0)
               strcpy(trafile, filename[0]);
         }

         infp = fopen(trafile, "r");
         if (infp == NULL)
         {
            c_lasmsg(FALSE, "ltwgin-diskread", 
                     "Error reading trailer file from disk");
            free(in_buf);
            return(E_FAIL);
         }
      }

      /* Read and write the records 
      -----------------------------*/
      for (i = 0; i < *num_trailrec; i++)
      {
         if (cdflag)
         {
            if (fread(in_buf, 1, *trailsiz, infp) != (*trailsiz))
            {
               c_lasmsg(FALSE, "ltwgin-diskread",
                        "Error reading trailer file from disk");
               free(in_buf);
               return(E_FAIL);
            }
         } 
         else
         {
            status = c_tpread(unit,in_buf,trailsiz,&mode);
            if (status != *trailsiz)
            {
               c_lasmsg(FALSE, "ltwgin-read",
                        "Error reading trailer file from tape");
               free(in_buf);
               return(E_FAIL);
            }
         }
         if(fwrite(in_buf,1,*trailsiz,fp) != *trailsiz)
         {
            c_lasmsg(FALSE,"ltwgin-write","Error writing to the trialer file");
            free(in_buf);
            return(E_FAIL);
         }
      }

      /* Close the file
      -----------------*/
      if(fclose(fp) != 0)
      {
         c_lasmsg(FALSE,"ltwgin-write","Error closing the trailer file");
         free(in_buf);
         return(E_FAIL);
      }

      /* One more read to read EOF
      ----------------------------*/
      if (!cdflag)
      { 
         if (c_tpread(unit,in_buf,trailsiz,&mode) != TAPE_EOF)
         {
            c_lasmsg(FALSE, "ltwgin-read",
                     "Error reading trailer file from tape");
            free(in_buf);
            return(E_FAIL);
         }
      }

      /* Free allocated space and close infp if needed
      ------------------------------------------------*/
      free(in_buf);
      if (cdflag)
         fclose(infp);
   }
   else
   {
      /* Skip the file completely
      ---------------------------*/
      if (!cdflag)
      {
         if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
         {
            free(in_buf);
            return(E_FAIL);
         }
      }
   }

   /* Spot data does not have a supplemental file
   -----------------------------------------------*/
   if (wrtdsk[SUPP] && (strncmp(satnam,"SP",2) == 0))
   {
      c_lasmsg(FALSE, "ltwgin-supp", "No supplemental file");
      return(E_SUCC);
   }
   else if (strncmp(satnam,"SP",2) == 0)
      return(E_SUCC);

   /* Japan, South African CD, and Canada data has no supplemental file
   --------------------------------------------------------------------*/
   if ((JAPAN) || ((cdflag) && (ceosid == HBK)) || (ceosid == CAN))
   {
      if (wrtdsk[SUPP])
         c_lasmsg(FALSE, "ltwgin-supp", "No supplemental file");
      return(E_SUCC);
   }

   /* Read the volume directory file
   ---------------------------------*/
   if (getvdf(unit,&dum2,hosout,&dumlong,&dumlong,&dumlong,&dumlong,tempbuf,
              &numrecs,&recsize,&dumlong,hpath,&ceosid) != E_SUCC)
      return(E_FAIL);

   /* Write the supplemental file to disk
   -------------------------------------*/
   if (wrtdsk[SUPP])
   {
      /* Allocate memory for the input buffer
      ---------------------------------------*/
      in_buf = (unsigned char *) malloc(recsize);
      if (in_buf == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-alloc",
                  "Error allocating dynamic memory");
         return(E_FAIL);
      }

      /* Create the file name
      -----------------------*/
      c_lsmknm(hosout[0],"supp",hosout[*num_out]);
      fp = fopen(hosout[(*num_out)++],"w");
      if(fp == NULL)
      {
         c_lasmsg(FALSE,"ltwgin-write",
                  "Error opening the supplemental file for writing");
         free(in_buf);
         return(E_FAIL);
      }

      /* If NLAPS copy supplemental file 
      ----------------------------------*/
      if (cdflag == NLAPS) 
      {
         strcpy(supfile, hpath);
         strncat(supfile, sceneid, 8);
         strcat(supfile, ".sf");
         infp = fopen(supfile, "r");
         if (infp == 0)
         {
            c_lasmsg(FALSE, "ltwgin-diskread",
                     "Error reading SUPP file from disk");
            free(in_buf);
            return(E_FAIL);
         }
      }


      /* Read and write the records.  These are variable length
         so don't check status from tpread
      --------------------------------------------------------*/
      for (i = 0; i < numrecs; i++)
      {
         if (cdflag == NLAPS)
         {
            if(fread(in_buf, 1, recsize, infp) != recsize)
            {
               c_lasmsg(FALSE, "ltwgin-diskread",
                        "Error reading supplemental file from disk");
               free(in_buf);
               return(E_FAIL);
            }
            if(fwrite(in_buf,1,recsize,fp) != recsize)
            {
               c_lasmsg(FALSE,"ltwgin-write",
                        "Error writing to the trialer file");
               free(in_buf);
               return(E_FAIL);
            }
         }
         else
         {
            status = c_tpread(unit,in_buf,&recsize,&mode);
            if (status == TAPE_EOF)
               break;
            if(fwrite(in_buf,1,status,fp) != status)
            {
               c_lasmsg(FALSE,"ltwgin-write",
                        "Error writing to the trialer file");
               free(in_buf);
               return(E_FAIL);
            }
         }
      }
      /* Close the file
      -----------------*/
      if(fclose(fp) != 0)
      {
         c_lasmsg(FALSE,"ltwgin-write","Error closing the supplemental file");
         free(in_buf);
         return(E_FAIL);
      }

      /* Free allocated space
      -----------------------*/
      free(in_buf);
   }

   return(E_SUCC);
}
