#ifndef LTWGIN_H
#define LTWGIN_H

/* Defines for LTWGIN
---------------------*/

#define MAX_BAND 7    /* Maximum number of bands on a LTWG tape currently
                         handled 	*/
/*  will need to increase sceneid length here as well as getenf(line 230)
and getvdf(line 85) */ 
#define SCENELEN 8    /* Length of characters needed for sceneid	*/

/* Values for associated files */
#define VDF 0         
#define LEAD 1
#define TRAI 2
#define SUPP 3
#define IDR  4

/* Values for individual CD directory structures */
#define SAFRICAMSS 1
#define SPOTBIL 2
#define NLAPS 3

/* Values for individual ground stations */
#define HBK 1
#define UNK 2
#define TK1 3
#define CAN 4
#define THAI 5
#define SAUDI 6

/* Temporary #defines for processing nonstandard data */
#define JAPAN ceosid == TK1


/* Prototypes for the functions in ltwgin
   NOTE: las.h must be included ahead of ltwgin.h
-------------------------------------------------*/

/* calcorn.c */
void calcorn
(
  double clat_l,                /* Center point latitude */
  double clon_l,                /* Center point longitude */
  char *rectec,                 /* Recording technique used */
  double *rtnlat,               /* Calculated corner point latitudes */
  double *rtnlon,               /* Calculated corner point longitudes */
  long sat                      /* Satellite number */
);

/* cdskip_bsq.c */
int cdskip_bsq
(
  long *window,                /* Window of image to be copied */
  long *cb,                    /* Current band being processed */
  long *dmod,                  /* Part of curblk needed for the window */
  long *blkfct,                /* Blocking factor */
  FILE *infp,                  /* Input file */
  long reclength,              /* Length of image descrip. record */
  long *recsize,               /* Length of image data records */
  char hpath[CMLEN]            /* Path to image data on disk */
);

/* check.c */
void chk_window
(
  long *outspc,                /* Output image file specs */
  long *window                 /* Window of image to be copied */
);

/* Line       67 */
void chk_bands
(
  long *bands,                /* Bands to be processed within image */
  long *numbands,             /* Input image file specs */
  long *totbnd                /* Total number of bands to copy */
);

/* det_ds.c */
void det_ds
(
  char hpath[CMLEN],            /* Qualified path to image on CD */
  long *ds,                     /* Denotes type of structure */
  char sceneid[SCENELEN]        /* Naming convention for NLAPS files */
);

/* get_ceosid.c */
int get_ceosid
(
  char idstring[32]             /* Extracted from image VDF file */
);

/* getenf.c */
int getenf
(
  char hosout[][CMLEN],         /* Ouput host names */
  long *unit,                   /* Tape unit number */
  long *wrtdsk,                 /* Ancillary files to read in */
  long *num_out,                /* Number of output files */
  char comment[][CMLEN],        /* Mount message to the operator */
  long *num_trailrec,           /* Number of trailer records */
  long *tapenum,                /* Current tape mounted */
  long *trailsiz,               /* Record size of the trailer file */
  long *numtapes,               /* Number of tapes specified */
  long *no_lead,                /* Flag if there is a leader file on tape
                                     There is not a leader file if an
                                     image band is split across two tapes */
  char *satnam,                 /* Satellite name */
  char hpath[CMLEN],            /* Host name of input file/directory */
  long ceosid                   /* Ground station identifier */
);

/* getlead.c */
int getlead
(
  long *unit,                    /* Tape unit number */
  long *wrtdsk,                  /* Ancillary files to read in */
  char hosout[][CMLEN],          /* Output host names */
  long *num_out,                 /* Number of output files */
  long *numlead,                 /* Number of leader records */
  long *leadsize,                /* Size of leader records */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Scene path */
  char *direction,               /* Satellite direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Image pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Sateelite sensor */
  double *angle,                 /* Angle of rotation in radians */
  long *datum_code,              /* Datum of the dataset */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char *data_type,               /* Type of data (corrected or uncorrected) */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  char hpath[CMLEN],             /* Path to image data on disk */
  long ceosid,                   /* Id number of receiving station */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8]           /* Line/sample corners */
);

/* getpar.c */
void getpar
(
  struct PARBLK *vblock,             /* TAE parameter block */
  long *bands,                       /* Bands to be processed in image */
  long *msgtim,                      /* Time between display message */
  long *window,                      /* Window of input image */
  long *minmax,                      /* Min/Max flag */
  char hosout[][CMLEN],              /* Output host name */
  char taeout[][CMLEN],              /* Output TAE name */
  char comment[][CMLEN],             /* Mount message to the operator */
  long *wrtdsk,                      /* Ancillary files to read in */
  long *numtapes,                    /* Number of tapes specified */
  long *numimg_files,                /* Number of output files */
  char dpath[CMLEN]                  /* TAE input filename */
);

/* getvdf.c */
int getvdf
(
  long *unit,                     /* Tape unit number */
  long *wrtdsk,                   /* Ancillary files to read in */
  char hosout[][CMLEN],           /* Output host names */
  long *num_out,                  /* Number of output files */
  long *recsize,                  /* Image record length */
  long *numimg_recs,              /* Number of image records on this tape */
  long *num_trailrec,             /* Number of trailer records */
  char *order,                    /* Band order */
  long *numlead,                  /* Number of leader files */
  long *leadsize,                 /* Size of leader records */
  long *trailsiz,                 /* Max size of trailer records */
  char hpath[CMLEN],              /* Host path to image data */
  long *ceosid                    /* Code for receiving ground station */
);

/* imgnum.c */
int imgnum
(
  char hpath[CMLEN],             /* Path to image data */
  char img[3]                    /* Image number */
);

/* ls_proj.c */
void ls_proj
(
  double *angle,                 /* Display rotation angle */
  double fcproj[],               /* Full scene center projection coords */
  double fcenls[],               /* Full scene center line/sample coords */
  double proj[],                 /* Projection coordinates calcuated */
  long linsamp[],                /* Line/sample values calculated */
  double *pixsiz                 /* Pixel size in meters */
);

/* proces_bil.c */
int proces_bil
(
  char hosout[][CMLEN],         /* Output host names */
  long *window,                 /* Window of image to be copied */
  long *totbnd,                 /* Total number of bands to be copied */
  long *unit,                   /* Tape unit number */
  long *wrtdsk,                 /* Ancillary files to read in */
  long num_out,                 /* Number of output files */
  long numimg_files,            /* Number of output image files */
  long *bands,                  /* Bands to be processed within image */
  long *msgtim,                 /* Time between display messages */
  long *minmax,                 /* Min/Max flag */
  char comment[][CMLEN],        /* Mount message to the operator */
  long *recsize,                /* Image record length */
  long *numimg_recs,            /* Number of image records on this tape */
  long *tapenum,                /* Current tape number */
  long *numtapes,               /* Number of tapes specified */
  long *no_lead,                /* Flag if sccessive tapes have leader file */
  char ttype[],
  char hpath[CMLEN],            /* Path to image data on disk */
  char *sensor,                 /* Satellite sensor */
  long ceosid
);

/* proces_bsq.c */
int proces_bsq
(
  char hosout[][CMLEN],        /* Output host names */
  long *window,                /* Window of image to be copied */
  long *totbnd,                /* Total number of bands to be copied */
  long *unit,                  /* Tape unit number */
  long *wrtdsk,                /* Ancillary files to read in */
  long *num_out,               /* Number of output files */
  long *numimg_files,          /* Number of output image files */
  long *bands,                 /* Bands to be processed within image */
  long *msgtim,                /* Time between display messages */
  long *minmax,                /* Min/Max flag */
  char comment[][CMLEN],       /* Mount message to the operator */
  long *recsize,               /* Image record length */
  long *tapenum,               /* Tape number mounted */
  long *numtapes,              /* Number of tapes specified */
  long *no_lead,               /* Flag if successive tapes have leader file */
  char ttype[],                /* Tape type */
  char hpath[CMLEN],           /* Path to image file on disk */
  char *sensor,                /* Satellite sensor */
  long ceosid                  /* Code for ground station */
);

/* skip_bil.c */
int skip_bil
(
  long *window,                /* Window of image to be copied */
  long *unit,                  /* Tape unit number */
  char comment[][CMLEN],       /* Mount message to the operator */
  long *numimg_recs,           /* Number of image records on this tape */
  long *tapenum,               /* Current tape number */
  long *numbands,              /* Number of bands taken from header record */
  long *numtapes,              /* Number of tapes specified */
  char hpath[CMLEN]            /* Path to image data on disk / CD */
);

/* skip_bsq.c */
int skip_bsq
(
  long *window,             /* Window of image to be copied */
  long *unit,               /* Tape unit number */
  char comment[][CMLEN],    /* Mount message to the operator */
  long *tapenum,            /* Current tape number mounted */
  long *numtapes,           /* Number of tape specified */
  long *cb,                 /* Current tape band being processed */
  long *dmod,               /* Part of curblk needed for the window */
  long *blkfct,             /* Blocking factor */
  long *ptrband,            /* Pointer to the band the tape is currently at */
  long *swreel              /* Flag if need to switch reels */
);

/* tape.c */
int mount_tape
(
  long *unit,                  /* Tape unit number */
  char comment[][CMLEN],       /* Mount message to the operator */
  long tapenum, 
  long *numtapes, 
  char ttype[]                 /* Tape type */
);

/* Line       67 */
void close_tape
(
  long *unit              /* Tape unit number */
);

/* update_ddr.c */
int update_ddr
(
  char hosout[][CMLEN],          /* Host file name */
  long *window,                  /* Input window */
  long *totbnd,                  /* Total number of bands to process */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Path number */
  char *direction,               /* Sat direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Sensor type */
  char data_type,                /* Type of data (corrected/uncorrected) */
  double *angle,                 /* Rotation angle */
  long *datum,                   /* Datum of the data */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8],          /* Line/sample corners */
  long *numimg_files,            /* Number of output image files */
  long ceosid                    /* ID number of ground station */
);

#endif
