/******************************************************************************
FUNCTION:       SKIP_BIL

PURPOSE:        Skip to the correct spot on BIL tapes.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwgin.h"

int skip_bil
(
  long *window,                /* Window of image to be copied */
  long *unit,                  /* Tape unit number */
  char comment[][CMLEN],       /* Mount message to the operator */
  long *numimg_recs,           /* Number of image records on this tape */
  long *tapenum,               /* Current tape number */
  long *numbands,              /* Number of bands taken from header record */
  long *numtapes,              /* Number of tapes specified */
  char hpath[CMLEN]            /* Path to image data on disk / CD */
)
{
   long  recsize;          /* image record length                       */
   long  mode = HANDLE;    /* tape I/O error flag                       */
   long  nrec;             /* number of records to skip                 */
   long  status;           /* return status                             */
   long  totrec;           /* total number of records to skip           */
   long  dumlong;          /* dummy value                               */
   long  dum2 = FALSE;     /* dummy value                               */
   
   char tempbuf[10];       /* temp work buffer                          */
   char tempbuf2[3][CMLEN];/* temp work buffer                          */
   char ttype[CMLEN];

   /* Skip records on tape to starting line of the window.
      Mount next tape as needed.
   ------------------------------------------------------*/
   if (window[SL] > 1)
   {
      totrec = nrec = (window[SL] - 1) * *numbands;
      if (totrec > *numimg_recs )
      {
         /* Close current tape, mount next tape, and read
            volume directory file
         -----------------------------------------------*/
         nrec = totrec - *numimg_recs;
         close_tape(unit);
         if (mount_tape(unit, comment, ++(*tapenum), numtapes,ttype) != E_SUCC)
            return(E_FAIL);

         if (getvdf(unit, &dum2, tempbuf2, &dumlong, &recsize, numimg_recs,
                    &dumlong, tempbuf, &dumlong, &dumlong, &dumlong,
                    hpath, &dumlong) != E_SUCC)
            return(E_FAIL);

         if (totrec > *numimg_recs)
         {
            /* Close current tape, mount next tape, and read
               volume directory file
            -----------------------------------------------*/
            nrec = totrec - *numimg_recs;
            close_tape(unit);
            if (mount_tape(unit, comment, ++(*tapenum), numtapes,ttype)
                != E_SUCC)
               return(E_FAIL);

            if (getvdf(unit, &dum2, tempbuf2, &dumlong, &recsize, numimg_recs, 
                      &dumlong, tempbuf, &dumlong, &dumlong, &dumlong,
                      hpath, &dumlong) != E_SUCC)
               return(E_FAIL);
         }
      }
   
      status = c_tpfsr(unit,&nrec,&mode);
      if (status != TAPE_SUCC)
         return(E_FAIL);
   }

   return(E_SUCC);
}
