/*******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "ddr.h"
#include "geompak.h"
#include "proj.h"
#include "tm.h"
#include "ltwgin.h"

int update_ddr
(
  char hosout[][CMLEN],          /* Host file name */
  long *window,                  /* Input window */
  long *totbnd,                  /* Total number of bands to process */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Path number */
  char *direction,               /* Sat direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Sensor type */
  char data_type,                /* Type of data (corrected/uncorrected) */
  double *angle,                 /* Rotation angle */
  long *datum,                   /* Datum of the data */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8],          /* Line/sample corners */
  long *numimg_files,            /* Number of output image files */
  long ceosid                    /* ID number of ground station */
)
{
struct FDESC *filedesc;    /* image file descriptors used for image I/O */

struct DDR ddr; 	/* DDR structure			*/
struct BDDR bddr[7];	/* Band DDR structure			*/
double out_corners[8];	/* output corners from c_file_corner	*/

unsigned char *buff;    /* Temporary buffer for image file read */

long access = IREAD;    /* read access to output image          */
long idtype = EBYTE;    /* the only data type handled by process functions */
long band = 1;		/* Band looping variable		*/
long line;		/* Line where center is located         */
long option[2];         /* output option array                  */
long inunit = 4;	/* input unit - deg			*/
long outunit = 2;	/* output unit - meters			*/
long inzone = 62;	/* input zone				*/
long outzone = 62;	/* output zone				*/
long indatum = 0;	/* input datum				*/
long outdatum = 0; 	/* output datum				*/
long line_samp[2];	/* line/sample pair			*/
long i;			/* loop counter				*/
long fill=0;		/* fill counter				*/
long inproj;		/* input projection			*/
long prtprm[2];         /* report to term and/or file           */
long one = 1;		/* constant of one			*/
long multfact;		/* sign value				*/
long status;		/* return status of call		*/

double inparm[15];      /* input projection parameters          */
double fcproj[2];       /* Full scene center degrees 		*/
double deglat[4];	/* corner corndinates in degrees        */
double deglong[4];	/* corner corndinates in degrees        */
double minutes;		/* minutes				*/
double seconds;		/* seconds				*/

char ctemp[5];          /* Temp buffer                          */

buff = malloc(sizeof(unsigned char) * window[NS]);
if (buff == NULL)
{
  c_lasmsg(FALSE,"ltwgin-alloc","Error allocating dynamic memory");
  return(E_FAIL);
}

/* Get the DDR information from the image file
----------------------------------------------*/
if (c_getddr(hosout[0],&ddr) != E_SUCC)
{
    free(buff);
    return(E_FAIL);
}

/* Calculate the corner coordinates for these projections
---------------------------------------------------------*/
if ((strncmp(maprj,"SOM",3) == 0) || (strncmp(maprj,"UTM",3) == 0))
   {
   inproj = 0;
   prtprm[0] = 0;
   prtprm[1] = 0;

   for (i = 0; i < 15; i++)
       inparm[i] = 0.0;

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;

   if (strncmp(maprj,"SOM",3) == 0)
      {
      ddr.proj_code = SOM;
      ddr.zone_code = 62;
      if (somprm(path,ddr.proj_coef) != E_SUCC)
          {
          free(buff);
          return(E_FAIL);
          }
      }
   else
      {
      ddr.proj_code = UTM;
      ddr.zone_code = *utmzone;
      ddr.proj_coef[0] = fcenll[1];
      ddr.proj_coef[1] = fcenll[0];
      if (ceosid == HBK)
          {

          /* South Africa MSS data has no information on these values in the
             leader file.  These hard-coded values are "best guess" based on
             LIB-REF-0052 (LTWG Production Description Chart) entry on
             South Africa MSS data. */

          /* ddr.proj_coef[0] = 6378249.145;
          ddr.proj_coef[1] = 6356514.86955; */
	  /* UTM proj_coef should be lat/lon not semimajor/semiminor. Sine
             we don't know the lat/lon, hard code to 0.0 */
          ddr.proj_coef[0] = 0.0; 
          ddr.proj_coef[1] = 0.0;
          *datum = 1;
          }
      inzone = *utmzone;
      outzone = *utmzone;
      }

   if (ceosid == HBK)
      {
      c_file_corner(proj_corners, ls_corners, window, out_corners);
      ddr.upleft[0] = out_corners[UPLEFT_Y];
      ddr.upleft[1] = out_corners[UPLEFT_X];
      ddr.upright[0] = out_corners[UPRIGHT_Y];
      ddr.upright[1] = out_corners[UPRIGHT_X];
      ddr.loright[0]= out_corners[LORIGHT_Y];
      ddr.loright[1] = out_corners[LORIGHT_X];
      ddr.loleft[0] = out_corners[LOLEFT_Y];
      ddr.loleft[1] = out_corners[LOLEFT_X];
      strcpy(ddr.proj_units,"meters");
      }
   else if ((JAPAN) && (strcmp(sensor,"MSS") == 0) && (data_type == 'A'))
      {
      c_errmsg("Corner coordinates will be approximate","ltwgin-corn",&one);

      /* calcorn will give us the approximate corners
      ---------------------------------------------------*/
      strncpy(ctemp,direction,1);
      ctemp[1] = '\0';
      calcorn (fcenll[0], fcenll[1], ctemp, deglat, deglong, *satid);

      ddr.zone_code = inzone;
      ddr.upleft[0] = deglat[0];
      ddr.upleft[1] = deglong[0];
      ddr.upright[0] = deglat[3];
      ddr.upright[1] = deglong[3];
      ddr.loright[0] = deglat[2];
      ddr.loright[1] = deglong[2];
      ddr.loleft[0] = deglat[1];
      ddr.loleft[1] = deglong[1];
      strcpy(ddr.proj_units,"degrees");
      } 
   else
      {
      /* Initialize the Projection Transformation Package
      --------------------------------------------------*/
      if (c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,&ddr.proj_code,
                &outunit,&outzone,&outdatum,ddr.proj_coef,prtprm,"") != E_SUCC)
            {
            free(buff);
            return(E_FAIL);
            }

      /* Calculate the center projection coordinates
      ----------------------------------------------*/
      if (c_trans(&inproj,&inunit,&ddr.proj_code,&outunit,
           &fcenll[1],&fcenll[0],&fcproj[1],&fcproj[0])
             != E_SUCC)
            {
            free(buff);
            return(E_FAIL);
            }

      if (ceosid == THAI)
         {
         option[0] = 0;
         option[1] = 0; 
         if (*numimg_files == 1)
            status = c_eopenr(&filedesc,hosout[0],totbnd,&one,&one,
                &window[NL], &window[NS],&access,&idtype,option,&one);
         else
            status = c_eopenr(&filedesc,hosout[0],&one,&one,&one,
                &window[NL], &window[NS],&access,&idtype,option,&one);
         if (status != E_SUCC)
            {
            free(buff);
            return(E_FAIL);
            }

         line = (long)fcenls[0];
         status = c_eread(&filedesc,&one,&line,buff,&one);
         if (status == E_FAIL)
            {
            free(buff);
            return(E_FAIL);
            }

         /* Close the image file
         -----------------------*/
         if (c_eclose(&filedesc) != E_SUCC)
            {
            free(buff);
            return(E_FAIL);
            }

         /* Interpret the values of buff as an integer
         --------------------------------------------*/
         while (buff[fill] == 0)
            fill++;
         fcenls[1] = fcenls[1] + fill;
         }

      /* Find the projection coordinates for the corners
      -------------------------------------------------*/
      line_samp[0] = window[SL];
      line_samp[1] = window[SS];
      ls_proj(angle,fcproj,fcenls,ddr.upleft,line_samp,pixsiz);

      line_samp[0] = window[SL] + window[NL] - 1;
      line_samp[1] = window[SS];
      ls_proj(angle,fcproj,fcenls,ddr.loleft,line_samp,pixsiz);

      line_samp[0] = window[SL];
      line_samp[1] = window[SS] + window[NS] - 1;
      ls_proj(angle,fcproj,fcenls,ddr.upright,line_samp,pixsiz);

      line_samp[0] = window[SL] + window[NL] - 1;
      line_samp[1] = window[SS] + window[NS] - 1;
      ls_proj(angle,fcproj,fcenls,ddr.loright,line_samp,pixsiz);

      strcpy(ddr.proj_units,"meters");
      }

   /* Update the values that are known
   -----------------------------------*/
   ddr.master_line   = window[SL] - 1;
   ddr.master_sample = window[SS] - 1;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;
   ddr.pdist_y = pixsiz[0];
   ddr.pdist_x = pixsiz[1];
   if (*datum < 0)
      ddr.valid[DDDCV] = INVAL;
   else
     ddr.datum_code = *datum;
   }

else if ((strncmp(maprj,"NNN",3) == 0) || (strncmp(maprj,"YN",2) == 0))
   {
   /* No map projection, not UTM, and not using the reserved field
   ---------------------------------------------------------------*/
   c_errmsg("Corner coordinates will be approximate","ltwgin-corn",&one);

   /* calcorn will give us the approximate corners
   ---------------------------------------------------*/
   strncpy(ctemp,direction,1);
   ctemp[1] = '\0';
   calcorn (fcenll[0], fcenll[1], ctemp, deglat, deglong, *satid);

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;

   ddr.zone_code = inzone;
   ddr.upleft[0] = deglat[0];
   ddr.upleft[1] = deglong[0];
   ddr.upright[0] = deglat[3];
   ddr.upright[1] = deglong[3];
   ddr.loright[0] = deglat[2];
   ddr.loright[1] = deglong[2];
   ddr.loleft[0] = deglat[1];
   ddr.loleft[1] = deglong[1];
   strcpy(ddr.proj_units,"degrees");
   if (*datum < 0)
      ddr.valid[DDDCV] = INVAL;
   else
      ddr.datum_code = *datum;

   /* Update some values
   ---------------------*/
   ddr.master_line   = 0;
   ddr.master_sample = 0;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;

   /* Approximate pixel size in degrees
   ------------------------------------*/
   ddr.pdist_y = fabs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
   ddr.pdist_x = ddr.pdist_y;

   }


else if (strncmp(satnam,"SP",2) == 0)
   {
   c_errmsg("Corner coordinates will be approximate","ltwgin-corn",&one);

   /* Set the corners for SPOT Levels 1A and 1B. We have the four
      corners in a character string in DDDMMSS.  Level 2 is registered
      to a map but we don't have level2 data.  First convert
      coordinates to total degrees.
   --------------------------------------------------------------*/
   for (i = 0; i < 4; i++)
       {
       if (cornlat[i][0] == 'N')
          multfact = 1;
       else
          multfact = -1;

       strncpy(ctemp,&cornlat[i][1],3);
       ctemp[3] = '\0';
       deglat[i] = (double)atol(ctemp);
       strncpy(ctemp,&cornlat[i][4],2);
       ctemp[2] = '\0';
       minutes = (double)atol(ctemp);
       strncpy(ctemp,&cornlat[i][6],2);
       ctemp[2] = '\0';
       seconds = (double)atol(ctemp);

       deglat[i] = (deglat[i] + (minutes/60) + (seconds/3600)) * multfact;

       if (cornlong[i][0] == 'E')
          multfact = 1;
       else
          multfact = -1;

       strncpy(ctemp,&cornlong[i][1],3);
       ctemp[3] = '\0';
       deglong[i] = (double)atol(ctemp);
       strncpy(ctemp,&cornlong[i][4],2);
       ctemp[2] = '\0';
       minutes = (double)atol(ctemp);
       strncpy(ctemp,&cornlong[i][6],2);
       ctemp[2] = '\0';
       seconds = (double)atol(ctemp);

       deglong[i] = (deglong[i] + (minutes/60) + (seconds/3600)) * multfact;
       }

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;

   /* Fill in DDR fields
   ---------------------*/
   ddr.zone_code = inzone;
   ddr.upleft[0] = deglat[0];
   ddr.upleft[1] = deglong[0];
   ddr.upright[0] = deglat[1];
   ddr.upright[1] = deglong[1];
   ddr.loright[0] = deglat[3];
   ddr.loright[1] = deglong[3];
   ddr.loleft[0] = deglat[2];
   ddr.loleft[1] = deglong[2];
   strcpy(ddr.proj_units,"degrees");
   if (*datum < 0)
      ddr.valid[DDDCV] = INVAL;
   else
      ddr.datum_code = *datum;

   /* Update some values
   ---------------------*/
   ddr.master_line   = 0;
   ddr.master_sample = 0;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;

   /* Approximate pixel size in degrees
   ------------------------------------*/
   ddr.pdist_y = fabs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
   ddr.pdist_x = ddr.pdist_y;

   }
else
   c_errmsg("Unable to determine projection information","ltwgin-proj",&one);

/* Update the Band DDR information
----------------------------------*/
for (band = 1; band <= *totbnd; band++)
   {
   /* Get the Band DDR information from the image
   ----------------------------------------------- */
   if (*numimg_files == 1)
      {
      if (c_getbdr(hosout[0],&bddr[band-1],&band) != E_SUCC)
         {
         free(buff);
         return(E_FAIL);
         }
      }
   else
      {
      if (c_getbdr(hosout[band-1],&bddr[band-1],&one) != E_SUCC)
         {
         free(buff);
         return(E_FAIL);
         }
      }

   /* Update the Band DDR information
   ----------------------------------*/
   if (strncmp(satnam,"LS",2) == 0)
      {
      sprintf(bddr[band-1].source,"Landsat %ld",*satid);
      strcpy(bddr[band-1].instrument,sensor);

      /* Update the direction of the pass
      -----------------------------------*/
      if (strncmp(direction,"D",1) == 0)
          strcpy(bddr[band-1].direction,"Descending");
      else
          strcpy(bddr[band-1].direction,"Ascending");
      }
   else if (strncmp(satnam,"SP",2) == 0)
      sprintf(bddr[band-1].source,"SPOT %ld",*satid);
   }

/* Put the updated DDR information to the image
-----------------------------------------------*/
for (i = 0; i < *numimg_files; i++)
   if (c_putddr(hosout[i],&ddr) != E_SUCC)
      {
      free(buff);
      return(E_FAIL);
      }

/* Put the updated Band DDR information to the image
----------------------------------------------------*/
for (band = 0; band < *totbnd; band++)
   if (*numimg_files == 1)
      {
      if (c_putbdr(hosout[0],&bddr[band]) != E_SUCC)
         {
         free(buff);
         return(E_FAIL);
         }
      }
   else
      {
       if (c_putbdr(hosout[band],&bddr[band]) != E_SUCC)
          {
          free(buff);
          return(E_FAIL);
          }
      }

free(buff);
return(E_SUCC);
}
