#include "tiepts.h"
#include "cr.h"
#include "mapsup.h"
#include "convert.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine displays the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92
 Modified by J. LaVergne, USGS/EROS Data Center, 7/93
   - Corrected an error where user-input point coordinates (in DMS) were 
     converted to DD  but stored in a long variable.  This truncated the values      to whole degrees, which kept the transformation from being refined.
Algorithm Description:
*******************************************************************************/
void manual_display
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   if (Manual_form == NULL)
      {
      Manual_form = Createmanual_form(Shell_Manual);
      XtManageChild(Manual_form);
      }

   if (!XtIsRealized(Shell_Manual))
      XtRealizeWidget(Shell_Manual);
}


/*******************************************************************************
Purpose:
    This routine submits a manual entry coordinate from the user

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by John LaVergne, USGS/EROS Data Center 6/93
    -  changed to get real values into tie_select.rtie (they get
       truncated when they are put into tie_select.saveref)
Modified by John LaVergne, USGS/EROS Data Center 12/93
    -  added checks for formatting of user-entered coordinates

Algorithm Description:
*******************************************************************************/
void manual_spark
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XEvent  xe;
   int     status;
   char    proj_descr[81];

   if ((strlen(XmTextGetString(manual_x)) == 0) ||
       (strlen(XmTextGetString(manual_y)) == 0))
      return;

   if (tie_select.seltie == tie_select.numalloc)
      {
      tie_select.numalloc += MAX_TIE_POINTS;
      tie_select.elev    =(double *)(realloc(tie_select.elev,
                               tie_select.numalloc*sizeof(double)));
      tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                               2*tie_select.numalloc*sizeof(double)));
      tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                               2*tie_select.numalloc*sizeof(double)));
      tie_select.stie    =(void *)(realloc(tie_select.stie,
                               2*tie_select.numalloc*sizeof(double)));
      tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                               2*tie_select.numalloc*sizeof(double)));
      tie_select.resid   =(void *)(realloc(tie_select.resid,
                               2*tie_select.numalloc*sizeof(double)));
      tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                              tie_select.numalloc*PTIDLEN*sizeof(char)));
      tie_select.mapname =(void *)(realloc(tie_select.mapname,
                               tie_select.numalloc*80*sizeof(char)));
      tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                               tie_select.numalloc*80*sizeof(char)));
      tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                             tie_select.numalloc*sizeof(unsigned char)));
      if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) ||
          (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
          (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
          (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) || 
          (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
          {
          tie_select.numalloc -= MAX_TIE_POINTS;
          printf("Error allocating memory\n");
          XBell(tie.display,0);
          return;
          }
      }

   if (tie_mapparm.units == MAP_DEGREES)
      {
      tie_select.rtie[tie_select.seltie][0] =
                     map_dms2deg(XmTextGetString(manual_x));
      tie_select.rtie[tie_select.seltie][1] =
                     map_dms2deg(XmTextGetString(manual_y));
      }
   else if (tie_mapparm.units == MAP_PROJ)
      {
      strncpy(proj_descr,XmTextGetString(manual_descr),80);
      sscanf(XmTextGetString(manual_x), "%lf",
                &tie_select.rtie[tie_select.seltie][0]);
      sscanf(XmTextGetString(manual_y), "%lf",
                &tie_select.rtie[tie_select.seltie][1]);
      status = map_proj2degrees(tie_select.rtie[tie_select.seltie],1,
                                proj_descr);
      if (status != E_SUCC)
         tie_message(Shell_Manual,
                     "Error converting projection\ncoordinates to degrees");
      }
   else
      {
      sscanf(XmTextGetString(manual_x), "%lf",
                &tie_select.rtie[tie_select.seltie][0]);
      sscanf(XmTextGetString(manual_y), "%lf",
                &tie_select.rtie[tie_select.seltie][1]);
      }

   strcpy(tie_mapparm.mapname,"");
   strcpy(tie_mapparm.mapid,"");
   tie_select.saveref[0] = tie_select.rtie[tie_select.seltie][0];
   tie_select.saveref[1] = tie_select.rtie[tie_select.seltie][1];
   tie_select.curref = TRUE;

  /*
   *  *** if overview image is already fullres, send event to zoom
   */
   if (tie_rt.calc_rt != RT_SEL)
      {
      xe.xbutton.type = ButtonRelease;
      xe.xbutton.button = 1;
      xe.xbutton.x = 0;
      xe.xbutton.y = 0;

      fullres_button(ref_image_bulletinboard, NULL, &xe, 0);
      }
}

/*******************************************************************************
Purpose:
    This routine is the callback to the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void manual_coord
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   if (client == NULL)
      return;

   if (strcmp((char*)client, "GEOG") == 0)
      {
      tie_mapparm.units = MAP_DEGREES;
      XtSetSensitive(manual_descr,False);
      }
   else if (strcmp((char*)client, "USER") == 0)
      {
      tie_mapparm.units = MAP_USERDEF;
      XtSetSensitive(manual_descr,False);
      }
   else if (strcmp((char*)client, "PROJ") == 0)
      {
      tie_mapparm.units = MAP_PROJ;
      XtSetSensitive(manual_descr,True);
      }
}

/*******************************************************************************
Purpose:
    This routine traverses the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void manual_traverse
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XmProcessTraversal(Manual_form, XmTRAVERSE_NEXT_TAB_GROUP);
}

