/******************************************************************************
NAME:                           GETPAR 

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate 
    the window and band specifications  
   Checking to ensure that all the input images have equal window sizes  
   Retreive the output image name
   Retreive the output data type
   Retreive the global parameters
   Return
******************************************************************************/
#include <stdio.h>
#include "las.h"

FUNCTION getpar(vblock,taein,hosin,taeout,hosout,comp, nmrimg,window,bands, nbands, totbnd,odtype,msgtim,minmax)

   struct PARBLK *vblock;       /* pointer to TAE parameter block            */
   char taein[][CMLEN];         /* TAE file names of input images            */
   char hosin[][CMLEN];         /* host file names of input images           */
   char *taeout;                /* TAE file name of output image             */
   char *hosout;                /* host file name of output image            */
   char *comp;                  /* compression flag                          */
   long *nmrimg;                /* number of input images                    */
   long (*window)[4];           /* window specifications for each input image*/
   long (*bands)[MAXBND+1];     /* band specifications for each input image  */
   long *nbands;                /* number of bands per input image           */
   long *totbnd;                /* total number of bands to be processed     */
   long *odtype;                /* output data type                          */
   long *msgtim;                /* elapsed time between display messages     */
   char *minmax;                /* min max flag buffer                       */
{
struct VARIABLE *p_find();      /* function to find a TAE variable           */
struct VARIABLE *val;           /* pointer to a TAE variable within v-block  */

long    count;                  /* counter variable                          */
long    element;                /* parameter element number                  */
long    i;                      /* counter -- current image                  */
long   bnd,img;
long    imglim;                 /* maximum number of images to be processed  */
long    in_dtype;               /* input image data type                     */
long    one = 1;                /* value of one                              */
long    qflag[MAXIMG][2];       /* flag indicating if subwindows specified   */
long    same;                   /* flag indicating if images have same window*/
long    status;                 /* function return status code               */

char    dtype[5];               /* data type buffer                          */


/* Retrieve and parse input image specification
-----------------------------------------------*/
element = 0;
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
    c_lasmsg(TRUE,"translas5-fatal","Fatal error encountered");
if (same == 0)
    c_lasmsg(TRUE,"translas5-window","Input images do not have same window sizes");


/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
    c_lasmsg(TRUE,"translas5-fatal","Fatal error encountered");


/* Retrieve compression flag
--------------------------*/
if ((status = c_getcnt(vblock,"COMPFLG",&count)) == E_FAIL)
    c_lasmsg(TRUE,"translas5-fatal","Fatal error encountered");

if (count > 0)
    {
    val = p_find(vblock,"compflg");    
    strcpy(comp,SVAL(*val,0));
    }
else
    strcpy(comp,"\0");


/* Retrieve output data type
----------------------------*/
val = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
    *odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
    *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
    *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
    *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
    *odtype = EREAL;


/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return;
}
