/******************************************************************************
FUNCTION:       las50_c_eclose

PURPOSE         closes the file after all processing is done

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           06/85    T. Butzer           Original implementation
           03/86    T. Butzer           dealloc the buffer space used
           03/86    K. Johnson          handle multiple file groups
           12/87    B. Ailts            change include directory specifications
                                        change to raw 'C' types
                                        place bridge routines is seperate file
           04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with _errmsg
           05/88    B.Ailts             Replaced xi___ calls with i___ calls
           10/88    B.Ailts             Added statements to free dynamic
                                        allocated memory
           02/91    B.Ailts             Standardized error messages
           10/92    T.Mittan            Added compression option.
           12/92    T.Mittan            Corrected problem closing file.
           12/92    T.Mittan            Corrected problem freeing memory.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
        If CONVERSION or MOVEDATA or UPDATE
           Free conversion buffer memory
        Close the file
        Free the file descriptor memory
        Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include "las.h"

#include "locinc.h"

FUNCTION las50_c_eclose(fdesc)

        struct FDESC **fdesc;           /*  file descriptor array       */
{
int disp;
long nonfatal = 1;

if (((*fdesc)->flags & CONVERSION) || ((*fdesc)->flags & MOVEDATA) ||
    ((*fdesc)->acc == IUPDATE))
    free((*fdesc)->conv_buf);
if ((*fdesc)->compres == TRUE)
    {
    if ((*fdesc)->acc == IWRITE)
        {
        fseek((*fdesc)->fp,18,0);
        fwrite((*fdesc)->f_pointer,sizeof(long),
        ((*fdesc)->nl * (*fdesc)->nbands),(*fdesc)->fp);
        free((*fdesc)->f_pointer);
        free((*fdesc)->tempbuf);
        }
    else
        free((*fdesc)->foffset);
    fclose((*fdesc)->fp);
    c_dkfree((*fdesc)->fname);
    }
else
    {
    disp = I_SAVE;
    i_clse(&((*fdesc)->ifcb), disp);
    c_dkfree((*fdesc)->fname);
    }
free(*fdesc);
return(E_SUCC);
}
