/******************************************************************************
FUNCTION:       las50_c_egclse

PURPOSE         closes the group after all processing is done

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      06/85    T. Butzer           Original implementation
  1.1      03/86    T. Butzer           dealloc the buffer space used
  1.2      03/86    K. Johnson          handle multiple file groups
  5.0      12/87    B. Ailts            change include directory specifications
                                        change to raw 'C' types
                                        place bridge routines is seperate file
  5.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with _errmsg
  5.2      10/88    B. Ailts            Added statements to free dynamic
                                        memory that was used internal to imageio
  5.3      02/91    B. Ailts            Standardized error messages
  5.4      10/92    T. Mittan           Added compression option
  5.5      12/92    T. Mittan           Freed memory that was not being freed.  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
        For each file
           Close the file
           If access equals update
              Free the update and conversion buffers
           else
              Free the buffer and conversion buffers
           Free the file descriptors
        Free the group descriptors
        Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include "las.h"
#include "locinc.h"

FUNCTION las50_c_egclse(gdesc)

struct GDESC **gdesc; /* group desc from egroup */
{
long nonfatal = 1;      /*  nonfatal error code                 */
short i;
short j;
int disp;

disp = I_SAVE;

/* call xiclse for each file in group */

for (i=0; i<(*gdesc)->nfiles; i++)
    {
    if ((*gdesc)->fdarray[i]->compres == TRUE)
        {
        if ((*gdesc)->fdarray[i]->acc == IWRITE)
            {
            fseek((*gdesc)->fdarray[i]->fp,18,0);
            fwrite((*gdesc)->fdarray[i]->f_pointer,sizeof(long),
            ((*gdesc)->fdarray[i]->nl * (*gdesc)->fdarray[i]->nbands),
            (*gdesc)->fdarray[i]->fp);
            free((*gdesc)->fdarray[i]->f_pointer);
            free((*gdesc)->fdarray[i]->tempbuf);
            }
        else
            free((*gdesc)->fdarray[i]->foffset);
        fclose((*gdesc)->fdarray[i]->fp);
        c_dkfree((*gdesc)->fdarray[i]->fname);
        }
    else
        {
        i_clse(&((*gdesc)->fdarray[i]->ifcb), disp);
        c_dkfree((*gdesc)->fdarray[i]->fname);
        }
    
    /*  Free the update and conversion buffers
    -----------------------------------------*/
    if ((*gdesc)->fdarray[i]->acc == IUPDATE)
        {
        if ((*gdesc)->fdarray[i]->flags & CONVERSION)
            for (j = 0; j < (*gdesc)->fdarray[i]->nbands; j++)
                free((*gdesc)->fdarray[i]->upbuf[j].conbuf);
        free((*gdesc)->fdarray[i]->upbuf);
        }  /*  if access equals UPDATE  */
    else
        {
        if ((*gdesc)->fdarray[i]->flags & CONVERSION)
            for (j = 0; j < (*gdesc)->fdarray[i]->nbands; j++)
                free((*gdesc)->fdarray[i]->buf[j].conbuf);
        free((*gdesc)->fdarray[i]->buf);
        }  /*  else access not equals UPDATE  */
    
    /*  free the file descriptors  */
    free((*gdesc)->fdarray[i]);
    }  /*  for gdesc->nfiles  */

/*  Free the group descriptor  */
free(*gdesc);

return(E_SUCC);
}
