/******************************************************************************
FUNCTION:       las50_c_eopenr

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      12/87    B.Ailts             Placed bridge routines into a seperate
                                        file
                                        Used raw 'C' types
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
  2.2      05/88    B.Ailts             Replaced errh with _errmsg
                                        Changed the calling sequence of the 
                                        DDR calls -- one structure
  2.3      08/88    L. Huewe            Added argument for number of lines to
                                        be read at one time
  2.4      09/88    L. Huewe            Fixed the calling sequence to i_read
  2.5      09/88    B.Ailts             filesize is now checked only for READ
                                        access -- not for UPDATE access
  2.6      10/88    B.Ailts             Added 'if' statement to not allocate
                                        conversion buffer if it is not needed
  2.7      10/89    B.Davis             Placed the call to lused inside of a
                                        check for read access.  We don't
                                        want to change the operating system
                                        time stamp of the ddr disk file when
                                        all we are doing is reading an image
                                        for input.  (for Catalog Manager).
  2.8      02/91    B.Ailts             Standardized error messages
  2.9      05/92    T. Mittan           Check entire filesize not just
                                        requested filesize
  3        10/92    T.Mittan            Added compression option
  4        11/93    T. Mittan           Modifiied compression option to
                                        be set with a LAS global.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
If acess equals read
   Initialize the file descriptor members
   Compare the system of image and the system of the CPU
   Open the image
   Get the ddr information and place in the fdesc structure
   Check the window specifications
   Check the band specifications
   Check for dat type conversion
Else write access
   Check the band specification
   Check the options argument
   If UNIX_DISK
      Call the disk allocation software
   Initialize the DDR fields
   Fill in the DDR fields that are needed
   Write the band indepenant DDR fields
   Initialize the band depenant fields
   Write the band depentant fields
If access == UPDATE or flags & CONVERSION or flags & MOVEDATA
   Allocate the conversion buffer space
Write the last used date and time to the DDR

ALGORITHM REFERENCES    none
*******************************************************************************/

#ifdef _IBMR2
#define unix
#endif

#ifdef unix
#include <sys/types.h>
#include <sys/stat.h>
#endif

#define COMPRES1 "COMPRES"

#include "las.h"
#include "locinc.h"

extern long compres_opt;

FUNCTION las50_c_eopenr(fdesc,fname,nbands,sl,ss,nl,ns,acc,dtype,opt,maxline)

        struct FDESC **fdesc;   /*  file descrptor array                */
        char *fname;            /*  host file name                      */
        long *nbands;           /*  number of bands                     */
        long *nl,*ns,*sl,*ss;   /*  window specification                */
        long *acc;              /*  type of access                      */
        long *dtype;            /*  data type                           */
        long opt[];             /*  array of options                    */
        long *maxline;          /*  maximum number of lines to be read
                                    at one time                         */

{
struct DDR ddr;
struct BDDR bddr;
register struct FDESC *fd;

long band;
long dflag;
long filesize;
long i;
long linsiz;
long nonfatal = 1;
long stati;
long valid;

char errtxt[ERRLEN];
char system[DDSYLN];
char *compres;
char *getenv();

#ifdef unix
struct stat statstruct;
#endif

*fdesc = (struct FDESC *)calloc(1,sizeof(struct FDESC));
if (*fdesc == NULL)
    {
    c_lasmsg(FALSE,"eopenr-alloc","Error alocatting dynamic memory");
    return(E_FAIL);
    }
fd = *fdesc;    /* used to simplify expressions */
fd->nl = *nl;
fd->ns = *ns;
fd->sl = *sl;
fd->ss = *ss;
fd->acc = *acc;
fd->method = RANDOM;
fd->dtype = *dtype;
fd->nbuf = 0;
(void)strcpy(fd->fname, fname);
fd->flags = 0; /* clear flags */
fd->cnvlsz = 0;

fd->nbands = *nbands;

if ((fd->acc == IREAD) || (fd->acc == IUPDATE))
    {     
    /*  Compare the system that the data is stored on and the system that will
    process the data.
    --------------------------------------------------------------------------*/
    stati = c_comsys(fname,&dflag);
    if (stati == E_FAIL)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from comsys");
        return(E_FAIL);
        }
    if ((dflag == 3) ||
        ((dflag == 1) && (ddr.dtype == EREAL)) ||
        ((dflag == 2) && (ddr.dtype != EBYTE)))
            {
            c_lasmsg(FALSE,"eopenr-dtype","Data type of image is not compatible with the processor");
            return(E_FAIL);
            }
    
    
    stati = las50_do_open(fd);    
    if (stati != E_SUCC)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from doopen");
        return(E_FAIL);
        }
    
    stati = c_getddr(fname, &ddr);
    if (stati != E_SUCC)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from getddr");
        return(E_FAIL);
        }
    if (*dtype == 0)
        {
        *dtype = ddr.dtype;
        fd->dtype = ddr.dtype;
        }
    if (ddr.dtype != fd->dtype)
        {
        fd->flags |= CONVERSION;
        fd->conv_type = ddr.dtype;
        fd->cnvlsz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
        }
    
    if (*nbands == 0)
        {
        *nbands = ddr.nbands;
        for(i=1; i<=ddr.nbands; i++)
            fd->bands[i-1] = i;
        fd->nbands = *nbands;
        }
    
    if (*nl == 0)
        {
        *nl = ddr.nl;
        fd->nl = *nl;
        }
    if (*ns == 0)
        {
        *ns = ddr.ns;
        fd->ns = *ns;
        }
    fd->linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
    linsiz = (ddr.ns * datasize[ddr.dtype] + 511) & ~0x1ff;
    
    if (las50_check_window(fd, &ddr) != E_SUCC)
        {
        sprintf(errtxt,"Error returned from las50_check_window");
        c_lasmsg(FALSE,"eopenr-call",errtxt);
        return(E_FAIL);
        }
    
    if (fd->nbands > ddr.nbands)
        {
        sprintf(errtxt,"More bands specified than exist for %s",
        fd->fname);
        c_lasmsg(FALSE,"eopenr-nbands",errtxt);
        return(E_FAIL);
        }
    if (fd->compres == TRUE)
        {
        fd->mns = ddr.ns;
        if (fd->acc == IUPDATE)
            {
            c_lasmsg(FALSE,"eopenr-update","Compresed image can not be opened for update access");
            fclose(fd->fp);
            return(E_FAIL);
            }
        }
    
#ifdef unix
    if (fd->acc == IREAD && fd->compres != TRUE)
        {
        if (fd->flags & CONVERSION)
            if (((ddr.ns * datasize[ddr.dtype]) % 512) == 0)
                filesize = 512 + (fd->cnvlsz * ddr.nl * ddr.nbands);
            else
                filesize = 512 + (fd->cnvlsz * ddr.nl * ddr.nbands) -
                (512 - ((ddr.ns * datasize[ddr.dtype]) % 512));
        else
            /*
            filesize = 512 + (fd->linsiz * fd->nl * fd->nbands) -
            (512 - ((ddr.ns * datasize[ddr.dtype]) % 512));
        */
        if (((ddr.ns * datasize[ddr.dtype]) % 512) == 0)
            filesize = 512 + (linsiz * ddr.nl * ddr.nbands);
        else
            filesize = 512 + (linsiz * ddr.nl * ddr.nbands) -
            (512 - ((ddr.ns * datasize[ddr.dtype]) % 512));
        
        stat(fd->fname, &statstruct);
        if (statstruct.st_size < filesize)
            {
            sprintf(errtxt,"File %s is not %d bytes big",fd->fname,
            filesize);
            c_lasmsg(FALSE,"eopenr-size",errtxt);
            return(E_FAIL);
            }
        }  /*  if fd->acc == IREAD  */
#endif
    if ((*ns * datasize[*dtype]) != fd->linsiz)
        fd->flags |= MOVEDATA;
    
    if ((*ns != ddr.ns) && (*maxline > 1))
        fd->flags |= MOVEDATA;
    
    if (fd->acc != IREAD)
        {
        stati = c_lused(fname);
        if (stati == E_FAIL)
            {
            c_lasmsg(FALSE,"eopenr-call","Error returned from lused");
            return(E_FAIL);
            }
        }
    
    }  /*  if (fd->acc == IREAD) || (fd->acc == IUPDATE)  */
else
    {
    if (*nbands < 1)
        {
        sprintf(errtxt,"No bands specified for file %s",fd->fname);
        c_lasmsg(FALSE,"eopenr-nobands",errtxt);
        return(E_FAIL);
        }
    fd->linsiz = (*ns * datasize[*dtype] + 511) & ~0x1ff;
    
    if (opt[1] == COMPRES)
        {
        fd->compres = TRUE;
        fd->tempbuf = (unsigned char *) calloc(((long)(fd->linsiz *2))
        ,sizeof(unsigned char));
        fd->f_pointer = (long *) calloc ((fd->nl * fd->nbands),
        sizeof(long));
        }
    else if (opt[1] == NO_COMPRES)
        {
        fd->compres = FALSE;
        }
    else if (compres_opt == NO_COMPRES)
        {
        fd->compres = FALSE;
        }
    else if (compres_opt == COMPRES)
        {
        fd->compres = TRUE;
        fd->tempbuf = (unsigned char *) calloc(((long)(fd->linsiz *2))
        ,sizeof(unsigned char));
        fd->f_pointer = (long *) calloc ((fd->nl * fd->nbands),
        sizeof(long));
        }
    else if (compres = getenv(COMPRES1))
        if ((strcmp(compres,"ON") == 0) || (strcmp(compres,"on") == 0))
            {
            fd->compres = TRUE;
            fd->tempbuf = (unsigned char *) calloc((
            (long)(fd->linsiz *2)),sizeof(unsigned char));
            fd->f_pointer = (long *) calloc ((fd->nl * fd->nbands),
            sizeof(long));
            }
    
    if ((opt[0] > 0) && (opt[0] < NDTYPES))
        {
        if (opt[0] != fd->dtype)
            {
            fd->flags |= CONVERSION;
            fd->conv_type = opt[0];
            fd->cnvlsz = (*ns *
            datasize[opt[0]] + 511) & ~0x1ff;
            }
        }
    else if (opt[0] != 0)
        {
        c_lasmsg(FALSE,"eopenr-option","Invalid option specified");
        return(E_FAIL);
        }
    
    filesize = 512 + fd->linsiz * fd->nl * fd->nbands;
    if (opt[0] != 0)
        if (opt[0] != fd->dtype)
            filesize = 512 +
            fd->cnvlsz * fd->nl * fd->nbands;
    c_dkcre(0, fd->fname, filesize, 600);
    
    stati = las50_do_open(fd);    
    if (stati != E_SUCC)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from doopen");
        return(stati);
        }
    
    c_intddr(&ddr);
    ddr.nl = fd->nl;
    ddr.ns = fd->ns;
    ddr.nbands = fd->nbands;
    if (fd->flags & CONVERSION)
        {
        ddr.dtype = fd->conv_type;
        linsiz = fd->cnvlsz;
        }
    else
        {
        ddr.dtype = fd->dtype;
        linsiz = fd->linsiz;
        }
    
    stati = c_getsys(system);
    if (stati != E_SUCC)
        c_lasmsg(FALSE,"eopenr-call","Error returned from getsys");
    else
        strcpy(ddr.system,system);
    
    stati = c_putddr(fname, &ddr);
    if (stati != E_SUCC)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from putddr");
        return(E_FAIL);
        }
    for (i=1; i < *nbands + 1; i++)
        {
        c_intbdr(&bddr);
        bddr.bandno = i;
        stati = c_putbdr(fname, &bddr);
        if (stati != E_SUCC)
            {
            c_lasmsg(FALSE,"eopenr-call","Error returned from putbdr");
            return(E_FAIL);
            }
        } /* for i < nbands + 1  */
    
    if ((*ns * datasize[*dtype]) != fd->linsiz)
        fd->flags |= MOVEDATA;
    }

if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA) || (fd->acc == IUPDATE))
    {
    fd->conv_buf = (unsigned char *) calloc(1, ((ddr.ns * datasize[ddr.dtype]
    + 511) & ~0x1ff) * (*maxline));
    if (fd->conv_buf == NULL)
        {
        c_lasmsg(FALSE,"eopenr-allocbuf","Error allocating buffer space");
        return(E_FAIL);
        }
    }  /*  if conversion or movedata or update  */


/*
the image is only being read as input, do not
update the ddr last modified date and time
*/
if (fd->acc != IREAD)
    {
    stati = c_lused(fname);
    if (stati == E_FAIL)
        {
        c_lasmsg(FALSE,"eopenr-call","Error returned from lused");
        return(E_FAIL);
        }
    }

return(E_SUCC);
}
