/******************************************************************************
FUNCTION:       las50_c_eread

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      12/87    B.Ailts             Placed bridge routine in seperate file
                                        Changed name from eread to las50_c_eread
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with _errmsg
  2.2      05/88    B.Ailts             Replaced errh with _errmsg
                                        Changed calling sequences of the DDR
                                        routines -- only one structure
                                        Took out eerror.h
  2.3      08/88    L. Huewe            Added argument for number of lines to
                                        be read at one time
  2.4      12/88    B. Ailts            Output an error message when the data
                                        is being clipped
  2.5       2/88    B. Ailts            Replaced the 'ceil' function -- does
                                        not work correctly
  2.6      02/91    B. Ailts            Standardized error messages
  2.7      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  2.8      10/92    T.Mittan            Added compression option


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
If conversion
   Read the line into temporary buffer
   Convert the data into the output buffer
Else
   Read the line into the output buffer
Return number of samples read

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <math.h>
#include "las.h"
#include "locinc.h"
#include "pixman.h"
#ifdef vms
#define MAXBUF 65535
#endif

FUNCTION las50_c_eread(fdesc, eband, eline, euserbuf,nlines)

struct FDESC **fdesc;           /*  file descriptor array               */
long *eband;                    /*  band number to be read              */
long *eline;                    /*  line number to be read              */
unsigned char *euserbuf;        /*  buffer                              */
long *nlines;
{
struct FDESC *fd;

int hcode;

long status;
long line;
long nparm;
long nonfatal = 1;              /* non-fatal error code */
long fatal = -1;                /* fatal error code     */
long nreads;
long index;
long i;
long numlines;
long lsize;
long count = 1;
long count1;
long sample;
long band;
long *position_end;
unsigned char *tempbuf;
unsigned char *tempbuf1;
FILE *fp;

short big_first = TRUE;
short small_first = TRUE;

char errtxt[ERRLEN];
unsigned char *cbufptr, *ubufptr;
int sig_mask;

c_sigblk(&sig_mask);
fd = *fdesc;
line = *eline + fd->sl - 1;

#ifdef DEBUG
printf("read line = %d nlines = %d\n", line, *nlines);
#endif

if (fd->flags & CONVERSION)
    lsize = fd->cnvlsz;
else
    lsize = fd->linsiz;

if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA))
    {
    nreads = 1;
    numlines = *nlines;
    index = 0;
#ifdef vms
    numlines = MAXBUF / lsize;
    nreads = *nlines / numlines;
    if (*nlines % numlines != 0)
        ++nreads;
#endif
    for (i=0; i < nreads; i++)
        {
#ifdef vms
        index = i * (numlines * lsize);
        line = i * numlines + (*eline + fd->sl - 1);
        if (i == (nreads - 1))
            numlines = *nlines - (numlines * i);
#endif
        /* check compression flag 
        -----------------------*/
        if (fd->compres)
            {
            if ((position_end = (long *) calloc((fd->nl * fd->mbands),
                sizeof(long)))== NULL)
                    {
                    c_lasmsg(FALSE,"las50_c_eread-alloc","Error allocation dynamic memory");
                    return(E_FAIL);
                    }
            sample = 18 + ((line -1) * fd->mbands * 4);
            fseek(fd->fp,sample,0);
            if (fread(position_end,sizeof(long),(fd->nl * fd->mbands),fd->fp)
                == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
            
            while (count <= numlines)
                {
                sample = position_end[((count - 1)* fd->mbands) + (*eband - 1)];
                fseek(fd->fp,sample,0);
                if (fread(&count1,sizeof(long),1,fd->fp) == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
                if ((tempbuf = (unsigned char *)calloc(count1,
                    sizeof(unsigned char))) == NULL)
                        {
                        c_lasmsg(FALSE,"las50_c_eread-alloc","Error allocation dynamic memory");
                        return(E_FAIL);
                        }
                if (fread(tempbuf,sizeof(char),count1,fd->fp) == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
                if (fd->conv_type == 0)
                    {
                    cbufptr = fd->conv_buf + index;
                    las50_decode_76(tempbuf,cbufptr,fd->mns,datasize[fd->dtype]);
                    }
                else
                    {
                    cbufptr = fd->conv_buf + index;
                    las50_decode_76(tempbuf,cbufptr,fd->mns,datasize[fd->conv_type]);
                    }
                index =  (count * lsize);
                count++;
                free(tempbuf);
                }
            free(position_end);
            }
        else           /* fd->compress flag not set */
            {
            i_read(&(fd->ifcb), &(fd->conv_buf[index]),
            *eband, line, numlines);
            
            status = i_wait(&(fd->ifcb));
            if (status != SUCCESS)
                {
                hcode = i_herr(&(fd->ifcb));
                sprintf(errtxt,"Xiwait error %d encountered for file %s",
                hcode,fd->fname);
                c_lasmsg(FALSE,"eread-xiwait",errtxt);
                return(E_FAIL);
                }
            }
        }
    
    
    if (fd->flags & CONVERSION)
        cbufptr = fd->conv_buf + (fd->ss - 1) * datasize[fd->conv_type];
    else
        cbufptr = fd->conv_buf + (fd->ss - 1) * datasize[fd->dtype];
    ubufptr = euserbuf;
    
    for (i=0; i < *nlines; i++)
        {
        if (fd->flags & CONVERSION)
            {
            status = c_pxconv(&fd->conv_type, &fd->dtype,cbufptr,
            ubufptr, &fd->ns);
            if (status != E_SUCC)
                {
                if (((status & TOOBIG) == TOOBIG) && (big_first))
                    {
                    sprintf(errtxt,"%s%s", "Pixel values were larger than",
                    " maximum allowed by output data type");
                    c_lasmsg(FALSE,"eread-pixel",errtxt);
                    big_first = FALSE;
                    }
                else if (((status & TOOSMALL)  == TOOSMALL) && (small_first))
                    {
                    sprintf(errtxt,"%s%s", "Pixel values were smaller than",
                    " minimum allowed by output data type");
                    c_lasmsg(FALSE,"eread-pixel",errtxt);
                    small_first = FALSE;
                    }
                }
            }
        else
            c_pxcopy(cbufptr, ubufptr, &fd->dtype, &fd->ns);
        
        cbufptr += lsize;
        ubufptr += (fd->ns * datasize[fd->dtype]);
        }
    }
else
    {
    nreads = 1;
    numlines = *nlines;
    index = 0;
#ifdef vms
    numlines = MAXBUF / lsize;
    nreads = *nlines / numlines;
    if (*nlines % numlines != 0)
        ++nreads;
#endif
    for (i=0; i < nreads; i++)
        {
#ifdef vms
        index = i * (numlines * lsize);
        line = i * numlines + (*eline + fd->sl - 1);
        if (i == (nreads - 1))
            numlines = *nlines - (numlines * i);
#endif
        /* check compression flag 
        -----------------------*/
        if (fd->compres)
            {
            if ((position_end = (long *)calloc((fd->nl * fd->mbands),
                sizeof(long))) == NULL)
                    {
                    c_lasmsg(FALSE,"las50_c_eread-alloc","Error allocation dynamic memory");
                    return(E_FAIL);
                    }
            sample = 18 + (line -1) * fd->mbands * 4;
            fseek(fd->fp,sample,0);
            if (fread(position_end,sizeof(long),(fd->nl * fd->mbands),fd->fp)
                == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
            while (count <= numlines)
                {
                sample = position_end[((count - 1) * fd->mbands) +(*eband - 1)];
                fseek(fd->fp,sample,0);
                if (fread(&count1,sizeof(long),1,fd->fp) == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
                if ((tempbuf = (unsigned char *)calloc(count1,
                    sizeof(unsigned char))) == NULL)
                        {
                        c_lasmsg(FALSE,"las50_c_eread-alloc","Error allocation dynamic memory");
                        return(E_FAIL);
                        }
                if ((tempbuf1 = (unsigned char *)calloc((fd->mns *
                    datasize[fd->dtype]), sizeof(unsigned char))) == NULL)
                        {
                        c_lasmsg(FALSE,"las50_c_eread-alloc","Error allocation dynamic memory");
                        return(E_FAIL);
                        }
                if (fread(tempbuf,sizeof(char),count1,fd->fp) == E_FAIL)
                    {
                    c_lasmsg(FALSE,"eread-read","error reading file");
                    return(E_FAIL);
                    }
                las50_decode_76(tempbuf,tempbuf1,fd->mns, datasize[fd->dtype]);
                c_pxcopy(&tempbuf1[fd->ss-1],&euserbuf[index],&fd->dtype,
                &fd->ns);
                index =  (count * lsize);
                count++;
                free(tempbuf);
                free(tempbuf1);
                }
            free(position_end);
            }
        else           /* fd->compress flag not set */
            {
            i_read(&(fd->ifcb), &euserbuf[index], *eband, line, numlines);
            
            status = i_wait(&(fd->ifcb));
            if (status != SUCCESS)
                {
                xiherr(&(fd->ifcb), &hcode);
                sprintf(errtxt,"Xiwait error %d encountered for file %s",
                hcode,fd->fname);
                c_lasmsg(FALSE,"eread-xiwait",errtxt);
                return(E_FAIL);
                }
            }
        }
    }

c_sigunblk(&sig_mask);
return((fd->ns * (*nlines)));
}
