/******************************************************************************
FUNCTION:       las50_c_estep

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original Development
  2.0      12/87    B.Ailts             Used raw 'C' types
                                        Placed brigde routines in a seperate
                                        file
                                        Renamed from estep to las50_c_estep

                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
Do 
   Do for number of files
      Do for number of bands
          Process a line
While not done
Do for number of files
   If READ access and CONVERSIOn
      Wait for any outstanding I/Os

ALGORITHM REFERENCES    none
*******************************************************************************/

#include "las.h"
#include "locinc.h"

#define NLINES gd->gbl_nlines

FUNCTION las50_c_estep(gdesc)

struct GDESC **gdesc;           /*  group descriptor                    */
{
register struct GDESC *gd;
register struct FDESC *fd;

long band_cnt;
long done;
long band;
long j;
register long i;
long status;
long *ptr;

gd = *gdesc;

band_cnt = 0;

/* clear all done flags */
for (i=0; i < gd->nfiles; i++)
    gd->fdarray[i]->flags &= ~EDONE;

done = gd->nfiles;

do
    {
    for (i=0; i < gd->nfiles; i++)
        {
        if (band_cnt < gd->fdarray[i]->nbands)
            {
            status=las50_process_it(gd->fdarray[i], gd->line_cnt, band_cnt, gd);
            if (status != E_SUCC)
                return(status);
            }
        else
            {
            if (!(gd->fdarray[i]->flags & EDONE))
                {
                done--;
                gd->fdarray[i]->flags |= EDONE;
                }
            }
        }
    band_cnt++;
    }
while (done != 0);

gd->line_cnt++;

/* wait for any outstanding i/o's */

for (i=0; i < gd->nfiles; i++)
    {
    if (!((gd->fdarray[i]->acc == IREAD)
        && (gd->fdarray[i]->flags & CONVERSION)))
            {
            if (gd->fdarray[i]->compres != TRUE)
                {
                status = i_wait(&(gd->fdarray[i]->ifcb));
                if (status != SUCCESS)
                    if ((gd->line_cnt + NLINES)
                        < gd->fdarray[i]->nl)
                            {
                            c_lasmsg(FALSE,"estep-xiwait",
                            "Xiwait error encountered for file %s",
                            gd->fdarray[i]->fname);
                            return(E_FAIL);
                            }
                }
            }
    }
return(E_SUCC);
}
