/******************************************************************************
FUNCTION:       las50_c_ewrite

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original Development
  2.0      12/87    B. Ailts            Placed bridge routinue in a separate 
                                        file
                                        Changed include file specifications
                                        Renamed fron ewrite to las50_c_ewrite
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with _errmsg
  2.2      05/88    B. Ailts            Replaced eerh with _errmsg
                                        Deleted eerror.h
  2.3      08/88    L. Huewe            Added argument for number of lines to
                                        be read at one time
  2.4      09/88    L. Huewe            Fixed the calling sequence to i_read
  2.5      10/88    B. Ailts            Do not set cbufptr unless needed
  2.6      12/88    B. Ailts            Output an error message if values are
                                        clipped
  2.7       2/89    B. Ailts            Replaced the 'ceil' function -- did not
                                        work properly
  2.8       5/90    B. Ailts            Fixed a problem on VMS with the calc. of
                                        the index variable
  2.9      10/90    T. Baltzer          Rewrite so IUPDATE mode can convert data
  3.0      02/91    B. Ailts            Standardized error messages
  3.1      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  3.2      10/92    T. Mittan           Added compression option
  3.3      4/93     T. Mittan           Corrected problem when multiple lines
                                        are written in compressed mode.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
   
ALGORITHM REFERENCES    none
*******************************************************************************/

#include <math.h>
#include "las.h"
#include "locinc.h"
#include "pixman.h"
#ifdef vms
#define MAXBUF 65535
#endif

FUNCTION las50_c_ewrite(fdesc, eband, eline, euserbuf, nlines)

struct FDESC **fdesc;           /*  file descriptor                     */
long *eband;                    /*  band number to write                */
long *eline;                    /*  line number to write                */
unsigned char *euserbuf;        /*  buffer                              */
long *nlines;                   /*  number of lines to write            */
{
struct FDESC *fd;

long hosterr;
long line;
long nonfatal = 1;
long status;
long nparm;
long numpix;
long i,j;
long nwrites;
long numlines;
long index;
long lsize;
long count1;
long one = 1;
long conv_type;

static short big_first = TRUE;
static short small_first = TRUE;

char errtxt[ERRLEN];
unsigned char *cbufptr;
unsigned char *ubufptr;
int sig_mask;

c_sigblk(&sig_mask);
fd = *fdesc;
line = *eline + fd->sl - 1;
if (fd->flags & CONVERSION)
    lsize = fd->cnvlsz;
else
    lsize = fd->linsiz;

nwrites = 1;
numlines = *nlines;
index = 0;
#ifdef vms
numlines = MAXBUF / lsize;
nwrites = *nlines / numlines;
if (*nlines % numlines != 0)
    ++nwrites;
#endif
for (i=0; i < nwrites; i++)
    {
#ifdef vms
    index = i * (numlines * fd->ns * datasize[fd->dtype]);
    line = i * numlines + (*eline + fd->sl - 1);
    if (i == (nwrites - 1))
        numlines = *nlines - (numlines * i);
#endif
    
    if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA) ||
        (fd->acc == IUPDATE))
            {
            if (fd->acc == IUPDATE)
                {
                i_read(&(fd->ifcb), fd->conv_buf, *eband, line, numlines);
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                    {
                    hosterr = i_herr(&(fd->ifcb));
                    sprintf(errtxt,"Xiwait error %d encountered for file %s",
                    hosterr,fd->fname);
                    c_lasmsg(FALSE,"ewrite-xiwait",errtxt);
                    }
                if (fd->flags & CONVERSION)
                    cbufptr = fd->conv_buf;
                else
                    cbufptr = fd->conv_buf + (fd->ss - 1) * datasize[fd->dtype];
                }
            else
                {
                cbufptr = fd->conv_buf;
                }
            ubufptr = &euserbuf[index];
            for (j=0; j < numlines; j++)
                {
                if (fd->flags & CONVERSION)
                    {
                    status = c_pxconv(&fd->dtype, &fd->conv_type, ubufptr,
                    cbufptr, &fd->ns);
                    if (status != E_SUCC)
                        {
                        if (((status & TOOBIG) == TOOBIG) && (big_first))
                            {
                            sprintf(errtxt,"%s%s","Pixel values were larger than",
                            " maximum allowed by output data type");
                            c_lasmsg(FALSE,"ewrite-pixel",errtxt);
                            big_first = FALSE;
                            }
                        else if (((status & TOOSMALL) == TOOSMALL) &&
                            (small_first))
                                {
                                sprintf(errtxt,"%s%s","Pixel values were smaller than",
                                " minimum allowed by output data type");
                                c_lasmsg(FALSE,"ewrite-pixel",errtxt);
                                small_first = FALSE;
                                }
                            }
                    }
                else
                    {
                    c_pxcopy(ubufptr,cbufptr,&fd->dtype,&fd->ns);
                    }
                if ( fd->compres == TRUE)
                    {
                    fseek(fd->fp,0,2);
                    fd->f_pointer[((line + j - 1)*fd->nbands)+ (*eband - 1)] =
                    ftell(fd->fp);
                    if (fd->conv_type == 0)
                        conv_type = fd->dtype;
                    else
                        conv_type = fd->conv_type;
                    las50_encode_76(cbufptr,fd->tempbuf,fd->ns, &count1,
                    datasize[conv_type]);
                    fwrite(&count1,sizeof(long), one,fd->fp);
                    fwrite(fd->tempbuf,sizeof(char),count1,fd->fp);
                    }
                cbufptr += lsize;
                ubufptr += (fd->ns * datasize[fd->dtype]);
                }
            if (fd->compres != TRUE)
                {
                i_write(&(fd->ifcb), fd->conv_buf, *eband, line, numlines);
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                    {
                    hosterr = i_herr(&(fd->ifcb));
                    sprintf(errtxt,"Xiwait error %d encountered for file %s",
                    hosterr,fd->fname);
                    c_lasmsg(FALSE,"ewrite-xiwait",errtxt);
                    }
                }
            }
        else
            {
            if ( fd->compres == TRUE)
                {
                for (j=0; j < numlines; j++)
                    {
                    index += j * (fd->ns * datasize[fd->dtype]);
                    fseek(fd->fp,0,2);
                    fd->f_pointer[((line + j - 1) * fd->nbands) + (*eband -1)] =
                    ftell(fd->fp);
                    if (fd->conv_type == 0)
                        conv_type = fd->dtype;
                    else
                        conv_type = fd->conv_type;
                    las50_encode_76(&euserbuf[index],fd->tempbuf,fd->ns, &count1,
                    datasize[conv_type]);
                    fwrite(&count1,sizeof(long), one,fd->fp);
                    fwrite(fd->tempbuf,sizeof(char),count1,fd->fp);
                    }
                }
            else
                {
                i_write(&(fd->ifcb), &euserbuf[index], *eband, line, numlines);
                status = i_wait(&(fd->ifcb));
                if (status != SUCCESS)
                    {
                    hosterr = i_herr(&(fd->ifcb));
                    sprintf(errtxt,"Xiwait error %d encountered for file %s",
                    hosterr,fd->fname);
                    c_lasmsg(FALSE,"ewrite-xiwait",errtxt);
                    }
                }
            }
#ifdef DEBUG
        printf("write line = %d nlines = %d\n", line, *nlines);
#endif
    }

c_sigunblk(&sig_mask);
return(fd->ns * (*nlines));
}
