/******************************************************************************
FUNCTION:       las50_chkwindow

PURPOSE:        Checks to see if the specified window is outside the actual
                window for a LAS file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           06/85    T. Butzer           Original implementation
           12/87    B. Ailts            change include directory structure
           12/87    B. Ailts            implemented new DDR structure
           02/91    B. Ailts            Standardized error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Check the window specifications against the DDR window specs.
If the window specifications are outside the DDR window specs
   Return a fatal error

ALGORITHM REFERENCES    none
*******************************************************************************/

#include "las.h"
#include "locinc.h"

FUNCTION las50_check_window(fd, ddr)

struct FDESC *fd;       /* file descriptor */
struct DDR *ddr;        /* ddr structure */
        
{
long nonfatal = 1;
long status;


if ((fd->sl + fd->nl - 1) > ddr->nl)
    return(E_FAIL);

if ((fd->ss + fd->ns - 1) > ddr->ns)
    return (E_FAIL);

if ((fd->sl != 1) || (fd->ss != 1) ||
    (fd->nl != ddr->nl) || (fd->ns != ddr->ns))
        {
        c_lasmsg(FALSE,"las50_chkwindow-window",
        "Window specified for file %s is (%d %d %d %d)",fd->fname,
        fd->sl,fd->ss,fd->nl,fd->ns);
        }
return(E_SUCC);
}
