/******************************************************************************
FUNCTION:       las50_do_open

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with _errmsg
  2.1      05/88    B.Ailts             Replaced errh with _errmsg
                                        Took out eerror.h
  2.2      02/91    B.Ailts             Standardized error messages
  2.3      05/92    T. Mittan           Output more descriptive message
                                        when not enough room is available on a
                                        disk on VMS
  2.4      10/92    T. Mittan           Added compression option
  2.5      12/92    T. Mittan           Modified memory allocation for 
                                        fd->foffset

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
If write access
   Open a new file
Else
   Open an existing file

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "locinc.h"

extern short glun;

FUNCTION las50_do_open(fd)
struct FDESC *fd;               /*  file descriptor                     */
{
char errtxt[ERRLEN + 1];
long type, status, org, chans, lines, linsiz, labels, labsiz, lun;
long nonfatal = 1;
long herr;
long j;                        /* loop counter                 */
long num;                      /* temporary holding value      */
char buf[11];                  /* buffer to check type of file */
unsigned char *thebuf1;

glun = 7;

thebuf1 = (unsigned char *)calloc(((fd->nl * fd->nbands * 4) + 20),
sizeof(unsigned char));
if (fd->acc == IWRITE)
    {
    lun = glun++;
    org = ORG;
    chans = fd->nbands;
    lines = fd->nl;
    if (fd->flags & CONVERSION)
        linsiz = fd->ns * datasize[fd->conv_type];
    else
        linsiz = fd->ns * datasize[fd->dtype];
    labels = NLABELS;
    labsiz = LABSIZ;
    
    /* If data compression is desired insert header file
    ---------------------------------------------------*/
    if (fd->compres)
        {
        if ((fd->fp = fopen(fd->fname,"w")) == NULL)
            {
            c_lasmsg(FALSE,"doopen-open","Error opening file");
            return(E_FAIL);
            }
        /* Insert header file
        --------------------*/
        (void)strcpy(thebuf1,"Compressed");
        num = 10;
        fwrite(thebuf1,sizeof(char),num,fd->fp);
        fseek(fd->fp,14,0);
        num = 1;
        fwrite(&num,sizeof(long),1,fd->fp);
        for ( j = 0; j < (fd->nl * fd->nbands) * 4; j++)
            thebuf1[j] = 0;
        num = (fd->nl * fd->nbands) * 4;
        fwrite(thebuf1,sizeof(char),num,fd->fp);
        fseek(fd->fp,10,0);
        fwrite(&fd->nbands,sizeof(long),1,fd->fp);
        fseek(fd->fp,0,2);
        }
    else
        {
        status = i_opou(&(fd->ifcb), lun,fd->fname, org, chans, lines,
        				linsiz, labels, labsiz);
        if (status != SUCCESS)
            {
            herr = i_herr(&(fd->ifcb));
#ifdef VMS
            if (herr == 99652 )
                sprintf(errtxt,"Not enough disk space for file %s",
                fd->fname);
            else
                sprintf(errtxt,"Error %d encountered opening file %s",
                herr,fd->fname);
#else
            sprintf(errtxt,"Error %d encountered opening file %s",
            herr,fd->fname);
#endif
            c_lasmsg(FALSE,"doopen-open",errtxt);
            return(E_FAIL);
            }
        }
    }
else
    {
    lun = glun++;
    if (fd->acc == IREAD)
        type = I_INPUT;
    else
        type = I_INOUT;
    
    /* open file and check to see if it is in compresssed format
    ----------------------------------------------------------*/
    if ((fd->fp = fopen(fd->fname,"r")) == NULL)
        {
        c_lasmsg(FALSE,"doopen-open","Error opening file");
        return(E_FAIL);
        }
    if (fread(buf,sizeof(char),11,fd->fp) == E_FAIL)
        {
        c_lasmsg(FALSE,"doopen-read","Error reading file");
        return(E_FAIL);
        }
    buf[10] = '\0';
    if (strcmp(buf,"Compressed") != 0)
        {
        fclose(fd->fp);
        status = i_opin(&(fd->ifcb), lun,fd->fname, type);
        if (status != SUCCESS)
            {
            herr = i_herr(&(fd->ifcb));
            c_lasmsg(FALSE,"doopen-open","Error %d encountered opening file %s",
            herr,fd->fname);
            return(E_FAIL);
            }
        }
    else
        {
        fd->compres = TRUE;
        fseek(fd->fp,10,0);
        fread(&fd->mbands,sizeof(long),1,fd->fp);
        if (fd->nbands > fd->mbands)
            fd->foffset = (long *)calloc(fd->nbands, sizeof(long));
        else
            fd->foffset = (long *)calloc(fd->mbands, sizeof(long));
        }
    }
free(thebuf1);
return(E_SUCC);
}
