/*
************************************************************************
NAME: las50_open_img

PURPOSE: open the image at the proper location within the image file
         (random access) WHEN APPROPRIATE.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1           11/89   T. Baltzer   
  2.4           02/91    B.Ailts        Standardized error messages
  3             10/92    T. Mittan      Do not allow images to be opened
                                        for compression; because they will
                                        use UPDATE mode.



COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:
         Open the image at the proper location within the image file
         WHEN APPROPRIATE.  A close on the image may be required first
         (the file was previously opened to some other location within
         the image file).
         All bands are opened simultaneously for the image under
         consideration.

*****************************************************************
        arguments:
        name            description
        ----            -----------
        fdesc           file descriptor (output)
        host            host name of image (input)
        openflg         open flag specifier (input)
                           0 - no opening of image required
                           1 - open image for the first time
                           2 - close image file, reopen it
        bands           number of bands in image to be opened (input)
                        number of bands in image that were opened (output)
        ssrw            starting sample of read window (input)
        nlrw            number of lines in the read window (input)
        nsrw            number of samples in the read window (input)
        nlbuf           number of lines in the total buffer (input)
        access          access type for eopenr call (input)
        dtype           data type that image data should be converted 
                        to while in RAM (input/output)
        wtype           data type that data should be converted 
                        to and written as (input).
                        ( 0 if no conversion needed, or reading data)

************************************************************************
*/
#include "worgen.h"
#include "las.h"

int  las50_open_img(fdesc,host,openflg,bands,ssrw,nlrw,nsrw,nlbuf,access,
              dtype,wtype)

        int     *fdesc;
        char    host[];
        long    openflg;
        long    *bands;
        long    ssrw;
        long    nlrw;
        long    nsrw;
        long    nlbuf;
        long    access;
        long    *dtype;
        long    wtype;

{

long    status;         /* return status from functions */
long    inf_err = 1;    /* non-fatal error status       */
long    firstl;         /* first line to open to        */
long    nonfatal = 1;
long    opt[3];         /* write type array             */


opt[0] = wtype;
opt[1] = NO_COMPRES;
opt[2] = 0;

if (openflg > 0)     /* open is required */
    {
    if (openflg == 2)    /* need to close image before reopening */
        {
        if ( las50_c_eclose(fdesc) == E_FAIL )
            {
            c_lasmsg(FALSE,"openimg-call","Error returned from eclose");
            return(E_FAIL);
            }
        }
    firstl = 1;            /* always open relative to line one */
    status = las50_c_eopenr(fdesc,host,bands,&firstl,&ssrw,&nlrw,&nsrw,
    &access,dtype,opt,&nlbuf);
    if (status  ==  E_FAIL)
        {
        c_lasmsg(FALSE,"openimg-call","Error returned from eopenr");
        return(E_FAIL);
        }
    }

return(E_SUCC);

}     /* end of las50_open_img()   */

