/*******************************************************************************

NAME:                           TRANSLAS5

PURPOSE:    Copies an image into another image.  It allows the input 
            image(s) to be sub-windowed and sub-banded when copied to 
            the output image.  The input image may be BYTE, INTEGER*2, 
            INTEGER*4, or REAL*4.  The output image may be converted 
            to any one of the four data types (the default is no data 
            conversion).

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
 6.0       T.Wolbrink      09/95        EDC/CSB         Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   TRANSLAS5 must be run under TAE.

PROJECT:                        LAS

ALGORITHM DESCRIPTION:
        The input parameters and globals are retrieved.
        Error checking is done on the input image(s) to ensure that they all 
        have equal window sizes.  
        Each line of the input image is copied to the corresponding line of the
        output image.  
        The associated history file is created to represent the output 
        image.
        A completion message is written.

ALGORITHM REFERENCES:           none

******************************************************************************/

#include <stdio.h>
#include "las.h"        

main()
{
struct PARBLK vblock;           /* TAE parameter block               */

static long bands[MAXIMG][MAXBND+1];/* bands to be processed - all images*/
char    comp[4];                        /* compression flag                  */
long    msgtim;                         /* elapsed time between display msg  */
long    nbands[MAXIMG];                 /* number of bands per input image   */
long    nmrimg;                         /* number of input images            */
long    odtype;                         /* data type of output image         */
long    one = 1;                        /* value of one                      */
long    status;                         /* function return status code       */
long    std_out;                        /* LUN to standard output            */
long    totbnd;                         /* total number of bands             */
long    window[MAXIMG][4];              /* window for each input image       */

char    hosin[MAXIMG][CMLEN];           /* host names of input images        */
char    hosout[CMLEN];                  /* host name of output image         */
char    minmax[4];                      /* min max flag                      */
char    taein[MAXIMG][CMLEN];           /* TAE names of input images         */
char    taeout[CMLEN];                  /* TAE name of output image          */

struct INPUT *input;					/* Array of input specs				 */

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,taein[0],hosin[0],taeout,hosout,comp, &nmrimg,window,bands
,nbands, &totbnd,&odtype,&msgtim,minmax);

/* Process image 
------------------*/
proces(&vblock,hosin[0],hosout,comp,nmrimg,window,bands,nbands,totbnd,
       odtype,msgtim,minmax);

/*  Update history file
-----------------------*/
if (c_hstout(&vblock,hosout,hosin[0],&nmrimg," ") != E_SUCC)
    c_lasmsg(FALSE,"translas5-warn","Nonfatal error encountered");

/* Output completion message
----------------------------*/
c_complt("TRANSLAS5",taeout,&one);
}
