#ifndef CM_H
#define CM_H

#include "tae.h"
#include "imageio.h" /* MAXBND definition */
#include "desc.h"


/*****************************************************************************
 *                                                                           *
 *      This is the include file for the 'cm' routines.                      *
 *                                                                           *
 *      Modified by D. Akkerman, USGS/EROS Data Center, 5/90.                *
 *       - Added the TAE_UPATH define for changes to the cmhost and          *
 *         cmtae routines.                                                   *
 *                                                                           *
 *      Modified by B. Davis, USGS/EROS Data Center, 9/90.                   *
 *       - added the cm tape structures and defines for cmstor/cmrestor.     *
 *                                                                           *
 *      Modified by D. Etrheim, USGS/EROS Data Center, 11/90.                *
 *       - added online/offline/both defines                                 *
 *****************************************************************************/

#define TAE_EXT  ';'
#define HOST_EXT '.'
#define TAE_SEP  '.'
#define HOST_SEP '_'

#define TAE_DIR_BEG '['
#define TAE_DIR_SEP '.'
#define TAE_DIR_END ']'
#define TAE_PARENT  '-'
#define TAE_UPATH   '%'
#ifdef vms
#define HOST_DIR_BEG '['
#define HOST_DIR_SEP '.'
#define HOST_DIR_END ']'
#define HOST_PARENT  '-'
#else
#define HOST_DIR_BEG '/'
#define HOST_DIR_SEP '/'
#define HOST_DIR_END '/'
#define HOST_PARENT  '.'
#endif

/* Define the use of the external compression environment variable.
*/
#define EXT_COMP 99

#define MAXFILES 100                    /* maximum number of files to be
                                           associated with and image,
                                           in order to help limit the
                                           size of array declarations.  */

#define MAXTPWT 32768                   /* maximum number of bytes that
                                           can be streamed together and
                                           read/written to/from a single
                                           buffer by a single read/write.
                                           This value is the smallest one
                                           of the current systems,
                                           sun-bsd.  If LAS is ported to
                                           a new different machine with a
                                           smaller MAX, this value will
                                           need to be adjusted.         */


/* Structure for parse list. */
struct  PRS_LST
    {
    char    name[CMLEN+1];          /* image name */
    double  window[4];              /* image window */
    long    bands[MAXBND+1];        /* image bands */
    long    proj[MAXBND+1];         /* image bands */
    long    qflag[2];               /* window and band flags */
    };


int antoi
(
    char *srce,        /* source string to convert     */
    long *sz,          /* number of chars to convert   */
    long *value        /* decimal value to return      */
);

void bldname
(
    char *fname,        /* current host filename (i)    */
    char *newspec,      /* new host name specified (i)  */
    char *newname       /* new filename (no wildcard)(o)*/
);

int c_cmhost
(
    char *tae,          /* Input TAE file name                       */
    char *hname         /* Output host file name                     */
);

int c_cmhostcs
(
    char *tae,          /* Input TAE file name                       */
    char *hname         /* Output host file name                     */
);

int c_copy_image
(
  struct PARBLK *vblock,         /* I: TAE parameter block                */
  char hosin[][CMLEN],           /* I: Filenames of input images (Unix)
                                       hosin may be a single- or multi-band
                                       image and may contain BYTE, I*2,
                                       I*4, or R*4 pixel data.            */
  char hosout[CMLEN],            /* I: Filename of output image (Unix)    */
  int comp,                      /* I: Compression flag
                                       TRUE = Compress output image
                                       FALSE = Don't compress output image
                                       EXT_COMP = Use external compres_opt
                                         or COMPRES1 environment variable */
  long nmrimg,                   /* I: Number of input images             */
  long window[][4],              /* I: Window for each input image
                                       (SL, SS, NL, NS)
                                       If all values are 0s then the entire
                                       image will be processed.  If the
                                       images are not of the same size then
                                       processing will be stopped.        */
  long bands[][MAXBND+1],        /* I: Bands to process - all images
                                       If all 0s then all bands in all the
                                       images will be processed.  nbands
                                       also needs to be set to 0 for the
                                       default.                           */
  long nbands[],                 /* I: Number of bands per input image
                                       If bands are all 0s and nbands[i]
                                       is 0 then all bands in all the
                                       images will be processed           */
  long totbnd,                   /* I: Total number of bands.  Set to 0 if
                                       processing the default bands       */
  long odtype,                   /* I: Data type of output image
                                       (0=SAME, EBYTE=BYTE, EWORD=I*2,
                                        ELONG=I*4, EREAL=R*4)             */
  int rtopt_in,                  /* I: Round/truncation option
                                       TRUE = Round the input data
                                       FALSE = Truncate the input data    */
  long msgtim,                   /* I: Elapsed time between display msg
                                       for TAE                            */
  char minmax[4]                 /* I: Min max flag for TAE               */
);

int c_getfiles
(
    char *filespec,          /* Host file specification for files to find  */
    char file_names[][CMLEN],/* Array of file names to be output          */
    long *nfiles,            /* Number of files found matching filespec    */
    long maxfiles,           /* Max number of files to return at one time  */
    long first_call,         /* Flag for first call to this function (0,1) */
    long ck_lock             /* 1 = return list of unlocked files only,    */
                             /* 0 = return list of all files found         */
);

int c_getfiles
(
    char *filespec,          /* Specification of desired files to find     */
    char file_names[][CMLEN],/* Names of all the files found              */
    long *nfiles,            /* Number of files found                      */
    long maxfiles,           /* Max number of files to report              */
    long startf,             /* Flag for first call or not (0,1)           */
    long ck_lock             /* 1 = return list of unlocked files only,    */
);

int cmlock
(
    char *hname,             /* host name of file to open    */
    int *fd,                 /* file descriptor              */
    int flags,               /* file access                  */
    int maxtime              /* max time to wait (seconds)   */
);

int cmtaedir
(
    char *hostdir,           /* host directory specification */
    char *taedir             /* TAE directory specification  */
);

int copyfile
(
    char *hname,             /* host name of existing file   */
    char *hnewname,          /* host name of new file        */
    long report              /* report file?                 */
                             /* (0=tae,errs only, 1=taename, */
                             /* 2=host,errs only,3=hostname) */
);

int deletefile
( 
    char *hname,             /* host name of file    */
    char *type,              /* "file" or "dir"      */
    long verify,             /* verify? (yes=1)      */
    long report              /* report it? (yes=1)   */
);

int dirlist
(
    char *nameoffile,        /* pointer to temp file of file names */
    char *filespec,          /* input spec                         */
    char file_ary[][CMLEN],  /* array of file names to be output   */
    long *nfiles,            /* number of files generated in list  */
    long maxfiles,           /* max number to list at a time       */
    long startf              /* flag for first call                */
);

int dirnm
(
    char *hnam               /* pointer to host path name                  */
);

int get_assoc
(
    char *tmpnam,               /*ptr to name for temp file passed to fileutil*/
    char *host,                 /* pointer to host image name                 */
    char (*assoc_array)[CMLEN], /*pointer to array for files assoc with img   */
    long *nassoc                /* pointer to number of associated files      */
);

int getcredate
(
    char *hostname,           /* Host file name               */
    char *credate             /* Date host file created       */
);

int get_def_dir
(
    char *usercode,
    char *dir
);

int getdirs
(
    char *filespec,           /* host description of whole path and file name */
    char directories[][CMLEN],/* array of directory names that are found      */
    int *ndirs,               /* number of directories found                  */
    int maxdirs,              /* maximum number of directories to look for    */
    int startf                /* number of directories already found          */
);

int getmydef
(
    char *dir
);

int getprnm
(
    char *hostnm,             /* ptr to host file name        */
    long *nmlen               /* pointer length of file name allowed  */
);

char *gettoken
(
    char *string,
   char *delim
);

int isassoc
(
    char *hnam,               /* pointer to host file name                  */
    char *ext                 /* pointer to desired assoc file extension    */
);

int mkdr
(
    char *hostnm              /* pointer to directory path    */
);

void processfile
(
    struct PARBLK *vblock,    /* ptr to TAE param block*/
    char *func,               /* function to perform  */
    char *name,               /* current filename     */
    char *newname,            /* new filename         */
    long imgset,              /* img & assoc? (1=yes) */
    long imageflg,            /* Process image file   */
    char aftype[][CMLEN],     /* Assoc file types     */
    long *acount,             /* # assoc file types   */
    long verify,              /* verify? (1=yes)      */
    long *nproc,              /* # files processed    */
    long copyflg              /* flag if image data is copied or renamed    */
);

int renamefile
(
    char *hname,              /* current host name    */
    char *hnewname,           /* new host name        */
    long verify,              /* verify? (1=yes)      */
    long report,              /* report file? (1=yes) */
    long copyflg              /* flag if image data is copied or renamed    */
);

int unlock
(
    int *fd,                  /* file descriptor              */
    int maxtime               /* max time to wait (seconds)   */
);

BOOL isdir
(
   char *path                 /* host name            */
);

int isimage
(
    char *taen               /* pointer to tae name                        */
);

#endif
