#ifndef LASTAE_H
#define LASTAE_H

/**********************************************************************
 *
 *  This is an aggregate include file for TAE applications.
 *   
 ********************************************************************** */

#include "taeconf.inp"
#include "parblk.inc"
#include "terminc.inc"

void c_errmsg
(
    char *message,      /* Message associated with the error */
    char *key,          /* LAS error key or empty string */
    long *vrity         /* Severity: */
                        /*    <= 0   fatal */
                        /*     > 0   non-fatal */
);

int c_lasmsg
(
   int fatal_flag,
   const char *key,
   const char *fmt,
   ...
);

/* NOTE: the string returned by squeeze should be free'd by the calling
         routine to prevent a memory leak */
char *squeeze
(
   const char *str,
   int len
);
 
/* TEMPORARY TAE function prototypes
--------------------------*/
int m_msg
(
    char *message,  /* I: message to be logged */
    char *key       /* I: message key */
);

CODE t_init
(
   COUNT *lines,    /* out: number of lines on crt*/
   COUNT *columns,  /* out: number of columns */
   CODE  *type      /* out: type of terminal: T_CRT, T_NOTCRT, T_NOTTERM */
);

CODE t_pos
(
   FUNINT line,     /* in: line number (start at 1)   */
                    /*  -1 means use current line */
   FUNINT column    /* in: column number (start at 1) */
);

CODE t_output
(
    FUNINT line,    /* in: line number (start at 1)   */
    FUNINT column,  /* in: column number (start at 1) */
    TEXT   string[] /* in: string to write  */
);

CODE t_read
(
    TEXT string[STRINGSIZ+1],   /* out: string read */
    CODE *term                  /* out: terminator used:*/
                                /*      T_CR (carriage return)*/
                                /*      T_ESCAPE (escape)*/
);

int t_write
(
  char *,
  long
);

void t_bell(void);

void t_clear
(
  void
);

int t_highlight
(
    char *string /* I/O: string to highlight.  Make sure there is extra space
                         to attach escape characters. */
);

void q_init
(
    struct PARBLK *p,   /* I/O: PARBLK to initialize */
    int pool_size,      /* I: bytes allocated in the memory pool */
    int mode            /* I: P_ABORT or P_CONT */
);

int q_dynp
(
    struct PARBLK *p,   /* I: PARBLK to send */
    char *pdfspec,      /* I: pdf file spec */
    int  mode           /* I: M_SUBPDF or M_FULLPDF */  
);

int p_inim
(
    struct PARBLK *block,   /* I/O: parm block to be initialized */
    int blksiz,             /* I: size of block in bytes */
    int mode                /* I: P_ABORT or P_CONT */
);

CODE t_input
(
    FUNINT line,    			/* in: line number (start at 1)   */
    FUNINT column,              /* in: column number (start at 1) */
    TEXT   string[STRINGSIZ+1], /* out: string typed*/
    CODE   *term                /* out: terminator: */
                                /* T_CR or T_ESCAPE*/
);

int t_lclear
(
    int line,  /* I: line to clear (-1 = current line) */
    int col    /* I: start clearing at this column */
);
/* t_lclear returns SUCCESS when it succeeds (all the time) */

struct VARIABLE *p_find
(
  struct PARBLK *,
  char *
);

void p_inifor
(
  struct PARBLK *
);


int p_forp
(
   struct PARBLK *,
   char *,
   int
);


int q_intg
(
    struct PARBLK *p,      /* I: V-block                  */
    TEXT name[],           /* I: variable name            */
    FUNINT count,          /* I: count of variable        */
    TAEINT intg[],         /* I: value vector             */
    FUNINT mode            /* I: P_UPDATE or P_ADD        */
);

int q_real
(
    struct PARBLK *p,      /* I: V-block                  */
    TEXT name[],           /* I: variable name            */
    FUNINT count,          /* I: count of variable        */
    TAEFLOAT real[],       /* I: reals                    */
    FUNINT mode            /* I: P_UPDATE or P_ADD        */
);

int q_string
(
    struct PARBLK *p,      /* I: V-block                                  */
    TEXT name[],           /* I: variable name                            */
    FUNINT count,          /* I: count of vector (0 means set count = 0)  */
    TEXT *vector[],        /* I: vector of string ptrs                    */
    FUNINT mode            /* I: P_UPDATE or P_ADD                        */
);

int q_out(struct PARBLK *p);

void z_exit
(
    int sfi,      /* I: return code */
    char *skey    /* I: return key */
);

#endif
