/******************************************************************************
FUNCTION:       bldname

PURPOSE:        given the name of an existing file, and the user-supplied
                file specification of the output file (possibly including
                wildcard characters), bldname constructs and returns an
                output file name with no wildcard characters.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            include file changes
  5.0      04/88    D. Hollaren         changed include spec to las


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Find the directory, file, and extension parts of the user-supplied output file
specification.
if no wildcards were specified
   return the user-supplied name,
else if the file part is a wildcard
   use the file part from the existing input file,
else if the extension part is a wildcard
   use the extension from the existing input file.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <string.h>
#include "las.h"
#include "cm.h"

void bldname
(
    char *fname,            /* current host filename (i)    */
    char *newspec,          /* new host name specified (i)  */
    char *newname           /* new filename (no wildcard)(o)*/
)
{
int
    i;                                  /* index                        */

char
    c,                                  /* directory delimiter character*/
    newdirpart[CMLEN],                  /* directory part of new name   */
    newextpart[40],                     /* extension part of new name   */
    newfilepart[40],                    /* file part of new name        */
    oldextpart[40],                     /* extension part of new name   */
    oldfilepart[40],                    /* file part of new name        */
    *p;                                 /* temporary pointer            */

   
/************************************************************************/

/***  construct the filename  ***/

#ifdef vms
    c = ']';                                                    /* VMS  */
#else
    c = '/';                                                    /* unix */
#endif

if ((p = strrchr (newspec,c)) == NULL)          /* isolate file, dir    */
    {                                           /* of newspec           */
    strcpy (newdirpart,"");                     /* no directory         */
    strcpy (newfilepart,newspec);               /* isolate filepart     */
    i = strcspn (newfilepart,".");
    newfilepart[i] = 0;
    }
else
    {
    i = (strlen(newspec) - strlen(p)) + 1;      /* isolate directory    */
    strncpy (newdirpart,newspec,i);
    newdirpart[i] = 0;
    strcpy (newfilepart,++p);                   /* isolate filename     */
    i = strcspn (newfilepart,".");
    newfilepart[i] = 0;
    }

p = strrchr (newspec,'.');                      /* isolate ext of newspec*/
if (p == NULL) 
    {
    strcpy (newextpart,"*");                    
    }
else
    {
    strcpy (newextpart,++p);                    
    }


if ((strcmp(newfilepart,"*") != 0) &&           /* both filepart and    */
    (strcmp(newextpart,"*") != 0))              /* extension specified: */
    {
    strcpy (newname,newspec);                   /* use specified name   */
    }

else if (strcmp(newfilepart,"*") == 0)          /* filepart is "*"      */
    {
    if ((p = strrchr (fname,c)) == NULL)        /* isolate file part    */
        strcpy  (oldfilepart,fname);            /* of fname             */
    else
        strcpy (oldfilepart,++p);
    i = strcspn (oldfilepart,".");
    oldfilepart[i] = 0;

    strcpy (newname,newdirpart);                /* use new directory    */
    strcat (newname,oldfilepart);               /* use old filepart     */
    strcat (newname,".");
    strcat (newname,newextpart);                /* use new extension    */
    }

else if (strcmp(newextpart,"*") == 0)           /* extension is "*"     */
    {   
    p = strrchr (fname,'.');                    /* isolate extension    */
    strcpy (oldextpart,++p);                    /* of fname             */

    strcpy (newname,newdirpart);                /* use new directory    */
    strcat (newname,newfilepart);               /* use new filepart     */
    strcat (newname,".");
    strcat (newname,oldextpart);                /* use old extension    */
    }
}
