/******************************************************************************
FUNCTION:       copyfile

PURPOSE:        Makes a copy of the specified file with the specified name.
                E_SUCC is returned if the copy is successful, E_FAIL otherwise

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
            6/87    K. Zanter           Changed so that if the
                                        file does exist on disk,
                                        it is not overwritten.
  1.1      12/87    B. Ailts            change calls to cmtae
                                        to reflect DESC to string
                                        changes
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed staus code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0      09/88    B. Davis            fixed bug, addr of char arrays were
                                        being sent to cmtae
  5.0      11/88    B. Davis            system calls to dk calls, also brought
                                        up to 5.0 standards
  5.1      01/91    D.Etrheim           Standardized error message handling
                                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
If the TAE name is to reported to the user
    get the TAE file names from the host file names.
Make sure that the output host file doesn't already exist.
Copy the input host file to the output host file.
Check the report flag 
    report to the user the TAE file name or the host file name of the file
    copied.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "cm.h"

int copyfile
(
    char *hname,            /* host name of existing file   */
    char *hnewname,         /* host name of new file        */
    long report             /* report file?                 */
                            /* (0=tae,errs only, 1=taename, */
                            /* 2=host,errs only,3=hostname) */
)
{
long
    nonfatal=1;                         /* nonfatal error code          */

long
    status,                             /* status returned              */
    zero=0;                             /* zero (pass to lsstat)        */

char
    cmdstring[ERRLEN],                  /* error message text string    */
    name[CMLEN],                        /* TAE name of existing file    */
    newname[CMLEN],                     /* TAE name of new file         */
    temp[CMLEN];                        /* temporary character array    */
   
/************************************************************************/

strcpy (name,hname);
strcpy (newname,hnewname);

if (c_lsstat(hnewname,&zero) == E_SUCC)                 /* file exists, */
    {                                                   /* don't copy   */
    sprintf (temp,"Output file already exists: %s",newname);
    c_errmsg (temp,"copyfile-exist",&nonfatal);
    return(E_FAIL);
    }

status = c_dkcopy(hname,hnewname,NULL);
if (status != E_SUCC)
    {
    c_errmsg("Error returned from c_dkcopy","copyfile-call",&nonfatal);
    sprintf (cmdstring,"Error copying file %s to %s",name,newname);
    c_errmsg (cmdstring,"copyfile-copy",&nonfatal);
    }

if ((status == E_SUCC) && ((report==1)||(report==3)))
    {
    sprintf (temp,"File created: %s",newname);
    c_errmsg (temp," ",&nonfatal);
    }

return (status);
}
