/******************************************************************************
FUNCTION:   DIRLIST

PURPOSE:    Dirlist handles wildcarded file names and returns a list of the
            file specified by the filename.

PROGRAM HISTORY:
Version  Author        Date   Change Request
-------  ------------  -----  --------------
  ???    T.Butzer      03/86  Original Implementation
  ???    B.Ailts       12/87  Changed the include file specifications
  ???    K.Gacke       04/88
  5.0    D.Hollaren    04/88  changed include spec to las changed cerrmsg to
                              c_errmsg and pass status code by address return
                              status of E_SUCC/E_FAIL
  5.0    B.Davis       09/88  conversion to 5.0, added an argument to allow 
                              temp files to remain hidden, fixed bugs
  5.0    B.Davis       01/89  Fixed unix bug of returning nfiles of 0 and 
                              status of E_FAIL when file(s) not found.  Now
                              returns nfiles of 0 and status of E_SUCC.
  5.0    B.Davis &
         D.Hollaren    01/89  Upcased vms exentsion before comparision.
  5.0    J.McGuthry    02/89  added a condition such that if compiled on IBM
                              AIX getcwd is executed else getwd is executed
                              ( get working directory )
  5.1    D.Etrheim     01/91  Standardized error message handling
  5.1    J.Fenno       09/91  Fixed index problem when reading the files names
                              in the UNIX version.
  5.2    T.Ratliff     06/97  Removed the goto statements.  Also added 
                              c_getfiles to handle wildcards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Under UNIX;
    Open a tempory file,
        get the file names of the specified directory
        put files of the directory into the temporary file.
    After all file names are in the temporary file
        read the temp file placing nulls at the end of each string
        place the file names into the return array of file names.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "las.h"
#include "labserv.h"
#include "cm.h"

int dirlist
(
    char *nameoffile,        /* pointer to temp file of file names */
    char *filespec,          /* input spec                         */
    char file_ary[][CMLEN],  /* array of file names to be output   */
    long *nfiles,            /* number of files generated in list  */
    long maxfiles,           /* max number to list at a time       */
    long startf             /* flag for first call                */
)
{
long count;                 /* counter for nfiles                 */
long exists;                /* flag for file existence            */
long i = 0;                 /* char array index                   */
long j = 0;                 /* counter variable                   */
long len;                   /* length of hname                    */
long n_files=0;             /* file_array index                   */
long nonfatal = 1;          /* nonfatal error                     */    
long rstat=-99;             /* return status and condition check  */

short c;   /* char return of fget */

char *ptr,*p;          /* temporary pointer                      */
char temp[CMLEN];      /* temporary string storage               */
char tmpnam[CMLEN];
char sysstr[CMLEN];    /* contains echo filespec for popen       */
char wd[CMLEN];        /* current working directory              */

struct stat buf;       /* usd for lstat info                     */

static FILE *tmpfp;    /* file pointer to file of file names   */

/******************************************************************************/

/* Checking if a wildcard is in the filename
--------------------------------------------*/
p = strrchr(filespec,'*');
if (startf == 1)
   {    

   /* This is setup for processing filenames with no
      wildcards.
   -------------------------------------------------*/
   if (p == NULL)
     {
     sprintf(sysstr,"/bin/echo %s ",filespec);
     tmpfp = popen(sysstr, "r");
     if (tmpfp == NULL)
       {
       c_errmsg("Error opening temporary file","dirlist-open",&nonfatal);
       rstat = E_FAIL;
       }
     } /* end if p == NULL */
   } /* end if startf == 1 */


if (p != NULL)
{
   c_getfiles(filespec,file_ary,&n_files,300,1,0);
   len = (long)strlen(filespec);
   if ((filespec[len-1] == '*') && (filespec[len-2] == '.') &&
       (filespec[len-3] == '/'))
   {
     for (i = 0; i < n_files-2; i++)
       strcpy(file_ary[i],file_ary[i+2]);
     
     for (j = i; j < n_files; j++)
       memset(file_ary[j],0,CMLEN);
     n_files -= 2;
   }
}


/* Processing without wildcards continues on here
-------------------------------------------------*/
else if (rstat != E_FAIL)
   {
   getcwd(wd,CMLEN);

   /* get the full name of the temporary file that will be passed to fileutil,
      this needs to be checked because it will not be around after processing,
      but will show up in a directory listing during processing
---------------------------------------------------------------------------*/
   strcpy(tmpnam,wd);
   strcat(tmpnam,"/");
   strcat(tmpnam,nameoffile);
   n_files = 0;
   count = 0;
   i = 0;

   while ((c=fgetc(tmpfp)) != EOF && (count < maxfiles))
      {
      if(c == ' ' || c == '\n')
         {
         c = '\0';
         file_ary[n_files][i] = c;
         if (strcmp(tmpnam,file_ary[n_files]) != 0)
            {
            ptr = (char *)strchr(file_ary[n_files], '/');
            if (ptr == 0)
               {
               sprintf(temp,"%s/%s",wd, file_ary[n_files]);
               strcpy(file_ary[n_files],temp);
               }
            n_files++;
            count++;
            }
         i = 0;
         }
      else
         {
         if (c == '*')
            {   
            *nfiles = 0;
            rstat = E_SUCC;
            }
         else
            {
            file_ary[n_files][i] = c;
            i++;
            }
         }
      } /* while c != EOF */
   if (rstat != E_SUCC)
      {
      i--;
      file_ary[n_files][i] = '\0';
      }

   }   /* end else of if p != NULL */

   if (rstat != E_SUCC)
      {
      *nfiles = n_files;

      if (*nfiles == 1)
         {
         exists = 0;
         if (c_lsstat(file_ary[0], &exists) != E_SUCC) /*check pointed to file*/
            {
            if (lstat(file_ary[0],&buf) != 0) /* check the symbolic link */
               {
               *nfiles = 0;
               rstat = E_SUCC;
               }
            }
         }
      }
   rstat = E_SUCC;

if ((*nfiles < maxfiles) && (p == NULL))
   pclose(tmpfp);

return (rstat);
}
