/******************************************************************************

FUNCTION:       GET_ASSOC

PURPOSE:        Retrieves the associated files pertaining to an image file.

PROGRAM HISTORY:
  Version       Author       Date    Change Request
  -------       ------       -----   --------------
  5.0           B.Davis      05/89   Original development
  5.1           B.Davis      11/90   get rid of leading device name produced
                                     by genf(ulimately).  subsequent routines
                                     all use cmhost, which will not pick up
                                     device name(it calls getwd ultimately).

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                none.

PROJECT:        LAS 5.0

                                        
FATAL ERROR CONDITIONS: unsuccessful dirlist call.

ALGORITHM(flow):
                change file name from root.ext to root.*
                get a directory list of all file names of root.* and also
                the number of them

ALGORITHM REFERENCES:
                none.
******************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include <string.h>
#include "las.h"
#include "cm.h"

int get_assoc
(
    char *tmpnam,               /*ptr to name for temp file passed to fileutil*/
    char *host,                 /* pointer to host image name                 */
    char (*assoc_array)[CMLEN], /*pointer to array for files assoc with img   */
    long *nassoc                /* pointer to number of associated files      */
)

{
    long nonfatal = 1;          /* mode for errmsg                            */
    long start;                 /* value of one to pass to dirlist            */
    long status;                /* return status of calls                     */

    char *ptr;                  /* pointer to step through name by characters */
    char temp[CMLEN];           /* temp host name for parsing                 */

    start = 1;
    strcpy(temp, host);

    /* change from .img to .* */
    ptr = (char *)strrchr(temp,HOST_EXT);
    ptr++;
    *ptr = '*';
    ptr++;
    *ptr = '\0';

    status = dirlist(tmpnam, temp, assoc_array, nassoc, MAXFILES, start);
    if (status != E_SUCC)
    {
       c_errmsg("Error returned from dirlist","get_assoc-call",&nonfatal);
    }
    return (status);
}
