/******************************************************************************
FUNCTION:       unlock

PURPOSE:        calls flock as does cmlock, but passes LOCK_UN to the opera-
                tion argument.  this means unlock.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      05/90    B. Davis            New Development
  1.1      11/90    D. Etrheim          Close the file on VMS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Open the specified host file.
VMS automatically locks a file so nothing to be done
    error if the file was to be opened for output and it already exists.
In UNIX
   attempt to unlock the file with the unblock option until it gets locked
   or the maximum wait time is reached.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/file.h>
#ifdef sun
#include <unistd.h>
#endif

#include "las.h"
#include "cm.h"

int unlock
(
    int *fd,                /* file descriptor              */
    int maxtime             /* max time to wait (seconds)   */
)
{
    long    status;                         /* return status                */
    int gotit=0;                            /* got the lock? (1=yes)        */

    while ((!gotit) && (maxtime>0))
    {
    #ifdef sun
        if ( lockf(*fd,F_ULOCK,0) == 0)
    #else
        if (flock (*fd,(LOCK_UN)) == 0)
    #endif
            gotit = 1;
        else
        {
            sleep(5);
            maxtime -= 5;
        }
    }


    if (maxtime <= 0)
        return (E_FAIL);

    status = close (*fd);
    if (status == EOF)
        return (E_FAIL); 


    return (E_SUCC);
}
