/*******************************************************************************
NAME			       C_GETBDF

PURPOSE	     Retrieve specified field of the band dependent record

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Apr. 1988		Original development
B. Ailts	      DEC. 1990		Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key retrieve the correct field of the BDDR structure.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"

int c_getbdf
(
   struct BDDR *bddr,    /* I:  input band depenent record               */
   long *key,            /* I:  specified key of field needed            */
   unsigned char *value  /* O:  output value of specified field          */
)

{ /*  getbdf  */

double *dblval;			/* temporary pointer to a double field	      */

long *longval;			/* temporary pointer to a long field	      */
long one = 1;			/* errmsg status value			      */

char *charval;			/* temporary pointer to a char field	      */
char errtxt[ERRLEN + 1];		/* buffer space for errmsg		      */

/*  Assign the temporary pointers the address of the output value argument    
--------------------------------------------------------------------------*/
dblval = (double *) value;
longval = (long *) value;
charval = (char *) value;

/*  Based upon the value of the input key return the value of the corresponding
field of the DDR structure.
------------------------------------------------------------------------------*/
switch (*key)
   {
   case DDBAND: *longval = bddr->bandno;
		break;
   case DDMMV:  *longval = bddr->valid;
		break;
   case DDMIN:	*dblval = bddr->minval;
		break;
   case DDMAX:	*dblval = bddr->maxval;
		break;
   case DDSRC:  strcpy(charval,bddr->source);
		break;
   case DDINST: strcpy(charval,bddr->instrument);
		break;
   case DDDIR:	strcpy(charval,bddr->direction);
		break;
   case DDCDAT: strcpy(charval,bddr->date);
		break;
   case DDCTIM: strcpy(charval,bddr->time);
		break;
   default:	sprintf(errtxt,"%d is an invalid input key",*key);
		c_errmsg(errtxt,"getbdf-key",&one);
		return(E_FAIL);
   } /* switch (*key)  */

return(E_SUCC);
} /*  getbdf  */
