/*******************************************************************************
NAME			       C_GETDDF

PURPOSE	     Retrieve specified field of the band independent record

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Apr. 1988		Original development
B. Ailts	      Dec. 1990		Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key, retrieve the correct field of the DDR structure.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"

int c_getddf
(
   struct DDR *ddr,      /* I:  input band depenent record                */
   long *key,            /* I:  specified key of field needed             */
   unsigned char *value  /* O:  output value of specified field           */
)

{ /*  getddf  */

double *dblval;			/* temporary pointer to a double field	      */

long i;				/* counter				      */
long *longval;			/* temporary pointer to a long field	      */
long one = 1;			/* errmsg status value			      */

char *charval;			/* temporary pointer to a char field	      */
char errtxt[ERRLEN + 1];	/* buffer space for err message		      */

/*  Assign the temporary pointers the address of the output value argument    
--------------------------------------------------------------------------*/
dblval = (double *) value;
longval = (long *) value;
charval = (char *) value;

/*  Based upon the value of the input key return the value of the corresponding
field of the DDR structure.
------------------------------------------------------------------------------*/
switch (*key)
   {
   case DDNL: 	*longval = ddr->nl;
		break;
   case DDNS:  	*longval = ddr->ns;
		break;
   case DDNB:  	*longval = ddr->nbands;
		break;
   case DDDTYP: *longval = ddr->dtype;
		break;
   case DDML: 	*longval = ddr->master_line;
		break;
   case DDMS: 	*longval = ddr->master_sample;
		break;
   case DDVFLG: 
   		for (i = 0; i < DDNVAL; i++)
		   *(longval + i) = ddr->valid[i];
		break;
   case DDPCOD:	*longval = ddr->proj_code;
		break;
   case DDZCOD:	*longval = ddr->zone_code;
		break;
   case DDDCOD:	*longval = ddr->datum_code;
		break;
   case DDSYS: 	strcpy(charval,ddr->system);
		break;
   case DDUNIT:	strcpy(charval,ddr->proj_units);
		break;
   case DDLDAT: strcpy(charval,ddr->last_used_date);
		break;
   case DDLTIM: strcpy(charval,ddr->last_used_time);
		break;
   case DDPCF:	
		for (i = 0; i < 15; i++)
		   *(dblval + i) = ddr->proj_coef[i];
		break;
   case DDUL:	*dblval = ddr->upleft[0];
		*(dblval + 1) = ddr->upleft[1];
		break;
   case DDLL:	*dblval = ddr->loleft[0];
		*(dblval + 1) = ddr->loleft[1];
		break;
   case DDUR:	*dblval = ddr->upright[0];
		*(dblval + 1) = ddr->upright[1];
		break;
   case DDLR:	*dblval = ddr->loright[0];
		*(dblval + 1) = ddr->loright[1];
		break;
   case DDPDY:	*dblval = ddr->pdist_y;
		break;
   case DDPDX:	*dblval = ddr->pdist_x;
		break;
   case DDLINC:	*dblval = ddr->line_inc;
		break;
   case DDSINC:	*dblval = ddr->sample_inc;
		break;
   default:	sprintf(errtxt,"%d is an invalid input key",*key);
		c_errmsg(errtxt,"getddf-key",&one);
		return(E_FAIL);
   } /* switch (*key)  */

return(E_SUCC);
} /*  getddf  */
