/*******************************************************************************
NAME			       C_PUTDDF

PURPOSE	     Place a value in a specified field of the band independent record

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Apr. 1988		Original development
B. Ailts	      Dec. 1990		Updated error messages

PROJECT      LAS

ALGORITHM 
   Based upon the input key, place a value into the correct field of the 
   DDR structure.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"

int c_putddf
(
   struct DDR *ddr,     /* I/O:  input band depenent record               */
   long *key,           /* I:  specified key of field needed              */
   unsigned char *value /* I:  output value of specified field            */
)

{ /*  putddf  */

double *dblval;			/* temporary pointer to a double field	      */

long i;				/* counter				      */
long *longval;			/* temporary pointer to a long field	      */
long one = 1;			/* errmsg status value			      */

char *charval;			/* temporary pointer to a char field	      */
char errtxt[ERRLEN + 1];	/* buffer space for err message		      */

/*  Assign the temporary pointers the address of the output value argument    
--------------------------------------------------------------------------*/
dblval = (double *) value;
longval = (long *) value;
charval = (char *) value;

/*  Based upon the value of the input key return the value of the corresponding
field of the DDR structure.
------------------------------------------------------------------------------*/
switch (*key)
   {
   case DDNL: 	ddr->nl = *longval;
		break;
   case DDNS:  	ddr->ns = *longval;
		break;
   case DDNB:  	ddr->nbands = *longval;
		break;
   case DDDTYP: ddr->dtype = *longval;
		break;
   case DDML: 	ddr->master_line = *longval;
		break;
   case DDMS: 	ddr->master_sample = *longval;
		break;
   case DDVFLG: 
   		for (i = 0; i < DDNVAL; i++)
		   ddr->valid[i] = *(longval + i);
		break;
   case DDPCOD:	ddr->proj_code = *longval;
		break;
   case DDZCOD:	ddr->zone_code = *longval;
		break;
   case DDDCOD:	ddr->datum_code = *longval;
		break;
   case DDSYS: 	strcpy( ddr->system , charval);
		break;
   case DDUNIT:	strcpy( ddr->proj_units , charval);
		break;
   case DDLDAT: strcpy( ddr->last_used_date , charval);
		break;
   case DDLTIM: strcpy( ddr->last_used_time , charval);
		break;
   case DDPCF:	
		for (i = 0; i < 15; i++)
		   ddr->proj_coef[i] = *(dblval + i);
		break;
   case DDUL:	ddr->upleft[0] = *dblval;
		ddr->upleft[1] = *(dblval + 1);
		break;
   case DDLL:	ddr->loleft[0] = *dblval;
		ddr->loleft[1] = *(dblval + 1);
		break;
   case DDUR:	ddr->upright[0] = *dblval;
		ddr->upright[1] = *(dblval + 1);
		break;
   case DDLR:	ddr->loright[0] = *dblval;
		ddr->loright[1] = *(dblval + 1);
		break;
   case DDPDY:	ddr->pdist_y = *dblval;
		break; 
   case DDPDX:	ddr->pdist_x = *dblval;
		break;
   case DDLINC:	ddr->line_inc = *dblval; 
		break;
   case DDSINC:	ddr->sample_inc = *dblval; 
		break;
   default:	sprintf(errtxt,"%d is an invalid input key",*key);
		c_errmsg(errtxt,"putddf-key",&one);
		return(E_FAIL);
   } /* switch (*key)  */

return(E_SUCC);
} /*  putddf  */
