/******************************************************************************

NAME:	CKMNMX

FUNCTION:
	CKMNMX is a function which is called by MINMAX to 
	read the DDR file for the specified image.  

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         5/86       B.Ailts      initial development
    2.0		8/86	   K. Gacke	used the label service routines
    3.0         9/86       K. Gacke     used host file names instead of TAE
					file names
    4.0		7/88	   B.Ailts	LAS 5.0 standards
    5.0        12/90       B.Ailts      Updated error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
For each band specified
   GETBDR is called to retreive the min, max, and valid values of DDR
   Bddr.valid is copied into the valid array
   If valid 
      Bddr.minval is copied into the minval array
      Bddr.maxval is copied into the maxval array

ALGORITHM REFERENCES:

******************************************************************************/
#include <stdio.h>

#include "las.h"

int ckmnmx
(
   char *hname,     /* host image name                                  */
   long *bands,     /* array containing bands that were processed       */
   long nbands,     /* no. of bands processed                           */
   double *minval,  /* pointer to array of minimum pixel values         */
   double *maxval,  /* pointer to array of maximum pixel values         */
   long *valid      /* array containing a flag stating if valid         */
)

{
struct  BDDR  bddr;		/* band ddr structure			     */
long    i;			/* counter -- current band		     */
long    nonfatal = 1;		/* nonfatal error code			     */
long	status;			/* function return status code               */

/*  Retreive the min/max and validity values from the DDR file of all needed 
    bands
----------------------------------------------------------------------------*/
for (i = 0; i < nbands; i++)
   {
   status = c_getbdr(hname,&bddr,&(bands[i]));
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from getbdr","minmax-call",&nonfatal);
      return(E_FAIL);
      }
   valid[i] = bddr.valid;
   if (valid[i] == VALID)
      {
      minval[i] = bddr.minval;
      maxval[i] = bddr.maxval;
      }
   }

return(E_SUCC);
}  /*  ckmnmx  */
