/******************************************************************************
FUNCTION:       c_dkdel

PURPOSE:        Delete the specified file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         added calles to c_errmsg
  5.0      11/88    B. Davis            added sys_errlist, allowed dir deletion
  5.0      10/90    T. Baltzer          fix return status checking, errmsg calls
  5.1      09/91    B. Ailts            fixed a spelling error

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Dynamically allocate space for the symbolic link name and file status struct
  Assume the file name is a symbolic link and try to find it
        If an error is found looking for the link
                return an error
        
  Otherwise; get the status information about the symbolic link file
        If an error occurs getting the status
                return an error
        else
                delete the linked file

  Delete the file name
  Free the space for the symbolic link name and file status structure

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include      "las.h"
#include      "diskio.h"

#define PATHNAMESZ      1024


int c_dkdel
(
    char *filename              /* name of file to be deleted           */
)
{
        long sz;                /* size of symbolic link name           */
        long nonfatal = 1;      /* Nonfatal error code                  */
        long status = E_SUCC;   /* Set successful status code           */

        char msg[ERRLEN];       /* Error message string                 */
        char *linkname;         /* symbolic link name                   */
        struct stat statp;      /* file status structure                */

        
        if ((linkname = malloc(PATHNAMESZ)) == NULL)
            {
            c_errmsg("Error allocating dynamic memory","dkdel-alloc",
                        &nonfatal);
            return(E_FAIL);
            }

        /*
         * assume it is a symbolic link and read it
         */
        if ((sz = readlink(filename,linkname,PATHNAMESZ)) == -1)
            {
            if (errno == ENOENT)                       /* file does not exist */
                {
                status = DISK_GONE;
                }
            else if (errno != ENXIO && errno != EINVAL) /* readlink error */
                {
                sprintf(msg,"Error reading the symbolic link for %s",
                        filename);
                c_errmsg(msg,"dkdel-symlink",&nonfatal);
                return(E_FAIL);
                }
            }
        else                                    /* file is a symbolic link */
            {
            linkname[sz] = '\0';                /* this is stupid! */

            if (lstat(linkname,&statp) == -1)   /* status of the linked file*/
                {
                if (errno != ENOENT)            /* status error */
                    {
                    sprintf(msg,"Error getting status of linked file %s",
                        linkname);
                    c_errmsg(msg,"dkdel-stat",&nonfatal);
                    return(E_FAIL);
                    }
                }
            else
                if ( unlink(linkname) == -1 )   /* delete the linked file */
                   {
                   sprintf(msg,"Error deleting the file %s",linkname);
                   c_errmsg(msg,"dkdel-delete",&nonfatal);
                   return(E_FAIL);
                   }
            }

        if (status != DISK_GONE)
            {
            /* Delete the file          */
            if ((status = unlink(filename)) == -1 && errno != ENOENT)
               {
               if (errno == EPERM)
                  {
                  status = rmdir(filename);
                  if (status != E_SUCC)
                     {
                     sprintf (msg,"%s",strerror(errno));
                     c_errmsg (msg,"dkdel-direct",&nonfatal);
                     status = E_FAIL;
                     }
                  }
               else

                  {
                  sprintf(msg,"Error deleting the file %s", filename);
                  c_errmsg(msg,"dkdel-delete",&nonfatal);
                  status = E_FAIL;
                  }
               }
             }

        free(linkname);

        return(status);
}
