/******************************************************************************
FUNCTION:       c_dkmove

PURPOSE:        Move a file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0                                   Original development
  5.0      05/88    D. Hollaren         changed to use c_errmsg
  5.0      10/90    T. Baltzer          check return status's, add errmsg calls

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  If the destination name is in the same file system,
     the move can be done by simply creating a new directory link and
     removing the old link.

  If the link fails because the destination is in a different file system,
     we copy the data to the new file and
     remove the old file


ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include        <errno.h>

#include        "las.h"
#include        "diskio.h"


int c_dkmove
(
    char *from,                 /* name of file to be moved             */
    char *to,                   /* name of output file                  */
    char directories[][CMLEN]   /* list of destination directories      */
)
{
        long nonfatal = 1;      /* Non-fatal error code                 */
        char msg[ERRLEN];       /* error message string                 */

again:
        /* Create the new link                  */
        if (link(from,to) == 0)
            {
            /* Remove the input file            */
            switch ( c_dkdel(from) )
               {
               case DISK_GONE: sprintf(msg,"Error deleting the file %s",from);
                               c_errmsg(msg,"dkmove-delete",&nonfatal);
               case E_FAIL:    return(E_FAIL);
               case E_SUCC:    break;   /* This break terminates all cases */
               }
            }
        else
            {
            /*
             * If the link failed because the destination is
             * on a different filesystem, we have to copy
             * the data.
             */
            if (errno == EXDEV)
                {
                /* Copy the data from the input file to the output file  */
                switch ( c_dkcopy(from,to,directories) )
                   {
                   case DISK_NOSPACE:
                   case DISK_GONE: sprintf(msg,"Error copying the file %s",
                                           from);
                                   c_errmsg(msg,"dkmove-copy",&nonfatal);
                   case E_FAIL:    return(E_FAIL);
                   case E_SUCC:    break; /* This break terminates all cases */
                   }

                /* Remove the input file        */
                switch ( c_dkdel(from) )
                   {
                   case DISK_GONE: sprintf(msg,"Error deleting the file %s",
                                           from);
                                   c_errmsg(msg,"dkmove-delete",&nonfatal);
                   case E_FAIL:    return(E_FAIL);
                   case E_SUCC:    break; /* This break terminates all cases */
                   }
                }

            /*
             * If the link failed because there is already a
             * directory link by that name, remove the existing
             * link and try again.
             */
            else if (errno == EEXIST)
                {
                if (unlink(to) < 0)
                    {
                    sprintf(msg,"Error unlinking the file %s",to);
                    c_errmsg(msg,"dkmove-unlink",&nonfatal);
                    return(E_FAIL);
                    }
                goto again;
                }
            /*
             * If the link fails for any other reason, punt.
             */
            else
                {
                sprintf(msg,"Error moving the file %s",from);
                c_errmsg(msg,"dkmove-move",&nonfatal);
                return(E_FAIL);
                }
            }

        return(E_SUCC);
}
