/******************************************************************************
NAME:		c_gofrd									      *
PURPOSE:  Read the specified type of record from the GOF and put	
	  the information into the defined structure.

PROGRAM HISTORY:
  Version	Date	Contrib		Change
  -------	----	-------		------
   1.0		3/86	L. Huewe	Intial code
   2.0		5/92	L. Huewe	Modified to be a more generic
					WORLD support routine.
   2.1		1/95 	D. Eichacker	To read a full polygon or line 
					two read are required the 1st 
					call to this function should be 
					with HPOLIN the 2nd with XYPAIRS.

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "gof.h"

int c_gofrd 
(
    int *fd,                /* File descriptor of the GOF to read (I). */
    int *obyte,             /* Number of bytes read (O).*/
    int *nbyte,             /* Number of bytes to next record (O).*/
    int *type,              /* Type of record being read (I).*/
    union GOF *gofptr       /* Pointer to the proper structure (O). */
)
{

struct 	RWHDR	arec;		/* Attribute record structure		      */

struct 	PPLGOF	buf;		/* Point, polygon or line GOF record 
				   structure				      */
struct 	ANOT1	a1buf;		/* Character portion of annotation record
				   structure				      */
struct 	ANOT2	a2buf;		/* Float portion of annotation record
				   structure				      */

	float	xypair[2]; 	/* XY pair for polygon or line.		      */

	int	clen,		/* Character length			      */
		dlen,		/* Data length				      */
		i,		/* Loop index				      */
		numbyt;		/* Number of bytes read			      */
    long one = 1;	/* Indicates nonfatal error		      */

/*
   Read the attribute definition record.
*/ 
	if (*type == HDR)
	    {
    	    *obyte = read (*fd, &arec, HDRLEN);
    	    if (*obyte < 0)
	        {
	        c_errmsg ("Error reading GOF","gofrd-read",&one);
	        return (E_FAIL);
	        }
	    else if (*obyte == 0)
		return (E_EOF);
/*
   Determine the number of bytes to the next record.
*/
	    *nbyte = 0; 
/*
   Copy the attribute mask into the structure.
*/
	    for (i=0; i < MAXATT; i++)
		gofptr->hdrec.mask[i] = arec.mask[i];
/*
   Copy the attribute names and types into the structure.
*/
	    for (i=0; i < MAXATT; i++)
		{
	    	strcpy (gofptr->hdrec.attrb[i],arec.attrb[i]);
		strcpy (gofptr->hdrec.attyp[i],arec.attyp[i]);
	    	}
/*
   Copy the number of records in the file into the structure.
*/
	     sscanf (arec.rnum,"%d",&gofptr->hdrec.nrec);
	     return (E_SUCC);
	     }
/*
   Read the appropriate number of bytes based on the type 
   of record being read. 
*/ 
	switch (*type)
	    {
	    case HPOLIN :		/* Polygon or line header record*/
	    case PNT :			/* Point record			*/
    	    	*obyte = read (*fd, &buf, LENTYP + CHARLEN);
    		if (*obyte < 0)
	    	    {
	    	    c_errmsg ("Error reading GOF","gofrd-read",&one);
	    	    return (E_FAIL);
	    	    }
		else if (*obyte == 0)
	    	    return (E_EOF);
    	    	numbyt = read (*fd, &(buf.x), XYLEN);
		*obyte += numbyt;
	    	break;
	    case ANOT:			/* Annotation record		*/
		*obyte = read (*fd,&a1buf,LENTYP);
    		if (*obyte < 0)
	    	    {
	    	    c_errmsg ("Error reading GOF","gofrd-read",&one);
	    	    return (E_FAIL);
	    	    }
		else if (*obyte == 0)
	    	    return (E_EOF);
		sscanf (a1buf.length,"%d/%d",&clen,&dlen);
		numbyt = read (*fd,a1buf.attrb,clen);
		*obyte = numbyt + LENTYP;
		numbyt = read (*fd,&a2buf,dlen);
		*obyte += numbyt;
		break;
	    case XYPAIRS:		/* XY pairs of a line or polygon*/
		buf.xpoints = (float *)malloc(gofptr->plrec.numpts * sizeof(
		    float));
		if (buf.xpoints == NULL)
		    {
		    c_errmsg ("Error allocating dynamic memory","gofrd-read",
			&one);
		    return (E_FAIL);
		    }
		buf.ypoints = (float *)malloc(gofptr->plrec.numpts * sizeof(
		    float));
		if (buf.ypoints == NULL)
		    {
		    c_errmsg ("Error allocating dynamic memory","gofrd-read",
			&one);
		    return (E_FAIL);
		    }
		for (i=0; i < gofptr->plrec.numpts; i++)
		    {
		    *obyte = read(*fd,xypair,XYLEN);
		    if (*obyte < 0)
			break;
		    buf.xpoints[i] = xypair[0];
		    buf.ypoints[i] = xypair[1];
		    }
		break;
	    }   /* End Switch */

    	if (*obyte < 0)
	    {
	    c_errmsg ("Error reading GOF","gofrd-read",&one);
	    return (E_FAIL);
	    }
	else if (*obyte == 0) return (E_EOF);

	if ((*type == HPOLIN) || (*type == ANOT) || (*type == PNT))
	    {
/*
   Determine the total record length.
*/
	    if (*type != ANOT)
	        sscanf (buf.length,"%d/%d",&clen,&dlen);
	    *nbyte = clen + dlen + LENTYP;
/*
   Determine the number of bytes to the next record.
*/
	    *nbyte = *nbyte - *obyte;
/*
   Copy the label into the struture.
*/ 
	    if (*type == PNT)
	        strcpy (gofptr->ptrec.label,buf.label);
	    else if (*type == HPOLIN)
	        strcpy (gofptr->plrec.label,buf.label);
	    else
	        strcpy (gofptr->antrec.label,a1buf.label);
/*
   Copy the point or annotation XY pair into the structure.
*/
	    if (*type == PNT) 
	    	{
	    	gofptr->ptrec.x = buf.x;
	    	gofptr->ptrec.y = buf.y;
	    	}
	    else if (*type == ANOT)
	    	{
	    	gofptr->antrec.x = a2buf.x;
	    	gofptr->antrec.y = a2buf.y;
	    	}
/*
   Convert the number of points to integer.
*/
	    else 
	    	gofptr->plrec.numpts = (int)buf.x;
/*
   Assign the address of the XY point values into the structure. 
*/ 
	    if (*type == PNT)
	        for (i=0; i < MAXATT; i++)
	            strcpy (gofptr->ptrec.attrb[i],buf.attrb[i]);
	    else if (*type == ANOT)
	        for (i=0; i < MAXATT; i++)
	            strcpy (gofptr->antrec.attrb[i],a1buf.attrb[i]);
	    else
	        for (i=0; i < MAXATT; i++)
	            strcpy (gofptr->plrec.attrb[i],buf.attrb[i]);
	    }
/*
   Copy the XY point values into the structure.
*/
	if (*type == XYPAIRS)
	    {
	    *nbyte = 0;
	    gofptr->plrec.x = buf.xpoints;
	    gofptr->plrec.y = buf.ypoints;
	    }
/*
   Copy the annotation string into the structure.
*/ 
	if (*type == ANOT)
	    {
	    *nbyte = 0;
	    gofptr->antrec.size = (int)a2buf.size;
	    gofptr->antrec.angle = a2buf.angle;
	    strcpy (gofptr->antrec.antext,a1buf.anotstr);
	    }

    return (E_SUCC);
}
