/******************************************************************************
NAME:		c_gofwr

PURPOSE:  Write the specified type of record to the GOF from the 
	  information received in the defined structure.

PROGRAM HISTORY:
Version         Date    Contrib         Change
-------         ----    -------         ------
 1.0            3/86    L. Huewe        Intial code
 2.0            5/92    L. Huewe        Modified to be a more generic
                                        WORLD support routine.

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "gof.h"

int c_gofwr
(
    int *fd, 	 	/* File descriptor of the GOF to read (I).	*/
    int *nbyte, 	/* Number of bytes to next record (O).		*/
    int *type,	 	/* Type of record being read (I).		*/
    union GOF *gofptr, 	/* Pointer to the appropriate structure (I).	*/
    int *addrec 	/* TRUE - Adding records to the GOF	
		    	   FALSE - Updating existing records (I).	*/
)

{
static struct	RWHDR 	arec, 	/* Attribute definition record structure*/
			oldrec; /* Old header record			*/

static struct 	PPLGOF 	buf;  	/* Point, polygon or line GOF record 
				   structure				*/
static struct 	ANOT1 	a1buf; 	/* Character portion of annotation record 
				   structure				*/
static struct 	ANOT2 	a2buf; 	/* Float portion of annotation record 
				   structure				*/

		float	xypair[2];
				/* XY pair for polygon or line.		*/

		int	anotchar,
				/* Length of the attributes and annotation
				   string			     	*/
			current,/* Current number of records in the GOF	*/
			dlen,	/* Data length 				*/
			i,	/* Loop index				*/
			obyte,  /* Number of bytes written		*/
			pos;	/* Current file position		*/
		long one = 1;/* Indicates a nonfatal error	        */

		char 	temp[LENL];
				/* Temporary string containing character 
				   and data lengths   			*/

/*
   Write a GOF header record.
*/
if (*type == HDR)
    {
    obyte = read (*fd, &oldrec, HDRLEN);
    lseek (*fd,0,0);
    if (obyte < 0)
        {
        c_errmsg ("Error reading GOF","gofwr-read",&one);
        return (E_FAIL);
        }
    else if (obyte == 0) strcpy (oldrec.rnum,"0"); 

/* 
   Set up the length, type and number of records.
*/
    *nbyte = HDRLEN;
/* 
   Length is split into 2 parts.  The first is the length of the 
   character portion of the record, the second is the length of the
   data portion of the record.
*/
    sprintf (temp,"%d/%d",*nbyte - LENTYP - MAXATT,MAXATT);
    sprintf (arec.length,"%-*s",LENL,temp);
    sprintf (arec.type,"%-*s",TYPL,"B");
    strcpy (arec.key,HDRKEY);
    sprintf (arec.rnum,"%d",gofptr->hdrec.nrec);

/* 
   Copy the attributes to the output buffer.
*/ 
    for (i=0; i < MAXATT; i++)
	{
	arec.mask[i] = gofptr->hdrec.mask[i];
    	strcpy (arec.attrb[i],gofptr->hdrec.attrb[i]);
    	strcpy (arec.attyp[i],gofptr->hdrec.attyp[i]);
	}

/* 
   Write the number of specified bytes.
*/ 
    obyte = write (*fd, &arec, *nbyte);
    if ((obyte <= 0) || (obyte != *nbyte))
	{
    	c_errmsg ("Error writing to GOF","gofwr-write",&one);
	return (E_FAIL);
	}

/* 
   Set the number of bytes to the next record.
*/
    *nbyte = 0;
    return (E_SUCC);
    }

/*
   Determine the number of bytes to write and the number of bytes to the next 
   record.
*/
switch (*type)
    {
    case HPOLIN:	/* Polygon or line subheader		*/
	*nbyte = gofptr->plrec.numpts * XYLEN;
/* 
   Length is split into 2 parts.  The first is the length of the 
   character portion of the record, the second is the length of the
   data portion of the record.
*/
	dlen = XYLEN;
        sprintf (temp,"%d/%d",CHARLEN,gofptr->plrec.numpts * XYLEN + XYLEN);
        sprintf (buf.length,"%-*s",LENL,temp);
/*
   Set up the data type of the record.
*/
        sprintf (buf.type,"%-*s",TYPL,"R4");
/*
   Copy the label to the output buffer.
*/ 
        strcpy (buf.label,gofptr->plrec.label);
	break;
    case FPOLIN:	/* Full polygon or line record		*/
	*nbyte = 0;
/* 
   Length is split into 2 parts.  The first is the length of the 
   character portion of the record, the second is the length of the
   data portion of the record.
*/
	dlen = gofptr->plrec.numpts * XYLEN + XYLEN;
        sprintf (temp,"%d/%d",CHARLEN,dlen);
        sprintf (buf.length,"%-*s",LENL,temp);
/*
   Set up the data type of the record.
*/
        sprintf (buf.type,"%-*s",TYPL,"R4");
/*
   Copy the label to the output buffer.
*/ 
        strcpy (buf.label,gofptr->plrec.label);
	break;
    case ANOT:		/* Annotation record			*/
	*nbyte = 0;
/* 
   Length is split into 2 parts.  The first is the length of the 
   character portion of the record, the second is the length of the
   data portion of the record.
*/
	anotchar = CHARLEN + strlen(gofptr->antrec.antext) + 1;
        sprintf (temp,"%d/%d",anotchar,2 * XYLEN);
        sprintf (a1buf.length,"%-*s",LENL,temp);
/*
   Set up the data type of the record.
*/
        sprintf (a1buf.type,"%-*s",TYPL,"R4");
/*
   Copy the label to the output buffer.
*/ 
        strcpy (a1buf.label,gofptr->antrec.label);
	break;
    case PNT:		/* Point record				*/
	*nbyte = 0;
/* 
   Length is split into 2 parts.  The first is the length of the 
   character portion of the record, the second is the length of the
   data portion of the record.
*/
	dlen = XYLEN;
        sprintf (temp,"%d/%d",CHARLEN,dlen);
        sprintf (buf.length,"%-*s",LENL,temp);
/*
   Set up the data type of the record. 
*/
        sprintf (buf.type,"%-*s",TYPL,"R4");
/*
   Copy the label to the output buffer.
*/ 
        strcpy (buf.label,gofptr->ptrec.label);
	break;
    case XYPAIRS:	/* XY pairs of a polygon or line record	*/
	*nbyte = 0;
	dlen = gofptr->plrec.numpts * XYLEN;
	break;
    }
/*
   Copy the point XY value to the output buffer.
*/
if (*type == PNT) 
    {
    buf.x = gofptr->ptrec.x;
    buf.y = gofptr->ptrec.y;
/*
   Copy the attributes to the output buffer.
*/ 
    for (i=0; i < MAXATT; i++)
        strcpy (buf.attrb[i],gofptr->ptrec.attrb[i]);
    }
else if (*type == ANOT)
    {
    a2buf.x = gofptr->antrec.x;
    a2buf.y = gofptr->antrec.y;
    for (i=0; i < MAXATT; i++)
        strcpy (a1buf.attrb[i],gofptr->antrec.attrb[i]);
/*
   Copy the size, angle and annotation string into the output buffer.
*/
    a2buf.size = (float)gofptr->antrec.size;
    a2buf.angle = gofptr->antrec.angle;
    strcpy (a1buf.anotstr,gofptr->antrec.antext);
    }
/*
   Convert the number of points to float.
*/
else if ((*type == HPOLIN) || (*type == FPOLIN)) 
    {
    buf.x = (float)gofptr->plrec.numpts;
    for (i=0; i < MAXATT; i++)
        strcpy (buf.attrb[i],gofptr->plrec.attrb[i]);
    }
/*
   Assign the address of the XY point values into the structure. 
*/
if ((*type == XYPAIRS) || (*type == FPOLIN)) 
    {
    buf.xpoints = gofptr->plrec.x;
    buf.ypoints = gofptr->plrec.y;
    }
/*
   Write the number of specified bytes.
*/ 
if ((*type == FPOLIN) || (*type == HPOLIN) || (*type == PNT))
    {
    obyte = write (*fd, &buf, LENTYP + CHARLEN);
    if ((obyte <= 0) || (obyte != LENTYP + CHARLEN))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    obyte = write (*fd, &(buf.x), XYLEN);
    if ((obyte <= 0) || (obyte != XYLEN))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    if (dlen > XYLEN)
	{
	for (i=0; i < buf.x; i++)
	    {
	    xypair[0] = buf.xpoints[i];
	    xypair[1] = buf.ypoints[i];
	    obyte = write (*fd, xypair, XYLEN);
	    if ((obyte <= 0) || (obyte != XYLEN))
		{
		c_errmsg ("Error writing to GOF","gofwr-write",&one);
		return (E_FAIL);
		}
	    }
	}
    }
else if (*type == ANOT)
    {
    obyte = write (*fd, &a1buf, anotchar+LENTYP);
    if ((obyte <= 0) || (obyte != anotchar+LENTYP))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    obyte = write (*fd, &a2buf, 2 * XYLEN);
    if ((obyte <= 0) || (obyte != 2 * XYLEN))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    }
else
    {
    obyte = write (*fd, buf.xpoints, dlen);
    if ((obyte <= 0) || (obyte != dlen))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    }

/*
   Increment the number of records in the GOF.
*/
if (*addrec)
    {
    pos = lseek (*fd,0,1);
    lseek (*fd,0,0);
    obyte = read (*fd, &oldrec, HDRLEN);
    if (obyte < 0)
        {
        c_errmsg ("Error reading GOF","gofwr-read",&one);
        return (E_FAIL);
        }
    sscanf (oldrec.rnum,"%d",&current);
    current++;
    sprintf (oldrec.rnum,"%d",current);
    lseek (*fd,0,0);
    obyte = write (*fd, &oldrec, HDRLEN);
    if ((obyte <= 0) || (obyte != HDRLEN))
        {
        c_errmsg ("Error writing to GOF","gofwr-write",&one);
        return (E_FAIL);
        }
    lseek (*fd,pos,0);
    }

return (E_SUCC);
}
