/*********************   Label Services Subsystem   *********************
FUNCTION:       lsrew

PURPOSE:        Rewind an associated file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           07/86    K. Gacke            initial development
           12.87    B. Ailts            place bridge routines in a seperate file
                                        use raw 'C' types
                                        change include file directory specs
  5.0      04/88    D. Hollaren         change include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed error code by address
                                        return E_SUCC/E_FAIL
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.3     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Seek to the beginning of the file

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <sys/types.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"

int c_labrew
(
    int fd              /* I: file descriptor */
)
{
    if (lseek(fd, 0, SEEK_SET) != 0)           /* rewind file  */
    {
        c_lasmsg(FALSE,"lsrew-rewind","Error rewinding label services file");
        return(E_FAIL);
    }

    return(E_SUCC);
}

#ifndef NO_OBSOLETE_INTERFACE

int c_lsrew
(
    const long *fd      /* I: file descriptor */
)
{
    if (lseek(*fd, 0, SEEK_SET) != 0)           /* rewind file  */
    {
        c_lasmsg(FALSE,"lsrew-rewind","Error rewinding label services file");
        return(E_FAIL);
    }

    return(E_SUCC);
}

#endif /* Obsolete interfaces */
