/*******************************************************************************
FUNCTION:	get_matrix

PURPOSE:	Copies the matrix from the LT record structure to the
		application programs buffer.  The matrix is converted to
		the specified data type when copied to the output buffer.

PROGRAM HISTORY:
  Version  Date   Author       	Request
  -------  ----	  ------    	-------
    1      12/86  K. Gacke      Original development
    2	    3/87  K. Gacke	Modified open_tab to allow directories to be 
				specified for the subfiles.  Replaced "strcat" 
				calls with "strcpy" to make the code more 
				efficient.  Replaced "sscanf" calls with "atol"
				and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
				for each attribute defined.
    4	    6/87  K. Gacke	Allow the LT to be opened for append.  Modified
				"close_tab", "get_vector", "open_tab", and 
				"put_vector".
    5	    7/87  K. Gacke	changed the calls malloc and free to xalloc and
				xree for the sel.  These routines are the C run
				time routines renamed and put into the newlas.a
				library to avoid linking the malloc and free in
				the fortran libraries.
    6	    8/87  K. Gacke	added the DATE and TIME data types.  Modified
				the routines get_vector, get_record, put_record,
				and put_vector.  Created the routines
				get_lt_date, get_lt_time, put_lt_date, and
				put_lt_time.
    7       9/87  K. Gacke	removed mod for "version 5" after installing
				new OS (UTX 2.0)
    8      12/87  B. Ailts	Change include directory structure
    9      05/88  B. Ailts	Standardized err messages
				Split routines into separate files
				Added worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	none.

PROJECT:	LAS

ALGORITHM:
Determine the input and output data types
Copy or convert matrix from the VECTOR structure into the output buffer

*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include "ltable.h"

void get_matrix
(
    struct VECTOR *vec,		/* Structure defining the record to copy      */
    long odtype,		/* Data type of the output matrix	      */
    unsigned char *matrix	/* Output matrix (any valid matrix data type) */
)
{
struct TAB_DEF dummy;
long in_dtype;
long nentity;
long out_dtype;

switch (vec->dtype)
   {
   case I1: in_dtype = 1;	break;
   case I2: in_dtype = 2;	break;
   case I4: in_dtype = 3;	break;
   case R4: in_dtype = 4;	break;
   case R8: in_dtype = 5;	break;
   default: 
       tab_error(&dummy,TAB_LT_DTYPE,FATAL,"Invalid data type for a matrix",
                 "getmatrix");
       return;
   }

switch (odtype)
   {
   case I1: out_dtype = 1;	break;
   case I2: out_dtype = 2;	break;
   case I4: out_dtype = 3;	break;
   case R4: out_dtype = 4;	break;
   case R8: out_dtype = 5;	break;
   default: 
       tab_error(&dummy,TAB_LT_DTYPE,FATAL,"Invalid data type for a matrix",
                 "getmatrix");
       return;
   }

nentity = vec->size.nrow * vec->size.ncol;

if (in_dtype == out_dtype)
   c_pxcopy(vec->ptr.i1,matrix,&out_dtype,&nentity);
else
   c_pxconv(&in_dtype,&out_dtype,vec->ptr.i1,matrix,&nentity);

}
