/*******************************************************************************
FUNCTION:	make_lt_name

PURPOSE:	Returns an associated LT file name.  

PROGRAM HISTORY:
  Version  Date   Author       	Request
  -------  ----	  ------    	-------
    1      12/86  K. Gacke      Original development
    2	    3/87  K. Gacke	Modified open_tab to allow directories to be 
				specified for the subfiles.  Replaced "strcat" 
				calls with "strcpy" to make the code more 
				efficient.  Replaced "sscanf" calls with "atol"
				and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
				for each attribute defined.
    4	    6/87  K. Gacke	Allow the LT to be opened for append.  Modified
				"close_tab", "get_vector", "open_tab", and 
				"put_vector".
    5	    7/87  K. Gacke	changed the calls malloc and free to xalloc and
				xree for the sel.  These routines are the C run
				time routines renamed and put into the newlas.a
				library to avoid linking the malloc and free in
				the fortran libraries.
    6	    8/87  K. Gacke	added the DATE and TIME data types.  Modified
				the routines get_vector, get_record, put_record,
				and put_vector.  Created the routines
				get_lt_date, get_lt_time, put_lt_date, and
				put_lt_time.
    7       9/87  K. Gacke	removed mod for "version 5" after installing
				new OS (UTX 2.0)
    8      12/87  B. Ailts	Change include directory structure
    9      05/88  B. Ailts	Standardized err messages
				Split routines into separate files
				Added worgen.h
   10      02/90  B. Ailts      Checked for allocation errors

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	none.

PROJECT:	LAS

ALGORITHM:
Create a subfile name by concatenating the master file name with a numeric value

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "worgen.h"
#include "ltable.h"

char *make_lt_name
(
    char innam[],	/* Master Labeled Table name			      */
    long nmr		/* Numeric value to be appended to the master name    */
)
{
char *outnam;
struct TAB_DEF dummy;	/* dummy address for struct received by tab_error     */

if ((outnam = malloc(strlen(innam) + 2)) == NULL)
     {
     tab_error(&dummy,TAB_ALLOC,FATAL," ","make_lt_name");
     return(NULL);
     }

sprintf(outnam,"%s%ld",innam,nmr);
return(outnam);
}
