/******************************************************************************
NAME			pxbtan

PURPOSE		Perform a bitwise and of each element of two input arrays
		generating a third output array which is the "anded" result.
		The input array can be of type byte, signed 2 byte, signed 4
		byte, or real. The output array will be of the same type as the
		input array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	7/85	Original development for NEWLAS
	B.Ailts		12/87	Place bridge routines in a seperate file
				change include directory specifications
				Use raw 'C' types
	B.AILTS		3/88	replaced newlas.h with worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				NEWLAS

ALGORITHM

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the bitwise and of the input array and
	the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void bbtan
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in & *map;
}

static void lbtan
(
    long *in,
    long *map,
    long *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in & *map;
}

static void wbtan
(
    short *in,
    short *map,
    short *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in & *map;
}


int c_pxbtan
(
   long *in,   /* I:  input array                                         */
   long *map,  /* I:  input mapping array                                 */
   long *out,  /* O:  output "anded" array                                */
   long *ns,   /* I:  number of samples                                   */
   long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)                 */
)

{
long	status = E_SUCC;	/* return status for pxadd */

switch(*dtype)
	{
   case (EBYTE): bbtan((unsigned char *)in,(unsigned char *)map,
                       (unsigned char *)out,*ns); 
                 break;
   case (EWORD): wbtan((short *)in,(short *)map,(short *)out,*ns); 
                 break;
   case (ELONG): 
   case (EREAL): lbtan((long *)in,(long *)map,(long *)out,*ns); 
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxbtan */

