/*************************************************************
NAME			pxbtor

PURPOSE		Perform a bitwise "or" of each element of two input arrays
		generating a third output array which is the "or"'d result.
		The input array can be of type byte, signed 2 byte, signed 4
		byte, or real. The output array will be of the same type as the
		input array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
  	R.MCKINNEY	9/87	Original development for NEWLAS
	B.Ailts		4/88    Replaced newlas.h with worgen.h
				Moved the fortran bridge to a seperate file
				Used raw C types
				added include pixman.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system 

PROJECT				LAS

ALGORITHM 

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the bitwise "or" of the input array and
	the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bbtor
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in | *map;
}




void wbtor
(
    short *in,
    short *map,
    short *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in | *map;
}




void lbtor
(
    long *in,
    long *map,
    long *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	*out = *in | *map;
}




int c_pxbtor
(
   long *in,   /* I:  input array                                        */
   long *map,  /* I:  input mapping array                                */
   long *out,  /* O:  output "ored" array                                */
   long *ns,   /* I:  number of samples                                  */
   long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)                */
)

{
long	status = E_SUCC;	/* return status for pxor */

switch(*dtype)
	{
   	case (EBYTE): bbtor((unsigned char *)in,(unsigned char *)map,
                            (unsigned char *)out,*ns); 
                      break;
   	case (EWORD): wbtor((short *)in,(short *)map,(short *)out,*ns); 
                      break;
   	case (ELONG): 
   	case (EREAL): lbtor((long *)in,(long *)map,(long *)out,*ns); 
                      break;
        default: status = TYPERR; break;
	} /* switch */

return(status);
} /* pxbtor */
