/*************************************************************
NAME			c_pxcomp

PURPOSE		Perform a one's complement of each element of the input 
		array generating a second output array which is the 
		result.  The input array can be of type byte, signed 2 byte, 
		signed 4 byte, or real. The output array will be of the same 
		type as the input array.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used.
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the first input
				array.
*out		long		The address of beginning of the output array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
bbcomp		performs pixel ones complement on unsigned byte data
wbcomp		performs pixel ones complement on signed integer data
lbcomp		performs pixel ones complement on signed long data or real
		data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxbxor must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the one's complement of the input 
	array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"




void bbcomp
(
    unsigned char *in,
    unsigned char *out,
    long ns
)
   {
   for(; ns; ns--,in++,out++)
      *out = ~*in;
   }




void wbcomp
(
    short *in,
    short *out,
    long ns
)
   {
   for(; ns; ns--,in++,out++)
      *out = ~*in;
   }




void lbcomp
(
    long *in,
    long *out,
    long ns
)
   {
   for(; ns; ns--,in++,out++)
      *out = ~*in;
   }




int c_pxcomp
(
   long *in,   /* I:  input array                                        */
   long *out,  /* O:  output array                                       */
   long *ns,   /* I:  number of samples                                  */
   long *dtype /* I:  data type (EBYTE,EWORD,ELONG,EREAL)                */
)

   {
   short	status = E_SUCC;	/* return status for pxadd */

   switch(*dtype)
      {
      case (EBYTE): bbcomp((unsigned char *)in,(unsigned char *)out,*ns); 
                    break;
      case (EWORD): wbcomp((short *)in,(short *)out,*ns); 
                    break;
      case (ELONG): 
      case (EREAL): lbcomp(in,out,*ns); break;
      default: status = TYPERR; break;
      } /* switch */
   return(status);
   } /* pxcomp */
