/*****************************************************************************
NAME			pxconv

PURPOSE		Convert an input array from one data type to another.  The 
		output array will be of the same type as 'totyp'.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	 Original development for NEWLAS
	J.REED		11/12/85 Optimized byte portion of routine.
	D.GORDON	11/20/85 Optimized all routines.
	B.Ailts		 12/87	 Changed include directory specifications
				 Use raw 'C' types
				 Place bridge routines in a seperate file
	B.Ailts		 04/88   replaced newlas.a with las.a
	B.Ailts		 12/88	 When converting to a smaller data type, the
				 output value is set to the max or min of the
				 output data type if the input value is above
				 or below the max or min of the output data
				 type

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				LAS

ALGORITHM

receive:
	-the input array and its data type
	-the output array and its data type
	-the number of samples of the input array contains.
depending on the type of input array values
	call a function for that particular type of conversion

for each pixel of the input arrays
	each the pixels from each input array are type cast into the output 
	array
return

ALGORITHM REFERENCES		none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"




void byte_word
(
    unsigned char *frombuf,
    short *tobuf,
    long size
)
{
for(; size--; *tobuf++ = (short)*frombuf++)
	;
}




void byte_long
(
    unsigned char *frombuf,
    long *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (long)*frombuf++)
		;
}




void byte_real
(
    unsigned char *frombuf,
    float *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (float)*frombuf++)
		;
}




void byte_double
(
    unsigned char *frombuf,
    double *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (double)*frombuf++)
		;
}




void word_byte
(
    short *frombuf,
    unsigned char *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXUTINY)
	      {
	      *tobuf = MAXUTINY;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINUTINY)
	      {
	      *tobuf = MINUTINY;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (unsigned char)*frombuf;
}




void word_long
(
    short *frombuf,
    long *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (long)*frombuf++)
		;
}




void word_real
(
    short *frombuf,
    float *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (float)*frombuf++)
		;
}




void word_double
(
    short *frombuf,
    double *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (double)*frombuf++)
		;
}




void long_byte
(
    long *frombuf,
    unsigned char *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXUTINY)
	      {
	      *tobuf = MAXUTINY;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINUTINY)
	      {
	      *tobuf = MINUTINY;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (unsigned char)*frombuf;
}




void long_word
(
    long *frombuf,
    short *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXWORD)
	      {
	      *tobuf = MAXWORD;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINWORD)
	      {
	      *tobuf = MINWORD;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (short)*frombuf;
}




void long_real
(
    long *frombuf,
    float *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (float)*frombuf++)
		;
}




void long_double
(
    long *frombuf,
    double *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (double)*frombuf++)
		;
}




void real_byte
(
    float *frombuf,
    unsigned char *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXUTINY)
	      {
	      *tobuf = MAXUTINY;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINUTINY)
	      {
	      *tobuf = MINUTINY;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (unsigned char)*frombuf;
}




void real_word
(
    float *frombuf,
    short *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXWORD)
	      {
	      *tobuf = MAXWORD;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINWORD)
	      {
	      *tobuf = MINWORD;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (short)*frombuf;
}




void real_long
(
    float *frombuf,
    long *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXLONG)
	      {
	      *tobuf = MAXLONG;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINLONG)
	      {
	      *tobuf = MINLONG;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (long)*frombuf;
}




void real_double
(
    float *frombuf,
    double *tobuf,
    long size
)
{
	for(; size--; *tobuf++ = (double)*frombuf++)
		;
}




void double_byte
(
    double *frombuf,
    unsigned char *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXUTINY)
	      {
	      *tobuf = MAXUTINY;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINUTINY)
	      {
	      *tobuf = MINUTINY;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (unsigned char)*frombuf;
}




void double_word
(
    double *frombuf,
    short *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXWORD)
	      {
	      *tobuf = MAXWORD;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINWORD)
	      {
	      *tobuf = MINWORD;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (short)*frombuf;
}




void double_long
(
    double *frombuf,
    long *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXLONG)
	      {
	      *tobuf = MAXLONG;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINLONG)
	      {
	      *tobuf = MINLONG;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (long)*frombuf;
}




void double_real
(
    double *frombuf,
    float *tobuf,
    long size,
    long *flag
)
{
	for(; size--; tobuf++, frombuf++)
	   if (*frombuf > MAXREAL)
	      {
	      *tobuf = MAXREAL;
	      *flag |= TOOBIG;
	      }
	   else if (*frombuf < MINREAL)
	      {
	      *tobuf = MINREAL;
	      *flag |= TOOSMALL;
	      }
	   else
	      *tobuf = (float)*frombuf;
}




int c_pxconv
(
    long *ftyp,             /* I: from type                              */
    long *totyp,            /* I: to type                                */
    unsigned char *frombuf, /* I: from buffer                            */
    unsigned char *tobuf,   /* O: to buffer                              */
    long *size              /* I: number of samples in the input buffer  */
)

{ /* pxconv */
	long status = 0;

	switch (*ftyp)
	{
	case EBYTE:
		switch (*totyp)
		{
		case EBYTE:
			c_pxcopy(frombuf, tobuf, totyp, size);
			break;
		case EWORD:
			byte_word(frombuf,(short *)tobuf,*size);
			break;
		case ELONG:
			byte_long(frombuf,(long *)tobuf,*size);
			break;
		case EREAL:
			byte_real(frombuf,(float *)tobuf,*size);
			break;
		case EDOUBLE:
			byte_double(frombuf,(double *)tobuf,*size);
			break;
		}
		break;
	case EWORD:
		switch (*totyp)
		{
		case EBYTE:
			word_byte((short *)frombuf,tobuf,*size,&status);
			break;
		case EWORD:
			c_pxcopy(frombuf,tobuf, totyp, size);
			break;
		case ELONG:
			word_long((short *)frombuf,(long *)tobuf,*size);
			break;
		case EREAL:
			word_real((short *)frombuf,(float *)tobuf,*size);
			break;
		case EDOUBLE:
			word_double((short *)frombuf,(double *)tobuf,*size);
			break;
		}
		break;
	case ELONG:
		switch (*totyp)
		{
		case EBYTE:
			long_byte((long *)frombuf,tobuf,*size,&status);
			break;
		case EWORD:
			long_word((long *)frombuf,(short *)tobuf,*size,
                                  &status);
			break;
		case ELONG:
			c_pxcopy(frombuf, tobuf, totyp, size);
			break;
		case EREAL:
			long_real((long *)frombuf,(float *)tobuf,*size);
			break;
		case EDOUBLE:
			long_double((long *)frombuf,(double *)tobuf,*size);
			break;
		}
		break;
	case EREAL:
		switch (*totyp)
		{
		case EBYTE:
			real_byte((float *)frombuf,tobuf,*size,&status);
			break;
		case EWORD:
			real_word((float *)frombuf,(short *)tobuf,*size,
                                  &status);
			break;
		case ELONG:
			real_long((float *)frombuf,(long *)tobuf,*size,
                                  &status);
			break;
		case EREAL:
			c_pxcopy(frombuf, tobuf, totyp, size);
			break;
		case EDOUBLE:
			real_double((float *)frombuf,(double *)tobuf,*size);
			break;
		}
		break;
	case EDOUBLE:
		switch (*totyp)
		{
		case EBYTE:
			double_byte((double *)frombuf,tobuf,*size,&status);
			break;
		case EWORD:
			double_word((double *)frombuf,(short *)tobuf,*size,
                                    &status);
			break;
		case ELONG:
			double_long((double *)frombuf,(long *)tobuf,*size,
                                    &status);
			break;
		case EREAL:
			double_real((double *)frombuf,(float *)tobuf,*size,
                                    &status);
			break;
		case EDOUBLE:
			c_pxcopy(frombuf, tobuf, totyp, size);
			break;
		}
		break;
	}
if (!status)
   status = E_SUCC;
return (status);
}
