/*****************************************************************************
NAME			pxcopy

PURPOSE		Copy an input array to an output array.  The 
		output array will be of the same type as the input array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	 Original development for NEWLAS
	J.REED		11/12/85 Optimized byte portion of routine.
	B.Ailts		 12/87	 Changed include directory specifications
				 Use raw 'C' types
				 Place bridge routines in a seperate file
	B.Ailts		 04/88   replaced newlas.a with las.a
	B.Ailts		 01/88	 Added the VAX memory copy call to help speed up
				 pxcopy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				NEWLAS

ALGORITHM 

receive:
	-the input array and its data type
	-the output array 
	-the number of samples of the input array contains.

Multiply the number of samples times the number of bytes that the input array's
data type takes up

for each byte of the input arrays
	copy each byte from each input array the output array
return

ALGORITHM REFERENCES		none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"

void c_pxcopy
(
    unsigned char *frombuf, /* I: source buffer of the data to be copied */
    unsigned char *tobuf,   /* O: destination buffer of the data         */
    long *dtype,            /* I: data type of the buffers               */
    long *ns                /* I: number of elements or pixels           */
)

{
static long datasize[] = {0,1,2,4,4,8};
register int lcnt;
register unsigned char *fptr, *tptr;

fptr = frombuf;
tptr = tobuf;

lcnt = *ns * datasize[*dtype];
#ifndef vms
for(; lcnt--; *tptr++ = *fptr++)
	;
#else
memcpy(tptr,fptr,lcnt);
#endif
}
