/*************************************************************
NAME			c_pxe10

PURPOSE		Performs a scaled exponentiation of each element of the 
		input array and returns it an output array.
		The input array can be of type byte or signed 2 byte or 4
		byte real.  The output array will be of type double

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used 
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the input array.
*out		double		The address of beginning of the output array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blexp		performs exponentiation on unsigned byte data
wlexp		performs exponentiation on signed integer data
flexp		performs exponentiation on signed real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxe10 must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
  Assign the output array pixel the exponentiation of the input array
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include <math.h>
#include "worgen.h"
#include "pixman.h"




static void bexp
(
    unsigned char *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = pow(10.0, (*in / 1000.0));
	}



static void wexp
(
    short *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = pow(10.0, (*in / 1000.0));
	}



static void fexpf
(
    float *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = pow(10., *in);
	}




int c_pxe10
(
   long *in,   /* I: Input array                                          */
   long *out,  /* O: output array                                         */
   long *ns,   /* I: Number of elements in the input array                */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for c_pxe10 */

switch(*dtype)
	{
   case (EBYTE): bexp((unsigned char *)in,(double *)out,*ns); 
                 break;
   case (EWORD): wexp((short *)in,(double *)out,*ns);  
                 break;
   case (EREAL): fexpf((float *)in,(double *)out,*ns);   
                 break;
   case (ELONG): 
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxe10 */
