/*************************************************************
NAME			c_pxe32k

PURPOSE		Performs a scaled exponentiation of each element of the 
		input array and returns it an output array.
		The input array can be of type signed 2 byte integer.
		The output array will be of type DOUBLE

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used 
*dtype		long		Pointer to the type of data in the input arrays
*in		short		Pointer to the beginning of the input array.
*out		DOUBLE		The address of beginning of the output array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
bex		performs exponentiation on unsigned byte data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EWORD)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxe32k must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
	call a function that performs the following:

for each pixel of the input arrays
  Assign the output array pixel the exponentiation of the input array
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include <math.h>
#include "worgen.h"
#include "pixman.h"




static void wex
(
    unsigned char *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = pow(32767, (*in / 32767));
	}




int c_pxe32k
(
   long *in,   /* I: Input array                                          */
   long *out,  /* O: output array                                         */
   long *ns,   /* I: Number of elements in the input array                */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for c_pxexp */

switch(*dtype)
	{
   case (EWORD): wex((unsigned char *)in,(double *)out,*ns); 
                 break;
   case (EBYTE):
   case (EREAL):
   case (ELONG):
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxe32k */
