/*************************************************************
NAME			c_pxl255

PURPOSE		Performs a scaled exponentiation of each element of the 
		input array and returns it an output array.
		The input array can be of type byte or signed 2 byte or 4
		byte real.  The output array will be of type double

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used 
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the input array.
*out		double		The address of beginning of the output array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blex		performs exponentiation on unsigned byte data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxl255 must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
	call a function that performs the following:

for each pixel of the input arrays
  Assign the output array pixel the exponentiation of the input array
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include <math.h>
#include "worgen.h"
#include "pixman.h"



static void blex
(
    unsigned char *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = pow(255, (*in / 255));
	}




int c_pxl255
(
   long *in,   /* I: input array                                          */   
   long *out,  /* O: output array                                         */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for pxexp */

switch(*dtype)
	{
   case (EBYTE): blex((unsigned char *)in,(double *)out,*ns);
                 break;
   case (EWORD):
   case (EREAL):
   case (ELONG):
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* pxl255 */
