/*************************************************************
NAME			pxmax


PURPOSE		Determine the maximum values of either the input array or
		the mapping array.  Assign the maximum value to the output
		array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
        R. MCKINNEY     9/87 	Original development for NEWLAS
	B. Ailts        4/88    Replaced newlas.h with worgen.h
				Used raw C types -- removed stdedc.h
				Moved fortran bridge to seperate file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system 

PROJECT				LAS

ALGORITHM 

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Determine the maximum pixel values.
	Assign the output array pixel the maximum of the input array and
		the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bmax
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in >= *map) *out = *in;
	if(*in < *map) *out = *map;
	}
}



void wmax
(
    short *in,
    short *map,
    short *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in >= *map) *out = *in;
	if(*in < *map) *out = *map;
	}
}



void lmax
(
    long *in,
    long *map,
    long *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in >= *map) *out = *in; 
	if(*in < *map) *out = *map;
	}
}



void rmax
(
    float *in,
    float *map,
    float *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in >= *map) *out = *in;
	if(*in < *map) *out = *map;
	}
}




int c_pxmax
(
   long *in,   /* I: input array                                          */
   long *map,  /* I: input mapping array                                  */
   long *out,  /* O: output  array                                        */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

{
long	status = E_SUCC;	/* return status for pxmax */

switch(*dtype)
	{
   	case (EBYTE): bmax((unsigned char *)in,(unsigned char *)map,
                           (unsigned char *)out,*ns);
                      break;
   	case (EWORD): wmax((short *)in,(short *)map,(short *)out,*ns);
                      break;
   	case (ELONG): lmax(in,map,out,*ns);
                      break;
   	case (EREAL): rmax((float *)in,(float *)map,(float *)out,*ns);
                      break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxmax */
