/*************************************************************
NAME			pxmin


PURPOSE		Determine the minimum values of either the input array or
		the mapping array.  Assign the minimum value to the output
		array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
        R. MCKINNEY    9/87 	Original development for NEWLAS
	B. Ailts	4/88	Replaced newlas.h with worgen.h
				Placed fortran bridges in a seperate file
				Used raw C types -- deleted stdedc.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of LAS system 

PROJECT				LAS

ALGORITHM

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Determine the minimum pixel values.
	Assign the output array pixel the minimum of the input array and
		the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bmin
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in <= *map) *out = *in;
	if(*in > *map) *out = *map;
	}
}



void wmin
(
    short *in,
    short *map,
    short *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in <= *map) *out = *in;
	if(*in > *map) *out = *map;
	}
}



void lmin
(
    long *in,
    long *map,
    long *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in <= *map) *out = *in;
	if(*in > *map) *out = *map;
	}
}



void rmin
(
    float *in,
    float *map,
    float *out,
    long ns
)
{
for(; ns; ns--,in++,map++,out++)
	{
	if(*in <= *map) *out = *in;
	if(*in > *map) *out = *map;
	}
}




int c_pxmin
(
   long *in,   /* I: input array                                          */
   long *map,  /* I: input mapping array                                  */
   long *out,  /* O: output  array                                        */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

{
long	status = E_SUCC;	/* return status for pxmin */

switch(*dtype)
	{
   	case (EBYTE): bmin((unsigned char *)in,(unsigned char *)map,
                           (unsigned char *)out,*ns);
                      break;
   	case (EWORD): wmin((short *)in,(short *)map,(short *)out,*ns);
                      break;
   	case (ELONG): lmin(in,map,out,*ns);
                      break;
   	case (EREAL): rmin((float *)in,(float *)map,(float *)out,*ns);
                      break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxmin */
