/*******************************************************************************
NAME			        C_PXMNMX

PURPOSE		Determine the minimum and maximum values of the input array and
		the minimum and maximum parameters themselves.  The input array
		can be of type byte, signed 2 byte, signed 4 byte, or real.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	7/85	Original development for NEWLAS
	B.Ailts		12/87	Changed include directory specifications
				moved bridge routines into a separate file
				Used raw 'C' types
	K. Zanter	 2/88   changed min and max arguments to be double
	B. Ailts	03/88	replaced newlas.h with worgen.h
	B. Davis	06/90	added double capibility

PROJECT				new LAS

ALGORITHM 

receive:
	-the input array and its type
	-the number of samples of the input array.
	-the minimum and maximum pixel values.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Determine the minimum and maximum pixel values.
	if the array's min and max values excede the min and max parameters
		the parameters min and/or max are reassigned 
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bmnmx
(
    unsigned char *in,
    long ns,
    unsigned char *bmin,
    unsigned char *bmax
)
{
for(; ns; ns--,in++)
	{
	if(*in < *bmin) *bmin = *in;
	if(*in > *bmax) *bmax = *in;
	}
}



void wmnmx
(
    short *in,
    long ns,
    short *wmin,
    short *wmax
)
{
for(; ns; ns--,in++)
	{
	if(*in < *wmin) *wmin = *in;
	if(*in > *wmax) *wmax = *in;
	}
}



void lmnmx
(
    long *in,
    long ns,
    long *lmin,
    long *lmax
)
{
for(; ns; ns--,in++)
	{
	if(*in < *lmin) *lmin = *in;
	if(*in > *lmax) *lmax = *in;
	}
}



void rmnmx
(
    float *in,
    long ns,
    float *fmin,
    float *fmax
)
{
for(; ns; ns--,in++)
	{
	if(*in < *fmin) *fmin = *in;
	if(*in > *fmax) *fmax = *in;
	}
}



void dblmnmx
(
    double *in,
    long ns,
    double *dblmin,
    double *dblmax
)
{
for(; ns; ns--,in++)
	{
	if(*in < *dblmin) *dblmin = *in;
	if(*in > *dblmax) *dblmax = *in;
	}
}




int c_pxmnmx
(
   long *in,     /* I: input array                                         */
   long *ns,     /* I: number of samples                                   */
   double *dmin, /* I/O: minimum pixel value                               */
   double *dmax, /* I/O: maximum pixel value                               */
   long *dtype   /* I: data type (EBYTE,EWORD,ELONG,EREAL)                 */
)
			/*  EBYTE = unsigned 1 byte  */
			/*  EWORD = signed 2 byte  */
			/*  ELONG = signed 4 byte  */
			/*  EREAL = real 4 byte */
			/*  EDOUBLE = real 8 byte */

{
double  dblmin;		/* double min value */
double	dblmax;		/* double max value */

float   fmin;		/* float min value */
float	fmax;		/* float max value */

long	lmin;		/* long min value  */
long	lmax;		/* long max value  */
long	status = E_SUCC;	/* return status for pxmnmx */

short  wmin;		/* short min value */
short  wmax;		/* short max value */

unsigned char  bmin; 	/* byte min value  */
unsigned char  bmax;	/* byte max value  */

switch(*dtype)
	{
   case (EBYTE): bmin = *dmin;
		 bmax = *dmax;
		 bmnmx((unsigned char *)in,*ns,&bmin,&bmax); 
	         *dmin = bmin;
		 *dmax = bmax;
		 break;
   case (EWORD): wmin = *dmin;
		 wmax = *dmax;
		 wmnmx((short *)in,*ns,&wmin,&wmax); 
		 *dmin = wmin;
		 *dmax = wmax;
		 break;
   case (ELONG): lmin = (long)*dmin;
		 lmax = (long)*dmax;
	         lmnmx(in,*ns,&lmin,&lmax); 
		 *dmin = lmin;
		 *dmax = lmax;
		 break;
   case (EREAL): fmin = *dmin;
		 fmax = *dmax;
		 rmnmx((float *)in,*ns,&fmin,&fmax); 
		 *dmin = fmin;
		 *dmax = fmax;
		 break;
   case (EDOUBLE): dblmin = *dmin;
		   dblmax = *dmax;
		   dblmnmx((double *)in,*ns,&dblmin,&dblmax); 
		   *dmin = dblmin;
		   *dmax = dblmax;
		   break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxmnmx */
