/*****************************************************************************
NAME			pxsamp

PURPOSE		Subsample an array of binary numbers.  The buffer is
		updated.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	K.GACKE         11/88   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT		LAS

ALGORITHM

depending on the type of input array values
	call a function for that particular data type 
subsample the array
return

ALGORITHM REFERENCES		none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"



void samp_byte
(
    long inns,
    long inc,
    unsigned char *buf
)
{
unsigned char *ptr1, *ptr2;

ptr1 = ptr2 = buf;
for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
	{
	*ptr1 = *ptr2;
	}
}



void samp_word
(
    long inns,
    long inc,
    short *buf
)
{
short *ptr1, *ptr2;

ptr1 = ptr2 = buf;
for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
	{
	*ptr1 = *ptr2;
	}
}



void samp_long
(
    long inns,
    long inc,
    long *buf
)
{
long *ptr1, *ptr2;

ptr1 = ptr2 = buf;
for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
	{
	*ptr1 = *ptr2;
	}
}



void samp_real
(
    long inns,
    long inc,
    float *buf
)
{
float *ptr1, *ptr2;

ptr1 = ptr2 = buf;
for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
	{
	*ptr1 = *ptr2;
	}
}



void samp_double
(
    long inns,
    long inc,
    double *buf
)
{
double *ptr1, *ptr2;

ptr1 = ptr2 = buf;
for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
	{
	*ptr1 = *ptr2;
	}
}




void c_pxsamp
(
   long *inns,        /* I: number of samples in input buffer              */
   long *dtype,       /* I: data type (EBYTE,EWORD,ELONG,EREAL)            */
   long *inc,         /* I: sample increment to subsample by               */
   unsigned char *buf /* I/O: buffer to be subsampled                      */
)

{
	switch (*dtype)
	{
	case EBYTE:
		samp_byte(*inns,*inc,buf);
		break;
	case EWORD:
		samp_word(*inns,*inc,(short *)buf);
		break;
	case ELONG:
		samp_long(*inns,*inc,(long *)buf);
		break;
	case EREAL:
		samp_real(*inns,*inc,(float *)buf);
		break;
	case EDOUBLE:
		samp_double(*inns,*inc,(double *)buf);
		break;
	}
}
