#include "las.h"
#include "desc.h"
#include "util.h"

#ifdef unix
/*  Fortran bridge routine for the UNIX  */
#ifdef _IBMR2
#define hstout_ hstout
#endif
FUNCTION hstout_(vblock,hosout,hosin,numin,histext,hosout_len,
                 hosin_len,htext_len)
    struct PARBLK	*vblock;	/* tae parameter block               */
    char		hosin[][CMLEN];	/* input image(s)                    */
    char		*hosout;	/* output image                      */
    long 		*numin;         /* number of input images            */
    char		*histext;       /* history text                      */
    long 		hosin_len;      /* length of "hosin"                 */
    long 		hosout_len;     /* length of "hosout"                */
    long 		htext_len;      /* length of "histxt"                */
{
char *hin_ptr;
char *hout_ptr;
char *text_ptr;
long nonfatal = 1;
long i;
long status;			/* return status code                */

#ifdef sgi
if (*numin != 0)
   {
   hin_ptr = (char *)malloc (CMLEN * (*numin));
   if (hin_ptr == 0)
      {
      c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
      return(E_FAIL);
      }
   }
else
   {
   hin_ptr = (char *)malloc (CMLEN);
   if (hin_ptr == 0)
      {
      c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
      return(E_FAIL);
      }
   }
#else
hin_ptr = (char *)malloc (CMLEN * (*numin));
if (hin_ptr == 0)
   {
   c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
   return(E_FAIL);
   }
#endif

hout_ptr = squeeze(hosout,hosout_len);
text_ptr = squeeze(histext,htext_len);
for (i= 0; i < *numin; i++)
    strcpy(hin_ptr + (i * CMLEN),squeeze(hosin[i],hosin_len));

status = c_hstout(vblock,hout_ptr,hin_ptr,numin,text_ptr);

free(hout_ptr);
free(text_ptr);
free(hin_ptr);

return(status);
}
#endif

#ifdef vms
/*  Fortran bridge routine for the VAX  */
FUNCTION hstout(vblock,hosout,hosin,numin,histext)
    struct PARBLK	*vblock;	/* tae parameter block               */
    struct DESC		*hosin;		/* input image(s)                    */
    struct DESC		*hosout;	/* output image                      */
    long 		*numin;         /* number of input images            */
    struct DESC		*histext;       /* history text                      */
{
char *hin_ptr;
char *hinadr;
char *hout_ptr;
char *text_ptr;
long nonfatal = 1;
long i;
long status;			/* return status code                */

hin_ptr = malloc (CMLEN * (*numin));
if (hin_ptr == 0)
   {
   c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
   return(E_FAIL);
   }

hout_ptr = squeeze(hosout->addr,hosout->length);
text_ptr = squeeze(histext->addr,histext->length);
hinadr = hosin->addr;
for (i=0; i < *numin; i++)
    {
    strcpy(hin_ptr + (i * CMLEN),squeeze(hinadr,hosin->length));
    hinadr = hinadr + hosin->length;
    }

status = c_hstout(vblock,hout_ptr,hin_ptr,numin,text_ptr);

if (free(hout_ptr) == -1)
   c_errmsg("Error freeing dynamic memory","hstout-free",&nonfatal);
if (free(text_ptr) == -1)
   c_errmsg("Error freeing dynamic memory","hstout-free",&nonfatal);

if (free(hin_ptr) == -1)
   c_errmsg("Error freeing dynamic memory","hstout-free",&nonfatal);

return(status);
}
#endif
