/******************************************************************************
*
*    delfil
*
*    Purpose:	Delete the specifed host file name.  If the host file is an
*    		image (extension "img") then the DDR file and History file 
*		are also deleted.
*
*    Program history:
*	K. Gacke	8/86	initial development
*       K. Gacke        9/86    if image file, deleted the DDR file
*	K. Gacke       11/86	if running on SEL then call dkdelete; 
*				otherwise call unlink
*	B. Ailts       12/87    Change include directory specifications
*				Place bridge routines in a seperate file
*				replace DESC arguments with char. strings
*				use raw 'C' types
*	D. Hollaren    04/88    1. changed newlas.h to las.h
*				2. changed cerrmsg to c_errmsg
*       B. Ailts       09/88    Made sure that the hostname was not corrupted
*				Changed dkdel to c_dkdel
*	D. Etrheim     06/90	Added "diskio.h" include file 
*	D. VanderZee 	7/90	Standardized error handling 
*	S. Nelson	1/92	If image file, delete history file.
*
*    Project		LAS
*
*    Algorithm 
* 	delete the specified host file
*       if the host file was an image file
*	    delete the associated DDR file
*	exit
*
*    Algorithm References		none
*******************************************************************************/

#ifdef vms
#define unlink delete
#endif

#include <string.h>
#include "las.h"
#include "diskio.h"
#include "util.h"
#include "labserv.h"

void c_delfil
(
   char *host        /* I: host file name to be deleted                */
)

{
char ddr[CMLEN];
char his[CMLEN];
char msg[ERRLEN];
char *ptr;
long nonfatal = 1;
long status;
long zero = 0;

/* Delete the specified host file */
#ifdef UNIX_DISK
   status = c_dkdel(host);
   /*printf("status = %d\n",status);*/
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from dkdel","delfil-call",&nonfatal);
      sprintf(msg,"Error deleting host file %s",host);
      c_errmsg(msg,"delfil-delete",&nonfatal);
      }
#else
   status = unlink(host); 
   if (status == -1)
      {
      sprintf(msg,"Error deleting host file %s",host);
      c_errmsg(msg,"delfil-delete",&nonfatal);
      }
#endif

/* If the file is an image file delete the associated DDR host file 
   	and the associated history file if it exists */

strcpy(ddr,host);
if ((ptr = strrchr(ddr,HOST_EXT)) != 0)
   {
   if (strcmp(ptr+1,"img") == 0)		/* Host file is an image     */
      {
      *(ptr + 1) = '\0';			/* Delete the ddr */
      strcat(ddr,"ddr");
#ifdef UNIX_DISK
      if (c_dkdel(ddr) != E_SUCC)
         {
         c_errmsg("Error returned from dkdel","delfil-call",&nonfatal);
         sprintf(msg,"Error deleting DDR host file %s",ddr);
         c_errmsg(msg,"delfil-delete",&nonfatal);
         }
#else
      if (unlink(ddr) == -1)
         {
         sprintf(msg,"Error deleting DDR host file %s",ddr);
         c_errmsg(msg,"delfil-delete",&nonfatal);
         }

#endif
      strcpy(his,host);				/* Delete the history file */
      ptr = strrchr(his,HOST_EXT);
      *(ptr + 1) = '\0';
      strcat(his,"his");
      if (c_lsstat(his, &zero) == E_SUCC)
         {
#ifdef UNIX_DISK
	 if (c_dkdel(his) != E_SUCC)
	    {
	    c_errmsg("Error returned from dkdel","delfil-call",&nonfatal);
	    sprintf(msg,"Error deleting HIS host file %s",his);
	    c_errmsg(msg,"delfil-delete",&nonfatal);
	    }

#else
	 if (unlink(his) == -1)
	    {
	    sprintf(msg,"Error deleting HIS host file %s",his);
	    c_errmsg(msg,"delfil-delete",&nonfatal);
	    }
#endif
	 }
      }
   }
}
