/******************************************************************************
NAME			C_INMNMX

PURPOSE    Initializes the min and max arrays for the current data type.
           The elements of the min array are set to the maximum allowed
           value of the specified data type, and the elements of the max
           array are set to the minimum allowed value of the data type.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
           08/86    K. Gacke		initial development
           02/88    K. Zanter		change min and max arrays to double
  5.0      04/88    D. Hollaren         changed include spec to las
  5.1      07/90    D. VanderZee 	Standardized error handling 


PROJECT:	LAS

ALGORITHM:
	For each band initialize the min array to be the maximum pixel value
	for that data type, and initialize the max array to be the minimum
	pixel value for that data type.

ALGORITHM REFERENCES:   none
******************************************************************************/

#include "las.h"
#include "pixman.h"
#include "util.h"
#include "typlim.h"

int c_inmnmx
(
   long *dtype,       /* I:  data type                                 */
   long *totbnd,      /* I:  total number of bands                     */
   double *minval,    /* O:  minimum intensity values array            */
   double *maxval     /* O:  maximum intensity values array            */
)

{
long band;			/* loop index for each band                  */
long nonfatal = 1;
char msg[ERRLEN+1];

/*
   For all possible bands initialize the min max arrays for the current data 
   type.  Set the elements of the min array to the maximum allowed value 
   of the data type, and set the elements of the max array to the minimum 
   allowed value of the data type.  
*/
switch (*dtype)
   {
   case EBYTE:
      	for (band = 0;band < *totbnd; band++)
	    {
	    *(maxval + band) = MINUTINY;
	    *(minval + band) = MAXUTINY;
	    }
	break;
   case EWORD:
      	for (band = 0;band < *totbnd; band++)
	    {
	    *(maxval + band) =  MINWORD;
	    *(minval + band) =  MAXWORD;
	    }
	break;
   case ELONG:
      	for (band = 0;band < *totbnd; band++)
	    {
	    *(maxval + band) =  MINLONG;
	    *(minval + band) =  MAXLONG;
	    }
	break;
   case EREAL:
      	for (band = 0;band < *totbnd; band++)
	    {
	    *(maxval + band) =  MINREAL;
	    *(minval + band) =  MAXREAL;
	    }
	break;
   default:
	sprintf (msg,"Error: Invalid data type: %d",*dtype);
	c_errmsg (msg,"inmnmx-dtype",&nonfatal);
	return(TYPERR);
   }
return(E_SUCC);
}
