/******************************************************************************
NAME:	MKIMG

FUNCTION:  MKIMG is a routine which takes a root file name, appends an
           input string to the file name, followed by the image extension.

	   It is assumed that the calling program retrieved the file name
	   specification from the TAE parameter block prior to calling MKIMG.

	   The utility optionally checks if the image file already 
	   exists on disk.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         11/86      K. Zanter    initial development
    1.1         12/87      B. Ailts     Change include directory specifictions
					Use raw 'C' types
					Replace DESC arguments with char. str.
					Place bridge routines in seperate file
    5.0		04/88	   D. Hollaren  changed include spec to las
					changed cerrmsg to c_errmsg and
					passed status key by address
					return E_SUCC/E_FAIL
     5.1        07/90      D. VanderZee Standardized error handling 


PROJECT:	LAS

ALGORITHM:
   Check if the image extension is in the input file name.
   If it is not,
	concatenate the root file name with the input extension, and
	then append the image extension to create the new TAE name.
  Otherwise,
	put a null terminator in place of the image extension in
	the input file name and create the new TAE name as above.
  The full host file name is then retrieved, and an optional check is
  made to see if the host file exists.
  Finally, the full TAE name is retrieved.

ALGORITH REFERENCES:   GETOUT utility
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"

int c_mkimg
(
    char *in_string,
    char *ext,
    char *tname,
    char *hname,
    long *flag
)
{
long  	mode=0;
long	nonfatal = 1;
long	temp;

char    in_temp[CMLEN];
char	newtae[CMLEN];
char	*ptr;
char 	msg[ERRLEN+1];


strcpy(in_temp,in_string);

if (strchr(in_temp, TAE_DIR_BEG) || strchr(in_temp, TAE_EXT))
   {
   if ((ptr = strrchr(in_temp,TAE_EXT)) == 0)
      {
      strcat(in_temp,ext);
      sprintf(newtae,"%s%c%s",in_temp,TAE_EXT,"img");
      }
   else
      {
      if (strchr(in_temp,TAE_EXT) != ptr)
         {
         c_errmsg(
             "Error: Invalid LAS image name: Multiple extensions not allowed",
             "mkimg-name",&nonfatal);
         return(E_FAIL);
         }

      if (strcmp(ptr+1,"img") != 0)
         {
         c_errmsg("Error: Invalid LAS image name: Extension must be 'img'",
                  "mkimg-name", &nonfatal);
         return(E_FAIL); 
         }

      *ptr = '\0';
      strcat(in_temp,ext);
      sprintf(newtae,"%s%c%s",in_temp,TAE_EXT,"img");
      }
   }
else
   {
   temp = (long)strlen(in_temp);
   if (strcmp(&in_temp[temp-4], ".img") == 0)
      in_temp[temp-4] = '\0';
   strcat(in_temp,ext);
   sprintf(newtae, "%s%c%s", in_temp, HOST_EXT, "img");
   }

if (c_cmhost(newtae,hname) != E_SUCC)
   {
   c_errmsg ("Error returned from cmhost","mkimg-call",&nonfatal);
   sprintf (msg,"Error retrieving host file name for %s",newtae);
   c_errmsg(msg,"mkimg-cmhost", &nonfatal);
   return(E_FAIL);
   }

if (*flag == 1)
   {
   if (c_lsstat(hname,&mode) == E_SUCC)
      {
      sprintf(msg,"Error: %s already exists",newtae);
      c_errmsg(msg,"mkimg-exists",&nonfatal);
      return(E_FAIL);
      }
   }

strcpy(tname, hname);

return(E_SUCC);
}
