/******************************************************************************
*    Name:	setprt
*
*    Purpose:	Set up boolean flags for the destination of generated output
*		to the terminal, file, or line printer.
*
*    Program history:
*	K. Gacke	8/86	initial development
*  	B. Ailts	12/87	Change include directory specifications
*				Place bridge routines in a seperate file
*				Use raw 'C' types
*	B. Ailts	10/88	Called squeeze for prtfile to be accessable
*				from FORTRAN
*	B. Ailts	11/88   Fixed pointer problem to prtfile
*	B. Ailts	12/88   Renamed the original SETPRT to HSETPR -- this
*				version of SETPRT will now return LAS name
*				It calls HSETPR
	D. VanderZee 	07/90	Standardized error handling 

*    Algorithm: 
*	Call HSETPR to set term, file, prtfile, and lp
*	Call MKNAM to check validity of input name
*	Call CMTAE to calculate the TAE name
*
*    Algorithm References		none
*******************************************************************************/

#include <string.h>
#include "las.h"			/* las include file   		   */
#include "util.h"

int c_setprt
(
   struct PARBLK *vblock, /* I:  TAE parameter block                     */
   long *term,            /* O:  Boolean for terminal output             */
   long *file,            /* O:  Boolean for file output                 */
   long *lp,              /* O:  Boolean for line printer output         */
   char *prtfile,         /* I/O:  file name of file or lp working file  */
   char *filename         /* O:  file name of file or lp output file     */
)

{
long  mode;			/* access mode				   */
long  nonfatal = 1;		/* nonfatal error status		   */
long  status;			/* error status				   */

char *t_hname;			/* temporary pointer to prtfile argument   */
char *hname;			/* temporary pointer to prtfile argument   */

 
t_hname = (char *)malloc(CMLEN);
if (t_hname == NULL)
   {
   c_errmsg("Error allocating dynamic memory","setprt-alloc",&nonfatal);
   return(E_FAIL);
   }

status = c_hsetpr(vblock,term,file,lp,prtfile,t_hname);
if (status != E_SUCC)
   {
   c_errmsg ("Error returned from hsetpr","setprt-call",&nonfatal);
   return(E_FAIL);
   }

if (*file)
   {
   hname = (char *)malloc(CMLEN);
   if (hname == NULL)
      {
      c_errmsg("Error allocating dynamic memory","setprt-alloc",&nonfatal);
      return(E_FAIL);
      }

   mode = IWRITE;
   status = c_mknam(t_hname,hname,"prt",&mode);
   if (status != E_SUCC)
      {
      c_errmsg ("Error returned from mknam","setprt-call",&nonfatal);
      return(E_FAIL);
      }

   strcpy(filename, hname);

   free(hname);
   }

free(t_hname);

return(E_SUCC);
}  /*  setprt  */
