/****************************************************************************

NAME: c_unblock_cleanup

PURPOSE: Returns the SIGTERM signal to the state specified by oldmask.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Requires a system that provides POSIX signal handling.

PROJECT: LAS

NOTES:
  The oldmask value returned from c_block_cleanup is the oldmask parameter
  needed for this routine.

****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "las.h"
#include "worgen.h"

int c_unblock_cleanup
(
   const sigset_t *oldmask   /* I: status of signal mask before SIGTERM was
                                   masked to be blocked */
)

{
   /* Reset the SIGTERM signal mask
      -----------------------------*/
   if (sigprocmask (SIG_SETMASK, oldmask, NULL) < 0)
   {
      c_lasmsg (FALSE, "unblock-cleanup",
         "Warning: Cannot unblock the SIGTERM signal");
      return (E_FAIL);
   }
   else
      return (E_SUCC);
}
