/******************************************************************************
NAME:    c_wndconv

PURPOSE: Convert window specifications to line sample using an existing DDR
         file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
07/93     T. Mittan	  Original development
02/94     T. Mittan	  Modified to adjust negative line and sample numbers 
                          to 1.	
12/94     T. Mittan	  Modified to Fatal error if entire window is outside 
                          of the image.
04/95     S. Nelson	  Changed the order in which the window specifications 
                          are sent to the function c_rtcoef.  This was done 
                          because errors were found when running COPY with 
                          certain images.
May 96    David Zokaites  Adjusted checks for window being outside the image,
                          changed list of include files, called c_lasmsg()
			  instead of c_errmsg()
02/00     L. Johnson      Corrected line/sample window computations.
                          Replaced fixed number values with defined values
                          for projection types.
12/00     G. Schmidt      Changed c_wndconv to call c_window_proj_to_ls
                          which does all the work c_wndconv used to do.

NOTE: c_wndconv calls c_window_proj_to_ls after reading the existing DDR file.
  If the user is looking for a routine to convert the window from an
  existing DDR structure (rather than DDR file), then c_window_proj_to_ls is
  the routine to use.
******************************************************************************/

#include "las.h"
#include "util.h"

int c_wndconv
(
    char hname[],     /* I: host name of image specified */
    double window[],  /* I/O: image window
                              on input: uly, ulx, lry, lrx
                              on output: sl, ss, nl, ns */
    long units[]      /* I: winow units */ 
)

{
    struct DDR ddr;      /* DDR structure */

    /* Get the DDR
       -----------*/
    if (c_getddr (hname, &ddr) != E_SUCC)
    {
        c_lasmsg (FALSE, "wndconv-ddr", "Error retrieving DDR for %s", hname); 
        return (E_FAIL);
    }

    /* Do the window conversion
       ------------------------*/
    if (c_window_proj_to_ls (&ddr, window, units) != E_SUCC)
    {
        c_lasmsg (FALSE, "wndconv-window", "Error converting the window");
        return (E_FAIL);
    }

    return (E_SUCC);
}
