/*******************************************************************************
NAME				C_COMPLT

PURPOSE		Write a standard program completion message and all output 
		file names generated by a program to screen and to session 
		log of TAE.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	Original development for NEWLAS
	K.Gacke		5/86	change m_msg call to errmsg to solve batch log
				problem for FORTRAN programs, compute the 
				amount of CPU time required for the process,
				called xzexit to halt the TAE process.
	K.Gacke         9/86    On unix systems used getrusage to compute the
				CPU time.
	K. Zanter      12/87    1.  switched to raw C types
				2.  changed include file specification
				3.  changed DESC arguments to be
				    character strings
				4.  renamed source from complt to
				    c_complt
				5.  moved bridge routines to separate
				    source file
	B. Ailts	12/87	Changed complt to accept host file names
				instead of TAE file names
	D. Hollaren	04/88   changed include spec to las
				changed cerrmsg to c_errmsg and passed
				the status variable by address
				removed call to squeeze routine
	B. Ailts	05/88   Changed complt back to accepting TAE file names
				This was done because DMS can ouput names that
				are not file names e.i. graphics plane numbers
        P. Pease         6/90   Changed to work on the IBM-RT (aiws); 
                                getrusage does not exist on the IBM-RT.

	D. VanderZee	07/90	Standardized error handling

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	C_COMPLT must be run under TAE.

PROJECT				LAS

ALGORITHM 
	write to log file 
        generate and output program completion string 
	for each output file
		output file name to screen and session log
	compute cpu time required to run the application
	output cpu time to the screen and session log
	exit

ALGORITHM REFERENCES		none
*******************************************************************************/


#include <stdio.h>
#include "las.h"
#include "util.h"

#ifdef aiws
#define UNITS 10
#endif
#ifdef vms
#define UNITS 100   	/* units of 10 milliseconds  */
#else
#include <sys/time.h>
#ifdef _IBMR2
#include <time.h>
#endif
#include <sys/resource.h>
#endif
#ifdef sun
#define UNITS CLK_TCK
#include <sys/times.h>
#include <limits.h>
#endif


#define MAXMESG 69

void c_complt
(
   char  *pname,          /* I:  program name                          */
   char  tnames[][CMLEN], /* I:  output file names                     */
   long  *count           /* I:  number of output files                */
)

	{
	extern int begin_time;
#ifdef sun
	struct tms tbuf;
#else
	struct
	   {
	   int utime;
	   int stime;
	   int child_time;
	   int child_stime;
	   }  tbuf;
#endif

#ifndef sun
    	struct rusage usage;
#endif

	long	i;			/* loop index			  */
	long	nonfatal = 1;		/* Non-fatal status for c_errmsg  */
	int	sec, mn, hr;
	char	message[ERRLEN+1];	/* character string for messsage  */

/* write to log file */
	logit(pname);

/* generate program completion string  */
	sprintf(message,"Program %s completed successfully.",pname);
	c_errmsg(message," ",&nonfatal);

/* if there are any output files print them out  */
	if (*count > 0)
		{
		c_errmsg("Created output:"," ",&nonfatal);

		for (i = 0; i < *count; i++)
			{
			sprintf(message,"   %s",tnames[i]);
			c_errmsg(message," ",&nonfatal);
			}
		}

/* output CPU time */
#ifdef vms
	times(&tbuf);
	sec = (tbuf.utime + tbuf.stime + tbuf.child_time + tbuf.child_stime -
	      begin_time) / UNITS;
#else 
#ifdef aiws
	times(&tbuf);
	sec = (tbuf.utime + tbuf.stime + tbuf.child_time + tbuf.child_stime -
	      begin_time) / UNITS;
#else
#ifdef sun
        times (&tbuf);
        sec = (tbuf.tms_utime + tbuf.tms_stime - begin_time) / UNITS;
#else
	getrusage(RUSAGE_SELF,&usage);
	sec = usage.ru_utime.tv_sec + usage.ru_stime.tv_sec;
#endif
#endif
#endif

	if (sec == 0)
 	   ++sec;
	mn = sec / 60;
	hr = mn / 60;
	sec = sec % 60;
	mn = mn % 60;
	sprintf(message,"Program %s required %02d:%02d:%02d of CPU.",
	        pname,hr,mn,sec);
	c_errmsg(message," ",&nonfatal);

/* exit */
	z_exit(1,pname);
	}
