#include <stdio.h>
#include "tae.h"
#include "las.h"
#include "desc.h"
#include "util.h"

int c_getnum
(
/*
   This function will get TAE variables of integer and real types.
   It is analagous to the TAE FORTRAN callable routines XRINTG and XRREAL.

   It checks to see that the parameter name exists and does error checking
   on the data type.  It does NOT return from these errors.

   It's return value is the number of entries actually read in.
   Return value 0 means the parameter has a "null value"
   and -1 means "no value".
*/
   struct PARBLK *vblock,  /* TAE parameter block                      */
   char parname[],         /* The parameter name                       */
   long *count,            /* Number of entries to read in             */
   long *dtype,            /* NEWLAS data type                         */
   unsigned char buf[]     /* Buffer to read into                      */
)
{
short	*w_buf;
long	*l_buf;
float	*r_buf;
double	*d_buf;

long	actual_count,
	fatal = -1,
	i;
struct	VARIABLE *var;		/* Pointer into variable block                */
char	msg[ERRLEN+1];

/*
   Get the parameter
   -----------------
*/
var = p_find (vblock, parname);
/*
   Check data type
   ---------------
*/
if (((*dtype == EBYTE || *dtype == EWORD || *dtype == ELONG) && var->v_type != V_INTEGER) ||
    ((*dtype == EREAL || *dtype == EDOUBLE) && var->v_type != V_REAL))
   {
   sprintf (msg, "Error: Unexpected parameter type for parameter %s", parname);
   c_errmsg (msg, "getnum-fatal", &fatal);
   }

w_buf = (short  *) buf;
l_buf = (long   *) buf;
r_buf = (float  *) buf;
d_buf = (double *) buf;
/*
   Get the min of count requested and count in vblock
   --------------------------------------------------
*/
actual_count = (*count <= var->v_count) ? *count : var->v_count;

for (i = 0; i < actual_count; i++)
   {
   switch (*dtype)
      {
      case (EBYTE):
         buf[i] = IVAL (*var, i);
         break;
      case (EWORD):
         w_buf[i] = IVAL (*var, i);
         break;
      case (ELONG):
         l_buf[i] = IVAL (*var, i);
         break;
      case (EREAL):
         r_buf[i] = RVAL (*var, i);
         break;
      case (EDOUBLE):
         d_buf[i] = RVAL (*var, i);
         break;
      }
   }
return (actual_count);
}

/*-----------------------------------------------------------*/
#ifdef vms
/*
   This is the VMS FORTRAN bridge
*/
void getnum
(
    struct PARBLK *vblock,
    struct DESC *d_parname,
    long *count,
    long *dtype,
    unsigned char buf[]
)
{
c_getnum (vblock, d_parname->addr, count, dtype, buf);
}
#endif
/*-----------------------------------------------------------*/
#ifdef unix
/*
   This is the Unix FORTRAN bridge
*/
#ifdef _IBMR2
#define getnum_ getnum
#endif
void getnum_ (vblock, parname, count, dtype, buf, parnamelen)

struct PARBLK	*vblock;
char		parname[];
long		*count,
		*dtype;
unsigned char	buf[];
long		parnamelen;
{
c_getnum (vblock, parname, count, dtype, buf);
}
#endif
