#include <stdio.h>
#include <string.h>

#include "las.h"
#include "util.h"

int c_getstr
(
/*
   This function will get TAE variables of type string.
   It is analagous to the TAE FORTRAN callable routine XRSTR.

   It checks to see that the parameter name exists and does error checking
   on the data type.  It does NOT return from these errors.

   It's return value is the number of entries actually read in.
   Return value 0 means the parameter has a "null value"
   and -1 means "no value".
*/
   struct PARBLK *vblock, /* TAE parameter block                           */
   char parname[],        /* The parameter name                            */
   long *count,           /* Number of entries to read in                  */
   long *slen,            /* Dimensioned length of string(s)               */
   char buf[]             /* Buffer to read into                           */
)
{
long	actual_count,
        fatal = -1,      /* fatal error flag                               */
	i;
struct	VARIABLE *var;		/* Pointer into variable block                */
char	msg[ERRLEN+1];

/*
   Get the parameter
   -----------------
*/
var = p_find (vblock, parname);
/*
   Check data type
   ---------------
*/
if (var->v_type != V_STRING)
   {
   sprintf (msg, "Error: Unexpected parameter type for parameter %s", parname);
   c_errmsg (msg, "getstr-fatal", &fatal);
   }

/*
   Get the min of count requested and count in vblock
   --------------------------------------------------
*/
actual_count = (*count <= var->v_count) ? *count : var->v_count;

for (i = 0; i < actual_count; i++)
   {
   strcpy (buf + (i * (*slen)), SVAL (*var, i));
   }
return (actual_count);
}

/*-----------------------------------------------------------*/
#ifdef vms
/*
   This is the VMS FORTRAN bridge
*/

#include "desc.h"

void getstr
(
    struct PARBLK *vblock,
    struct DESC *d_parname,
    long *count,
    long *slen,
    char buf[]
)
{
c_getstr (vblock, d_parname->addr, count, slen, buf);
}
#endif
/*-----------------------------------------------------------*/
#ifdef unix
/*
   This is the Unix FORTRAN bridge
*/
#ifdef _IBMR2
#define getstr_ getstr
#endif
void getstr_ (vblock, parname, count, slen, buf, parnamelen)

struct PARBLK	*vblock;
char		parname[];
long		*count,
		*slen;
char		buf[];
long		parnamelen;
{
c_getstr (vblock, parname, count, slen, buf);
}
#endif
