/******************************************************************************
*    Name:	hsetpr
*
*    Purpose:	Set up boolean flags for the destination of generated output
*		to the terminal, file, or line printer.
*
*    Program history:
*	K. Gacke	8/86	initial development
*  	B. Ailts	12/87	Change include directory specifications
*				Place bridge routines in a seperate file
*				Use raw 'C' types
*	B. Ailts	10/88	Called squeeze for prtfile to be accessable
*				from FORTRAN
*	B. Ailts	11/88   Fixed pointer problem to prtfile
*	B. Ailts	12/88   Renamed to HSETPR -- only returns host name
*				SETPRT will now return LAS name
*	B. Ailts	11/89   Added code to limit names to 14 char for IBM
*				Took out check to always go to LP when batch
*       P.Pease          6/90   Removed declaration of time to fix warning that
*                               it conflicts with a previous declaration
*	D. VanderZee	07/90	Standardized error handling
*
*    Algorithm: 
*	Get the PRINT parameter
*	   and set the term, file, and lp output parameters
*	If the application was run in batch
*	    force the output to the line printer
*
*    Algorithm References		none
*******************************************************************************/
#include <time.h>

#include "las.h"			/* tae include file   		   */
#include "util.h"

int c_hsetpr
(
   struct PARBLK *vblock,	/* TAE parameter block		           */
   long	*term,			/* Boolean for terminal output             */
   long	*file,			/* Boolean for file output                 */
   long  *lp,			/* Boolean for line printer output         */
   char  *prtfile,		/* file name of file or lp working file    */
   char  *filename 		/* file name of file or lp output file     */
)
{
struct VARIABLE *val;
struct tm *tm_ptr;		/* temporary pointer in time struct.       */
long  my_time;
long  index;			/* counter -- each value of PRINT	   */
long  nonfatal = 1;		/* nonfatal error status		   */
long  status;			/* error status				   */
/*long  time();*/

char *prnt;			/* temporary pointer to values of PRINT    */
char *file_ptr;			/* temporary pointer to prtfile argument   */
char *temp_ptr;			/* temporary pointer to prtfile argument   */

/* Initalize the flags 
----------------------*/
*term = *file = *lp = 0;

/* Retreive the parameter and set the flags
-------------------------------------------*/
val = p_find(vblock,"PRINT");
for (index = 0; index < val->v_count; index++)
   {
   prnt = SVAL(*val,index);
   if ((strcmp(prnt,"TERM") == 0) || (strcmp(prnt,"term") == 0))
      *term = 1;
   else if ((strcmp(prnt,"LP") == 0) || (strcmp(prnt,"lp") == 0))
      *lp = 1;
   else if (strlen(prnt) == 0)
      *file = 0;
   else
      {
      *file = 1;
      strcpy(filename,prnt);
      }
   }

/* Force the output to the line printer when the application was batched 
------------------------------------------------------------------------*/
val = p_find(vblock,"$RUNTYPE");
if ((strcmp(SVAL(*val,0),"BATCH") == 0) && (*term == 1))
   {
   *term = 0;
   *lp = 1;
   }

my_time = time(0);
tm_ptr = localtime(&my_time);

/* Calculate and format the time 
--------------------------------*/
file_ptr = squeeze(prtfile,CMLEN);
temp_ptr = file_ptr;
file_ptr += strlen(file_ptr);
strcpy(file_ptr,"_");
file_ptr ++;
sprintf (file_ptr,"%2d",tm_ptr->tm_hour);

if (tm_ptr->tm_hour < 10)
    *file_ptr = '0';
file_ptr += 2;
sprintf (file_ptr,"%2d",tm_ptr->tm_min);

if (tm_ptr->tm_min < 10)
    *file_ptr = '0';
file_ptr += 2;
sprintf (file_ptr,"%2d",tm_ptr->tm_sec);

if (tm_ptr->tm_sec < 10)
    *file_ptr = '0';
file_ptr += 2;
strcpy(file_ptr,".pr");
#ifdef aiws
if (strlen(temp_ptr) > 14)
   {
   file_ptr -= 7;
   sprintf (file_ptr,"%1d%2d%s",tm_ptr->tm_min,tm_ptr->tm_sec,".pr");
   }
#endif
strcpy(prtfile,temp_ptr);
free(temp_ptr);

/*  output the name to the local vblock variable prtfile
--------------------------------------------------------*/
status = q_string(vblock,"PRTFILE",1,&prtfile,P_UPDATE);
if (status != SUCCESS)
    {
    c_errmsg("Error sending PRTFILE to PARBLK","hsetpr-prtfile",
	     &nonfatal);
    return(E_FAIL);
    }

status = q_out(vblock);
if (status != SUCCESS)
    {
    c_errmsg("Error writing to PARBLK","hsetpr-parblk", &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}  /*  hsetpr  */
