/******************************************************************************
NAME:				C_INLAS

FUNCTION:
	C_INLAS is a support routine which initializes TAE and the LAS
	program.  It is used so that all programs (FORTRAN and C) use the
	C TAE I/O routines.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         5/86       K. Gacke     initial development
    2.0        11/87       K. Gacke  	change to use C TAE I/O (instead
					of FORTRAN) 
    3.0        12/87 	   K. Zanter    1.  switched to raw C types
				        2.  changed include file specification
					3.  renamed source from inlas to 
					    c_inlas
					4.  moved bridge routines to separate
					    source file
    3.1        12/91       D. Etrheim	Check for ASYNC runtype.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
   C_INLAS must be run under TAE.

PROJECT:	LAS

ALGORITHM:
	TAE is initialized by calling the "z_init".  It also
	saves the beginning execution time in the global "begin_time". 

******************************************************************************/

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "las.h"
#include "util.h"

#ifdef vms
#define STD_OUT 6
#else
#define STD_OUT 8
#endif

int begin_time;

void c_inlas
(
   struct PARBLK *vblock,       /* O:  TAE parameter block                 */
   long *sout                   /* O:  unit number for opening stdout      */
)
{
struct 
   {
   int utime;
   int stime;
   int child_utime;
   int child_stime;
   } tbuf;

struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */
long zero = 0;
char *runtype;

*sout = STD_OUT;			/* initialize standard out */

z_init(vblock,P_ABORT);		/* initialize TAE */

#ifdef vms				/* initialize time */
   times(&tbuf);
   begin_time = tbuf.utime + tbuf.stime + tbuf.child_utime + tbuf.child_stime;
#else
   begin_time = 0;
#endif

val = p_find(vblock,"$RUNTYPE");		/* if running in batch then  */
runtype = SVAL(*val,0);				/*   reset $MSGTIM to zero   */
if ((strcmp(runtype,"BATCH") == 0) || (strcmp(runtype,"ASYNC") == 0))
   {
   q_intg(vblock,"$MSGTIM",1,&zero,P_UPDATE);
   q_out(vblock);
   }
}
