#include "las.h"
#include "util.h"

/*******************************************************************************
NAME:		C_JULGRE

PURPOSE:	Convert a Julian date into a Gregorian date

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/

static long days1[13] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
static long days2[13] = {0,31,60,91,121,152,182,213,244,274,305,335,366};

void c_julgre
(
   long *year,          /* I:  Year                                     */
   long *jdate,         /* I:  Julian date                              */
   long *month,         /* O:  Gregorian month                          */
   long *day            /* O:  Gregorian day                            */
)
{
long	index;		/* Index into the days arrays	*/

/*
  Calculate the gregorian date
*/
if (c_isleap(year))
    {
    /*
      Find the gregorian date in a leap year
    */
    for (index = 1; index < 13; index++)
	{
	if (*jdate <= days2[index])
	    {
	    *day = *jdate - days2[index-1];
	    *month = index;
	    break;
	    }
	}
    }
else
    {
    /*
      Find the gregorian date in a non-leap year
    */
    for (index = 1; index < 13; index++)
	{
	if (*jdate <= days1[index])
	    {
	    *day = *jdate - days1[index-1];
	    *month = index;
	    break;
	    }
	}
    }
}



/*******************************************************************************
NAME:		C_GREJUL

PURPOSE:	Convert a Gregorian date to a Julian date.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
void c_grejul
(
   long *year,          /* I:  Year                                     */
   long *month,         /* I:  Gregorian month                          */
   long *day,           /* I:  Gregorian day                            */
   long *jdate          /* O:  Julian date                              */
)
{

/*
  Calculate the julian date
*/
if (c_isleap(year))
    *jdate = days2[*month-1] + *day;		/* Leap year		*/
else
    *jdate = days1[*month-1] + *day;		/* Non-leap year	*/
}
