/*******************************************************************************
NAME:    C_PUTPARM

PURPOSE: Write a TAE parameter to a project parameter file

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    D.Etrheim        11/94  EDC   Use c_lsmknm to get file name

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PUTPARM must run under TAE

PROJECT: LAS

*******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"


int c_putparm
(
   char *filename,         /* I: Hostname of the project parameter file */
   char *parmname,         /* I: Parameter name                         */
   char string_value[][CMLEN], /* I: String values of the parameter     */
   char *type,             /* I: Type of parameter to write (I, R, S)   */
   double double_value[],  /* I: The double values of the parameter     */
   long dimension,         /* I: Dimension of the parameter             */
   long long_value[]       /* I: The long values of the parameter       */
)

{
char tmp_filename[CMLEN];     /* Actual file name with the .ppf extension     */
long i;                       /* Loop control                                 */
long j;                       /* Loop control                                 */
long nonfatal =  1;           /* Nonfatal error flag                          */

FILE *parm_file;              /* File pointer to the project parameter file   */


/* Make sure there is a positive dimension
------------------------------------------*/
if (dimension <= 0)
   return(E_SUCC);

/* Add the .ppf extension to filename
-------------------------------------*/
c_lsmknm(filename,"ppf",tmp_filename);      

/* Open the project parameter file
----------------------------------*/
if ((parm_file = fopen(tmp_filename, "a")) == NULL)
   {
   c_errmsg("Error opening project parameter file","c_putparm-open",&nonfatal);
   return(E_FAIL);
   }

/* Convert the parameter name and type to uppercase
---------------------------------------------------*/
i = (long)strlen(parmname);
c_low2up(parmname, &i);
i = (long)strlen(type);
c_low2up(type, &i);

/* Write the parameter to disk
------------------------------*/
if (strcmp(type, "I") == 0)
   {
   fprintf(parm_file, "%-20s I %d", parmname, dimension);
   for (i = 0; i < dimension; i++)
      fprintf(parm_file," %d", long_value[i]);
   fprintf(parm_file, "\n");
   }
else if (strcmp(type, "R") == 0)
   {
   fprintf(parm_file, "%-20s R %d", parmname, dimension);
   for (i = 0; i < dimension; i++)
      fprintf(parm_file," %lf", double_value[i]);
   fprintf(parm_file, "\n");
   }
else if (strcmp(type, "S") == 0)
   {
   fprintf(parm_file, "%-20s S %d", parmname, dimension);
   for (i = 0; i < dimension; i++)
      {
      for (j = 0; j < strlen(string_value[i]); j++)
	 if (isspace(string_value[i][j]))
	    {
	    c_errmsg("Strings cannot contain white space",
		     "c_putparm-space", &nonfatal);
	    return(E_FAIL);
	    }
      fprintf(parm_file," %s", string_value[i]);
      }
   fprintf(parm_file, "\n");
   }
else
   {
   fclose(parm_file);
   c_errmsg("Invalid parameter type", "c_putparm-type", &nonfatal);
   return(E_FAIL);
   }

fclose(parm_file);
return(E_SUCC);
}
