
/*******************************************************************************
NAME:     errclean() 
PURPOSE:  errclean is called in the event of a fatal error.  It closes any open
	  images;  if $delflg="yes" output images are deleted;  a fatal error
	  message is issued.  This function can be called from anywhere in an
	  application after out_flags().  This function accepts a variable
	  number of input arguments much like c_lasmsg does.  Usage:  errclean
	  (messagekey, sprintf format, arguments to format).
*******************************************************************************/
#include <varargs.h>   /* for variable number of arguments;  stdarg.h would be
                          better but is unavailable on sun OS */ 
#include <stdlib.h>    /* for call to access() */ 
#include "cwheel.h"    /* application header file */ 



/* vars common to errclean() and main(), declared common to prevent excessive 
   parameter passing */ 
   extern struct OUTIMAGE outimage;  /* info on output las image */ 
   extern struct ANPROC   anproc;    /* proc to run annotate */ 



FUNCTION void errclean (va_alist) 
   va_dcl
{
/* declare arguments passed to this function */ 
   char *format;      /* sprintf format string for all additional arguments */ 
   va_list argumentp; /* lists a variable number of function arguments */ 

/* declare variables */ 
   long  one=1;           /* number of output file names in outimage.hostname */
   char  message[3*ERRLEN], /* text of message printed by c_lasmsg() */
	*mkey;              /* message key, parameter to c_lasmsg() */ 

/* get input variables and format message string */ 
   va_start (argumentp);  
   mkey   = va_arg (argumentp, char *);
   format = va_arg (argumentp, char *);
   vsprintf (message, format, argumentp); 
   va_end (argumentp);

/* close any open image files and free buffers */
   if (outimage.opened && !outimage.closed)  
      close_outimage(&outimage); 

/* optionally delete output image */ 
   if (outimage.opened && outimage.delflg) 
      c_delfil (outimage.hostname[0]); 

/* optionally delete anproc if it exists */ 
   if (outimage.delflg && !access(anproc.hostname,0) )
      c_delfil (anproc.hostname); 

/* issue fatal error and terminate processing */ 
   c_lasmsg (TRUE, mkey, message); 
}


