/****************************************************************************
   NAME:                          EDGE

   FUNCTION:
      EDGE does edge enhancement on the input 2D array.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C
****************************************************************************/
#include "picmatch.h"

void edge (
  float *a,     /* I/O: 2D array of size N x N */
  int n)        /* I: Size of the array a      */
{
  float **b = NULL;
  int fac[3] = {13, 10, 8},
      np1,                     /* n + 1                       */
      np2,                     /* n + 2                       */
      i, j, k, l,              /* looping variables           */
      iq,                      /* index for 2nd b array value */
      liq,                     /* last iq value               */
      ic,                      /* current i value minus 1     */
      b1, b2,
      jm1,                     /* current j value minus 1     */
      sum,                     /* sum of the b values         */
      kint;
  long one = 1,                /* constant one value          */
       fatal = -1;             /* constant fatal value        */

  /* Initialize np1 and np2 from n.
   */
  np1 = n + 1;
  np2 = n + 2;

  /* Allocate space for b to be b[np2][3].
   */
  b = (float **) malloc (np2 * sizeof (float *));
  if (b == NULL)
  {
    c_errmsg ("Error allocating memory", "picmatch-alloc", &one);
    c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  for (i = 0; i < np2; i++)
  {
    b[i] = (float *) malloc (EREALSIZE * 3);
    if (b[i] == NULL)
    {
      c_errmsg ("Error allocating memory", "picmatch-alloc", &one);
      c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
    }
  }

  /* Initialize B to 0.0
   */
  for (j = 0; j < np2; j++)
  {
    for (i = 0; i < 3; i++)
    {
      b[j][i] = 0.0;
    }
  }

  iq = 3;
  for (i = 0; i < np1; i++)
  {
    liq = iq;
    iq = (iq % 3) + 1;
    if (i != n)
    {
      for (j = 0; j < n; j++)
        b[j+1][iq-1] = a[j*n + i];
    }
    else
    {
      for (j = 1; j < np1; j++)
        b[j][iq-1] = 0.0;
    }

    ic = i - 1;
    if (ic < 0)
      continue;

    b1 = 0;
    if (ic == 0 || ic == n - 1)
      b1 = 1;

    for (j = 0; j < n; j++)
    {
      b2 = b1;
      if (j == 0 || j == n - 1)
        b2++;

      /* Initialize jm1 to j-1 and the sum to 0.
       */
      jm1 = j - 1;
      sum = 0;

      for (k = 0; k < 3; k++)
        for (l = 0; l < 3; l++)
          sum += b[jm1 + l + 1][k];

      /* Calculate the intersection.  If it is less than zero then set it
         to zero.
       */
      kint = ((b[j+1][liq-1] * fac[b2] - sum) / 4.0 > 0.0) ?
              (b[j+1][liq-1] * fac[b2] - sum) / 4.0 : 0.0;
      a[j*n + ic] = kint;
    }  /* for j */
  }  /* for i */

  /* Free b pointers
   */
  for (i = 0; i < np2; i++)
  {
    free (b[i]);
    b[i] = NULL;
  }

  free (b);
}
