#ifndef PMASK_H
#define PMASK_H

#include "las.h"
#include "util.h"     /* c_inlas, c_getin, c_getout proto */
#include "worgen.h"

void edge (
  float *a,         /* I/O: 2D array of size N x N */
  int n);           /* I: Size of the array a      */

void getpar (
  struct PARBLK *vblock,  /* O: TAE parameter block                        */
  float rmag[],           /* O: Magnification factor                       */
  int  *search,           /* O: Initial search area size                   */
  int  *msrc,             /* O: Minimum search area size                   */
  float *rmcor,           /* O: Minimum correlation value                  */
  int  *getw,             /* O: Kernal size                                */
  int  *getr,             /* O: Threshold value                            */
  int  *nohpf,            /* O: High-pass filter flag                      */
  int  *subpix,           /* O: Subpixel correlation flag                  */
  int  *doedge,           /* O: Edge enhancement flag                      */
  int  *ptinc,            /* O: Point increment or step size               */
  int  *retry,            /* O: Retry flag                                 */
  int  *nretry,           /* O: Number of retries (up to 4)                */
  float *tretry,          /* O: Correlation threshold                      */
  float *rretry,          /* O: Distance to move in the line/sample value
                                direction from the current location        */
  char hofile[][CMLEN],   /* O: First array value is output IBIS graphics
                                file name                                  */
  char tae2[],            /* O: TAE input filename (IN1)                   */
  char host2[],           /* O: Host input filename (IN1)                  */
  long window2[4],        /* O: Window of the input file (IN1)             */
  long *bands2,           /* O: Band in input filename (IN1)               */
  char tae3[],            /* O: TAE input filename (IN2)                   */
  char host3[],           /* O: Host input filename (IN2)                  */
  long window3[4],        /* O: Window of the input file (IN2)             */
  long *bands3,           /* O: Band in input filename (IN2)               */
  char host4[],           /* O: Host cutline filename                      */
  int  *contur,           /* O: Flag to specify if a cutline has been
                                specified                                  */
  float gridsp[4],        /* O: Grid display specifications (xl,xs,yl,ys)  */
  int  *nah,              /* O: Number of columns in the grid              */
  int  *nav,              /* O: Number of rows in the grid                 */
  float ritie[],          /* O: Tie points for first input image           */
  float rotie[],          /* O: Tie points for second input image          */
  int  *count);           /* O: Number of tie points divided by 2          */

void getzvl (
  float *a,         /* I: 2D array of size N x N                             */
  int n,            /* I: Size of the array a                                */
  float fl,         /* I: Fractional line value of the grid's center point   */
  float fs,         /* I: Fractional sample value of the grid's center point */
  int nw,           /* I: Kernal size.  The size of the square kernel
                          grid used to calculate the average radiometric
                          value (Z value) for the matching coordinates.      */
  int nr,           /* I: Threshold value.  The number of pixel values in
                          the kernel equal to zero that are accepted before
                          the point is rejected.                             */
  float *z);        /* O: Z value (average radiometric value)                */

void mygrid (
  struct FDESC *ifcb,
                  /* I: File descriptor of the input grid                */
  float *grid,    /* O: 2D output array of interpolated DN values        */
  int  ndim,      /* I: Dimension of the grid (max of 128), must be even */
  int  n,         /* I: Defines the portion of the grid to be processed,
                        must be even                                     */
  int  line,      /* I: Number of lines in the image                     */
  int  samp,      /* I: Number of samples in the image                   */
  float centr[2], /* I: Grid center, [0] = y, [1] = x (one-based)        */
  float *trans,   /* I: Array of the grid transformation coefficients:
                        ynew = trans[0]*yold + trans[1]*xold + trans[2]
                        xnew = trans[3]*yold + trans[4]*xold + trans[5]  */
  float shift[],  /* O: Vector to add to the transcalfactormed grid to
                        bring its center to "centr", [0] = y, [1] = x    */
  float magnif[], /* I: Array containing magnification of X and Y
                        distances in the grid, [0] = y, [1] = x          */
  int  *chop,     /* O: Number of grid points outside the image.
                        Corresponding DN values in the "grid" are set
                        to zero.                                         */
  long band,      /* I: Band number to be processed                      */
  int  *status);  /* O: Return status                                    */

void myread (
  float *bufin,      /* O: 2D image line return buffer                  */
  struct FDESC *fdesc,
                     /* I: Image file control block                     */
  long *record,      /* I/O: Record number to read, if 0 then read the
                             next record (one-based)                    */
  int skpbyt,        /* I: Number of bytes to skip before reading       */
  int nbytes,        /* I: Total number of bytes to read                */
  int ns,            /* I: Number of samples in the line                */
  int ilow,          /* I: Row to be read (zero-based)                  */
  long band,         /* I: Band to be read                              */
  int *status);      /* O: Return status
                           0 = success
                           4 = end of file
                           8 = read error                               */

int refine (
  float corr[3][3],    /* I:   2D input matrix  */
  float *vloff,        /* I/O: Line offset      */
  float *vsoff);       /* I/O: Sample offset    */

void rfit (
  int il,                /* I: Input start line value in search window    */
  int js,                /* I: Input start sample value in search window  */
  float *vmax,           /* O: Maximum value encountered in search window */
  float *vloff,          /* O: Line offset                                */
  float *vsoff,          /* O: Sample offset                              */
  float corr[3][3],      /* O: 2D correlation array for asrch             */
  int nohpf,             /* I: Flag for no hpf                            */
  int oldfft,            /* I: Flag to specify the old FFT or not         */
  float chip1[32][32],   /* I: 32x32 chip to fit                          */
  float asrch[128][128], /* I: 128x128 search window                      */
  int *status);          /* O: Status flag                                */

#endif
