/******************************************************************************
NAME:   			PROCES 

PURPOSE:    

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "typlim.h"
#include "exbrows.h"
#include "protos.h"

int procesx
(
    char *hosin,			/* host file names of input images      */
    char *ptype,			/* product type                         */
    struct LINE *brw_line,		/* header record for output file     	*/
    char *tapeid			/* tapeid				*/
)
{
struct FDESC *fdesc[2];			/* image file descriptors used for image I/O */
unsigned char *thebuf;		/* image I/O buffer                          */
long	nsamps;			/* number of samples			*/
long	nlines;			/* number of lines			*/

/*
  Misc allocations
*/
long	k;			/* Looping variable			*/
long    loop;			/* loop counter				*/
long    band;			/* band loop counter			*/
long    len;			/* temporary length variable		*/
long	status;			/* function return status code          */
long	nonfatal = 1;		/* Non-fatal error flag			*/
long	bd;			/* Browse band number			*/
long	use_flag;		/* scene use flag			*/
struct stat fstat;		/* file status				*/
long    nbands;			/* number of bands			*/

/*
  Database file information allocations
*/
FILE	*fp1;			/* file pointer	into the database file  */
char	infiles[3][CMLEN];	/* input file names			*/
char	outfile[CMLEN];		/* specific output name			*/
char	outname[CMLEN];	/* output name				*/
char	scene[61][17];		/* sene id				*/
char	command[CMLEN];		/* system command 			*/

long    sllist[61];		/* starting line list			*/
long	position;		/* position in database file		*/
long	rectec;                 /* Recording technique                  */
long	satnum;                 /* Satellite number                     */

double	center[2];              /* Center latitude/longitude (degrees)  */

/*
  Image I/O data allocations
*/
struct  DDR ddr;		/* DDR structure			*/
struct  BDDR bddr;		/* BDDR structure			*/

long	access;			/* image I/O access type                */
long    opt[3]; 		/* option array for image I/O		*/
long    one=1;			/* value of one				*/
long    imgsiz;			/* image size 				*/
long    size;			/* image size 				*/
long    count;			/* histogram count value		*/

char    wbuf[10];		/* working buffer 			*/
char    syst[12];		/* system				*/
char 	disk[CMLEN];		/* temporary directory name		*/
char 	badraw[CMLEN];		/* temporary directory name		*/
char	msg[CMLEN];		/* error message			*/

unsigned char *ptr;		/* Place holder for image io address	 */
char  direc[CMLEN],root[CMLEN],ext[CMLEN];
long cc,index;
long two55 = 255;


/*
  Histogram information
*/
long    histo[256];		/* histogram array			*/
long	mean;			/* average value			*/
long	minval;			/* minimum value			*/
long	maxval;			/* maximum value			*/
long	pnt0;			/* Value at  0.05 percent of image data */
long	pnt1;			/* Value at  2.5  percent of image data	*/
long	pnt2;			/* Value at 97.5  percent of image data	*/
long	zero = 0;		/* Pixel fill value of zero		*/
long	nfill = 256;		/* Number of pixels to be filled	*/
long	dtype = ELONG;		/* Data type of the fill data		*/
long    dummy = 0;		/* Dummy variable for pxhist call	*/
long i;

double	pc;			/* percentage value			*/
double	cum;			/* cummulative percentage		*/
double	sx;			/* cummulative sum			*/


/* Get the archive directory
--------------------------*/
strcpy(disk,getenv("LBINT"));

nlines = 390;
nsamps = 432;

/*Create the DDR for the browse set 
---------------------------------*/
c_intddr(&ddr);
c_intbdr(&bddr);


ddr.nl = nlines;
ddr.ns = nsamps;
ddr.nbands = 1;
ddr.dtype = EBYTE;
bddr.bandno = 1;
if (c_getsys(syst) != E_SUCC)
    c_errmsg("Error returned from getsys", "eopenr-call",&nonfatal);
else
    strcpy(ddr.system,syst);


/*
  Set up file names. hosin is the name of band 1. Create the other
  file names from this.
*/

len = (long) strlen(hosin);
hosin[len-1] = '\0';

c_splitname(hosin,direc,root,ext);

strcpy(infiles[0],direc);
strcat(infiles[0],root);
strcat(infiles[0],"1");
strcpy(infiles[1],direc);
strcat(infiles[1],root);
strcat(infiles[1],"2");
strcpy(infiles[2],direc);
strcat(infiles[2],root);
strcat(infiles[2],"4");

strcpy(scene[0],root);

opt[0] = opt[1] = opt[2] = 0;

nbands=3;
fp1 = NULL;

/* Proces only one scene at a time for MSSX 
  (Should probably take this out of here,
   but it ain't worth the effort at this time)
-------------------*/
for (loop = 0; loop < 1; loop++)
    {
    /*
      Check to be sure this is the scene to process
    */
    sllist[loop] = 0;
    getname(scene[loop],tapeid,sllist[loop],&use_flag,&position,ptype,
	    &fp1,zero,&rectec,&satnum,center);

    /*
      Found a browse image to extract
    */
    if (use_flag == TRUE)
	{

	/*
	  Allocate space for the entire browse image band
	*/
	imgsiz = nsamps * nlines;
	thebuf = (unsigned char *) malloc(imgsiz);

	/*
	  Save the address of the image io buffer.  This is needed later
          because the jpeg compression function manipulates the image i/o
          buffer to get the job done.
	*/
	ptr = thebuf;

	/*
	  Process each of the bands for this scene
	*/
	for (band = 0; band < nbands; band++)
	    {
            /* Create a DDR for this file */
            if (c_putddr(infiles[band],&ddr) != E_SUCC)
               {
               c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);
               return(E_FAIL);
               }

            if (c_putbdr(infiles[band],&bddr) != E_SUCC)
               {
               c_errmsg("Error creating DDR","exbrows-ddr",&nonfatal);
               return(E_FAIL);
               }

	    /*
	      Open the input file
	    */
            access = IREAD;
            status = c_eopenr(&fdesc[0],infiles[band],&one,&one,
			      &one,&nlines,&nsamps,&access,&ddr.dtype,opt,
			      &one);
            if (status != E_SUCC)
		return(E_FAIL);

	    /*
	      Append the browse band number to the output file name
	    */
	    len = (long) strlen(infiles[band]);
	    bd = atol(&infiles[band][len-1]);
	    sprintf(outfile,"%s/LM%s",disk,root);
	    sprintf(wbuf,"_%d",bd);
	    sprintf(outname,"LM%s",root);
   
	    /*
	      Create the output file name
	    */
	    if ((scene[loop][0] == '1') || (scene[loop][0] == '2') || 
		(scene[loop][0] == '3'))
		{
                if (wbuf[1] == '1')
		    strcat(outfile,"_4");
                else if (wbuf[1] == '2')
                    strcat(outfile,"_5");
                else if (wbuf[1] == '4')
                    strcat(outfile,"_7");
                }
	    else
	        strncat(outfile,wbuf,2);
       
	    /*
	      Initailize the histogram information
	    */
            c_pxfill(histo,&dtype,&nfill,&zero);
   
            /* Find the last pixel not equal to zero (fill data) */
            if (band == 0)
               {
               if (c_eread(&fdesc[0],&one,&one,thebuf,&one) != nsamps)
		           return(E_FAIL);
               c_pxindr(thebuf,&zero,&nsamps,&index,&ddr.dtype);
               index++; /* because we are zero relative */

               /* If this happens, check to see if the fill data
                  was 255.  For some reason some (one?) browse scene 
                  had fill data of 255.  This one scene has bad data in 
                  the original (BIP2) scene. */
               if (index == 432)
                  {
                  c_pxindr(thebuf,&two55,&nsamps,&index,&ddr.dtype);
                  index++;
                  }

	       /* This is a saftey check in case the first image line is
	       all zero.  The raw image varies from 3000 to 3450 samps;
	       the browse should vary from 375 to 431. 
               7-22-96 We have found instances where this is occurring.
                       Data Management wants to do QA on each of these
                       scenes.  We will copy these to a temp directory
                       and continue processing the remaining scenes.*/ 
               if (index < 375)
	              { 
                  c_errmsg("Invalid sample count","exbrows-samps",&nonfatal);

                  /* Close the input file */
	              c_eclose(&fdesc[0]);

                  /* Create a directory where these scenes will be copied*/
                  strcpy(badraw,getenv("LBRAW"));
                  strcat(badraw,"/bad_scenes");

                  /* Copy the files.  */
                  for (i = 0; i < 3; i++)   
                      {
                      sprintf(command,"cp %s %s",infiles[i],badraw); 
                      system(command);
                      }
                  
                  /* Dont forget band 3 */
                  len = (long) strlen(infiles[0]);
                  infiles[0][len-1] = '3';
                  sprintf(command,"cp %s %s",infiles[0],badraw); 
                  system(command);

                  /* Continue on with next scene.*/
                  return (E_SUCC);
                  }

               sprintf(msg,"Number of samples per image line: %d",index);
               c_errmsg(msg,"exbrows-samps",&nonfatal);
               }



            /* read entire image into memory, one line at a time.
               reset the pointer to ignore fill data */
            for (cc = 1; cc <= nlines; cc++)
               {
               if (c_eread(&fdesc[0],&one,&cc,thebuf,&one) != nsamps)
		           return(E_FAIL);
               thebuf+=index;
               }

            /* Reset to the begining of the buffer */
	        thebuf = ptr;

            /* histogram input image
            ----------------------*/
            size = index * nlines;
            c_pxhist(&size,thebuf,&ddr.dtype,histo,&dummy);
            count = size;

            /* zero out 255 even though it is MSS. For some reason
               some (one?) browse scene had fill data of 255.  This
               one scene has bad data in the original (BIP2) scene. */
            count -= histo[0];
            count -= histo[127];
            count -= histo[255];
            histo[0] = histo[127] = histo[255] = 0;

            minval = MAXUTINY;
            maxval = MINUTINY;
            cum = 0.0;
            sx = 0.0;
            pnt0 = pnt1 = pnt2 = 0;

            for (k = 0; k <= 255; k++)
		        {
                if (histo[k] > 0)
		            {
                    pc = 100.0 * histo[k]/count;
                    cum += pc;

                    if (cum <= 0.05)
                        pnt0 = k;
                    else if (cum <= 2.5)
                        pnt1 = k;
                    else if (cum <= 97.5)
                        pnt2 = k;
                      
                    if (k < minval)
                        minval = k;

                    if (k > maxval)
                        maxval = k;
      
		            sx += (k*histo[k]);
		            }
		        }
   
	        /* If this band is zero filled do not process
            -------------------------------------------*/
            if (count != 0)
		        {
		        mean = (long)sx/count;

		        /* Setup header
		        -------------*/
		        setup_head(brw_line,bd,nlines,index,outname,center,
			               rectec,satnum,minval,maxval,pnt0,pnt1,pnt2,mean);
   
		        /* Write JPEG image
		        -----------------*/
		        write_JPEG_file(thebuf,outfile,80,brw_line,nlines,index,1);

                /* Check file size as a safegaurd.  Some files have
                   been processed with the output file size being zero,
 		           possibly becuase /usr/tmp was full.  
                --------------------------------------------------------*/
                if (stat(outfile,&fstat) != 0)
                   {
                   c_errmsg("File status error","exbrows-stat",&nonfatal);
                   return (E_FAIL);
                   }

                if (fstat.st_size == 0)
                   {
                   c_errmsg("Invalid file size","exbrows-zero",&nonfatal);
                   return (E_FAIL);
                   }
		       }
	           else
		       {
		       sprintf(msg,"Zero data in band %d of %s: Not Processed\n",
			       bd,scene[loop]);
		       c_errmsg(msg,"exbrows-zero",&nonfatal);
		       }

	      /* Close input image
	       ------------------*/
	       c_eclose(&fdesc[0]);

	      /*
	        Reset the image io buffer address.  This gets changed in
	        the call to "write_JPEG_file" and it dosen't hurt if it
	        wasn't called.
	      */
	      thebuf = ptr;
	      }

	  /*
	    Update the database record to indicate the record was processed
	  */
	  if (updb(fp1,position,scene[loop],tapeid) != E_SUCC)
	      return(E_FAIL);

	  /*
	    Free the image I/O buffer space allocated
	  */
	  free(thebuf);
   	  }
    }

/*
  Close the database file
*/
if (fp1 != NULL)
    fclose(fp1);

return(E_SUCC);
}
